/* $Id: ComponentManagerImpl.java,v 1.4 2004/05/20 14:23:50 bures Exp $ */
package SOFA.Component;

import java.lang.reflect.Method;

import SOFA.Connector.Reference;
import SOFA.Connector.RoleBase;
import SOFA.SOFAnode.Run.Deployment.DeploymentDescriptor;

/** Implementation of component manager.
  *
  * @author Petr Hnetynka
  */
public abstract class ComponentManagerImpl implements ComponentManager, org.objectweb.fractal.api.Type {
  
  /** roles of connectors of provides interfaces */
  protected NamedElementRegister provides;
  /** roles of connectors to required interfaces */
  protected NamedElementRegister requires;
  /** sRoles of Requirements, which will be subsumed to subcomponents */
  protected NamedElementRegister subsumableRequires;
  /** cRoles of provisions, that are connect by delegation to subcomponent's provisions */
  protected NamedElementRegister delegatedProvides;
  /** roles of connectors to subcomponent CMs */
  protected NamedElementRegister subcomponents;
  /** deployment descriptor */
  protected DeploymentDescriptor dd;
  /** thread registry */
  protected ThreadIDRegistry thrRegistry;
  

  /** Name of the SOFAnode */
  protected String node;
  /** Name of the home deployment dock */
  protected String dockName;
//  /** Base classloader - loading of the implementation */
//  protected ClassLoader bcl;
//  /** Interface classloader - loading of the interfaces and connectors */
//  protected ClassLoader icl;
  /** Implemenatation version */
  protected String releaseVersion;
  /** Home local dock */
  protected SOFA.SOFAnode.Run.Deployment.DeplDock localdock;

  /** Full name of the component (with version) */
  protected String fullName;

  /** Runtime id. Used for connector. */
  protected String id;
  
  /** Constructor */
  public ComponentManagerImpl(DeploymentDescriptor _dd,  String _node, String _dockName, SOFA.SOFAnode.Run.Deployment.DeplDock _localdock, String _id) {
    dd = _dd;
    localdock = _localdock;
    releaseVersion = dd.getReleaseVersion();
    node = _node;
    dockName = _dockName;
    id = _id;

    try {
      thisRB = SOFA.Connector.Boot.DCUPComponentManagerConnector.createSrv(this, id);
    } catch (SOFA.Connector.ConnectorException e) {
      throw new ComponentLifecycleException("ConnectorException: "+e.getMessage(), e);
    }
  //  icl = SOFA.SOFAnode.Run.SOFAInterfaceClassLoader.getInterfaceClassLoader(dd.getCDLEntities());
  //  bcl = SOFA.SOFAnode.Run.SOFABaseClassLoader.getBaseClassLoader(icl, dd.getArchitectureAbsName() , releaseVersion);
    provides = new NamedElementRegister();
    requires = new NamedElementRegister();
    subsumableRequires = new NamedElementRegister();
    delegatedProvides = new NamedElementRegister();
    subcomponents = new NamedElementRegister();
    thrRegistry = new ThreadIDRegistryImpl(_node);

    fullName = dd.getArchitectureAbsName()+"["+dd.getReleaseVersion()+"]";
  }

  public void registerSubcomponent(String name, SOFA.Connector.Reference ref) throws NamingException, InstantiationException {
    try {
      RoleBase target = SOFA.Connector.Boot.DCUPComponentManagerConnector.createClt(ref);
      subcomponents.registerElementNamed(name, target);
    } catch (SOFA.Connector.ConnectorException e) {
      throw new InstantiationException("ConnectorException: "+e.getMessage());
    }
  }
  
  public void unregisterSubcomponent(String name) throws NamingException {
    subcomponents.unregisterElementNamed(name);
  }

  /** Return connector role to the provided interface.
    * @param name    name of the provision
    * @exception   NamingException   no provision with given name
    * @return      connector role to the provided interface
    */
  public RoleBase getProvision(String name) throws NamingException {
    return (RoleBase) provides.getElementNamed(name);
  }
  
  public Reference getProvisionReference(String name) throws NamingException {
    return ((RoleBase) provides.getElementNamed(name)).getSOFAReference();
  }

  public void setRequirement(String name, Reference ref) throws InstantiationException, NamingException {
    RoleBase cRole = instantiateConnector(name);
    try {
      cRole.link(ref);
    } catch (SOFA.Connector.LinkException e) {
      throw new InstantiationException("LinkException: "+e.getMessage(), e );
    }
    requires.forceRegisterElementNamed(name, cRole);
  }

  /** Set implementation object to provision, object must implement provided interface.
    * It is call from primitive components only.
    * @param name   name of the provision
    * @param prov   implementation object
    * @exception NamingException         no provision with given name
    * @exception InstantiationException  object cannot be set
    */
  public void setProvisionImpl(String name, Object prov) throws NamingException, InstantiationException {
    try {
    /*  SOFA.Connector.DeploymentDescriptor connDD = dd.getConnectorDeploymentDescriptor(name);
      String type = connDD.getType();
      RoleBase ret = SOFA.Connector.ConnectorBuilder.createConnector(type, connDD, cl, node, dockName);
      ret.setThrIDRegistry(thrRegistry);
      ret.setRegister(this); */
      RoleBase ret = instantiateConnector(name);
      ret.link(prov);
//      try {
//        provides.unregisterElementNamed(name);
//      } catch (NamingException e) {}
      provides.registerElementNamed(name, ret);
    } catch (SOFA.Connector.ConnectorException e) {
      throw new InstantiationException("ConnectorException: "+e.getMessage());
    }
  }
  
  /** Set new implementation object to provision, object must implement provided interface.
    * The method set object only, it suppose that everything is set.
    * It is call from primitive components only.
    * @param name   name of the provision
    * @param prov   implementation object
    * @exception NamingException         no provision with given name
    */
  public void reSetProvisionImpl(String name, Object prov) throws NamingException {
    try {
      RoleBase ret = (RoleBase) provides.getElementNamed(name);
      ret.link(prov);
    } catch (SOFA.Connector.ConnectorException e) {
      throw new InstantiationException("ConnectorException: "+e.getMessage());
    }
  }
  
  /** Delegate reference to subcomponent's provision. It is called from composed components only.
    * @param  name   name of the provision
    * @param  ref    reference to subcomponent's provision
    * @exception NamingException         no provision with given name
    * @exception InstantiationException  reference cannot be set
    */
  public void delegateProvision(String name, Reference ref) throws NamingException, InstantiationException {
    try {
    /*  SOFA.Connector.DeploymentDescriptor connDD = dd.getConnectorDeploymentDescriptor("_delegate_"+name);
      String type = connDD.getType();
      RoleBase ret = SOFA.Connector.ConnectorBuilder.createConnector(type, connDD, cl, node, dockName);
      ret.setThrIDRegistry(thrRegistry);
      ret.setRegister(this); */
      RoleBase ret = instantiateConnector("_delegate_"+name);
      ret.link(ref);
      delegatedProvides.registerElementNamed(name, ret);
    } catch (SOFA.Connector.ConnectorException e) {
      throw new InstantiationException("ConnectorException: "+e.getMessage());
    }
  }

  /** Creates component provision.
    * @param name   name of the provision
    * @exception NamingException         no provision with given name
    * @exception InstantiationException  provision cannot be created
    */
  public void createProvision(String name) throws NamingException, InstantiationException {
    RoleBase ret = instantiateConnector(name);
    provides.registerElementNamed(name, ret);
  }
  
  /** Connect component's provisions with provisions of all subcomponents. The methods createProvision
    * and delegateProvision must be called for all component's provision before calling this method.
    * @exception NamingException    error in the names of the provisions
    * @exception InstantiationException   delegated intefaces cannot be connected
    */
  public void connectDelegated()  throws NamingException, InstantiationException  {
    for (java.util.Enumeration e = provides.keys(); e.hasMoreElements(); ) {
      String name = (String) e.nextElement();
      RoleBase sRole = (RoleBase) provides.getElementNamed(name);
      RoleBase cRole = (RoleBase) delegatedProvides.getElementNamed(name);
      try {
        sRole.link(cRole);
      } catch (SOFA.Connector.LinkException ex) {
        throw new InstantiationException("LinkException: "+ex.getMessage() );
      }
    }
  }
  
  /** Creates subsumable requirment. Returned reference is passed to subcomponent by method setRequirement.
    * @param name   name of the requirment
    * @return SOFA reference
    * @exception NamingException    no requirment with given name
    * @exception InstantiationException   requirment cannot be created
    */
  public Reference createSubsumableRequirement(String name) throws NamingException, InstantiationException {
    /*  SOFA.Connector.DeploymentDescriptor connDD = dd.getConnectorDeploymentDescriptor("_subsume_"+name);
      String type = connDD.getType();
      RoleBase ret = SOFA.Connector.ConnectorBuilder.createConnector(type, connDD, cl, node, dockName);
      ret.setThrIDRegistry(thrRegistry);
      ret.setRegister(this);  */
    RoleBase ret = instantiateConnector("_subsume_"+name);
    subsumableRequires.forceRegisterElementNamed(name, ret);
    return ret.getSOFAReference();
  }

  public void connectSubsumable() throws NamingException, InstantiationException  {
    for (java.util.Enumeration e = subsumableRequires.keys(); e.hasMoreElements(); ) {
      String name = (String) e.nextElement();
      RoleBase sRole = (RoleBase) subsumableRequires.getElementNamed(name);
      RoleBase cRole = (RoleBase) requires.getElementNamed(name);
      try {
        sRole.link(cRole);
      } catch (SOFA.Connector.LinkException ex) {
        throw new InstantiationException("LinkException: "+ex.getMessage() );
      }
    }
  }

  /** Instantiated connector for provision/requirement with given name.
    * @param name   name of the provision/requirement
    */
  private RoleBase instantiateConnector(String name)  throws InstantiationException {
    try {
//      SOFA.SOFAnode.Run.SOFASystemClassLoader scl = new SOFA.SOFAnode.Run.SOFASystemClassLoader(); 
      SOFA.Connector.DeploymentDescriptor connDD = dd.getConnectorDeploymentDescriptor(name);
      String type = connDD.getConnectorBuilder();
      Class cl = Class.forName(type, true, SOFA.SOFAnode.Run.SOFACompClassLoader.getCompClassLoader());
      Class[] par = new Class[2];
      par[0] = Class.forName("SOFA.Connector.ECG.DeploymentDescriptor");
      par[1] = Class.forName("java.lang.ClassLoader");
      Method m = cl.getMethod("createConnector", par);
      Object[] arg = new Object[2];
      arg[0] = connDD;
      arg[1] = SOFA.SOFAnode.Run.SOFACompClassLoader.getCompClassLoader();
      RoleBase conn = (RoleBase) m.invoke(null, arg);
      conn.setThrIDRegistry(thrRegistry);
      conn.setComponentManager(thisRB);
      return conn;
      
   /*   RoleBase ret = SOFA.Connector.ConnectorBuilder.createConnector(type, connDD, cl, node, dockName);
      ret.setThrIDRegistry(thrRegistry);
      ret.setRegister(this);
      return ret; */
    } catch (ClassNotFoundException e)  {
      throw new InstantiationException("ClassNotFoundException: "+e.getMessage(), e);
    } catch (NoSuchMethodException e)  {
      throw new InstantiationException("NoSuchMethodException: "+e.getMessage(), e);
    } catch (IllegalAccessException e)  {
      throw new InstantiationException("IllegalAccessException: "+e.getMessage(), e);
    } catch (java.lang.reflect.InvocationTargetException e)  {
      throw new InstantiationException("InvocationTargetException: "+e.getMessage(), e);
    }
  }


  public SOFA.SOFAnode.Run.Deployment.DeplDock getLocalDeplDock() {
    return localdock;
  }

  /** Returns requirement with given name.
    * @param name  name of the requirement
    * @exception  NamingException    no requirement with the given name
    */
  public Object getRequirement(String name) throws NamingException {
    return requires.getElementNamed(name);
  }

  public String getFullName() {
    return fullName;
  }

  /** Loads implementation class of this component. It MUST be used for dynamic loading of classes
    * from implementation code of component.
    * 
    * @param name   class name
    * @return       class
    * @throws ClassNotFoundException class not found
    */
  public Class loadComponentClass(String name) throws ClassNotFoundException {
    return SOFA.SOFAnode.Run.SOFACompClassLoader.loadComponentClass(name, releaseVersion);
  }

    /** Server side part of the connector (SRole) to this CM */
  private SOFA.Connector.RoleBase thisRB;

  /** Sets role of the connector to this CM. 
    * 
    * @param rb connector role
    */
  public void connToThisCM(SOFA.Connector.RoleBase rb) {
    thisRB = rb;
  }

  /** Gets role of the connector to this CM.
    * 
    * @return connector role
    */
  public SOFA.Connector.RoleBase connToThisCM() {
    return thisRB;
  }



  //-- below - methods from fractal Component inteface

  public Object[] getFcInterfaces() {
    java.util.ArrayList ret = new java.util.ArrayList();
    for ( java.util.Enumeration e = provides.getAllNames(); e.hasMoreElements(); ) {
      ret.add(e.nextElement());
    }
    return ret.toArray();
  }
  
  public Object getFcInterface(java.lang.String itfName) throws org.objectweb.fractal.api.NoSuchInterfaceException {
    Object ret = null;
    try {
      ret = provides.getElementNamed(itfName);
    } catch (NamingException e) {
      throw new org.objectweb.fractal.api.NoSuchInterfaceException("No such interface", e);
    }
    return ret;
  }
  
  public org.objectweb.fractal.api.Type getFcType() {
    return this;
  }

  //-- below - methods from fractal Type inteface
  public boolean isFcSubtypeOf(org.objectweb.fractal.api.Type t) {
    // it is just temporal implementation
    return false;
  }
}
