/* $Id: DCUPComponentManagerImpl.java,v 1.4 2004/05/20 14:23:50 bures Exp $ */
package SOFA.Component.DCUP;

import SOFA.Component.ComponentLifecycleException;
import SOFA.SOFAnode.Run.Deployment.DeploymentDescriptor;

/** Implementation of DCUP component manager.
  *
  * @author Petr Hnetynka
  */
public class DCUPComponentManagerImpl extends SOFA.Component.ComponentManagerImpl implements DCUPComponentManager {
  
  /** Component builder */
  ComponentBuilder builder;


  /** Constructor */
  public DCUPComponentManagerImpl(DeploymentDescriptor _dd,  String _node, String _dockName, SOFA.SOFAnode.Run.Deployment.DeplDock _depldock, String _id) throws ComponentLifecycleException {
    super(_dd, _node, _dockName, _depldock, _id);
    System.out.println("Component manager");
    builder = null;
    try {
      Class builderClass = loadComponentClass(dd.getImplementationClass("builder"));
      builder = instantiateBuilder(builderClass);
    } catch (ClassNotFoundException e) {
      throw new ComponentLifecycleException("ClassNotFoundException in creation of component: "+e.getMessage(), e );
    }
  }
  
  
  public void createComponent(SOFA.SOFAnode.Run.Storage stateStore) throws SOFA.Component.ComponentLifecycleException {

    builder.onArrival(stateStore);
    System.out.println("createComponent");
  }

  
  public void destroyComponent(SOFA.SOFAnode.Run.Storage stateStore) throws SOFA.Component.ComponentLifecycleException {
    if (dd.isSystem()) {
      builder.onLeaving(stateStore);
      for (java.util.Enumeration sub = subcomponents.getAllNames(); sub.hasMoreElements(); ) {
        try {
          String sname = (String) sub.nextElement();
          DCUPComponentManager scm = (DCUPComponentManager) subcomponents.getElementNamed(sname);
          scm.destroyComponent(stateStore);
        } catch (SOFA.Component.NamingException e) {
          throw new ComponentLifecycleException("Error in the subcomponents", e);
        }
      }
      ((ComponentBuilderImpl) builder).stopped = true;
      builder.onLeaving(stateStore);
      for (java.util.Enumeration sub = subcomponents.getAllNames(); sub.hasMoreElements(); ) {
        try {
          String sname = (String) sub.nextElement();
          DCUPComponentManager scm = (DCUPComponentManager) subcomponents.getElementNamed(sname);
          scm.destroyComponent(stateStore);
        } catch (SOFA.Component.NamingException e) {
          throw new ComponentLifecycleException("Error in the subcomponents", e);
        }
      }
    } else { // if it isn't system component 
      if (!builder.isStopped()) {
        builder.onLeaving(stateStore);
        for (java.util.Enumeration sub = subcomponents.getAllNames(); sub.hasMoreElements(); ) {
          try {
            String sname = (String) sub.nextElement();
            DCUPComponentManager scm = (DCUPComponentManager) subcomponents.getElementNamed(sname);
            scm.destroyComponent(stateStore);
          } catch (SOFA.Component.NamingException e) {
            throw new ComponentLifecycleException("Error in the subcomponents", e);
          }
        }
        ((ComponentBuilderImpl) builder).stopped = true;
      } else {
        for (java.util.Enumeration sub = subcomponents.getAllNames(); sub.hasMoreElements(); ) {
          try {
            String sname = (String) sub.nextElement();
            DCUPComponentManager scm = (DCUPComponentManager) subcomponents.getElementNamed(sname);
            scm.destroyComponent(stateStore);
          } catch (SOFA.Component.NamingException e) {
            throw new ComponentLifecycleException("Error in the subcomponents", e);
          }
        }
        builder.onLeaving(stateStore);
      }
    }
     
    System.out.println("destroyComponent");
  }

  
  public void storeComponent(SOFA.SOFAnode.Run.Storage stateStore) throws SOFA.Component.ComponentLifecycleException {}
  public void restoreComponent(SOFA.SOFAnode.Run.Storage stateStore) throws SOFA.Component.ComponentLifecycleException {}

  public void updateComponent(SOFA.SOFAnode.Run.Deployment.DeploymentDescriptor newDeploymentDescriptor, SOFA.SOFAnode.Run.Storage stateStore) throws SOFA.Component.ComponentLifecycleException {
    if (!builder.isUpdatable())
      throw new SOFA.Component.ComponentLifecycleException("This component can't be updated.");
    if (dd.getArchitectureAbsNameWithVersion().compareTo(newDeploymentDescriptor.getArchitectureAbsNameWithVersion())!=0)
      throw new SOFA.Component.ComponentLifecycleException("Incompatible components.");
  //  destroyComponent(stateStore);
    ((SOFA.Component.ThreadIDRegistryImpl)thrRegistry).setInUpdate(true);
    String id = dd.getInstanceId();
    String iName = dd.getInstanceName();
    dd = newDeploymentDescriptor;
    ((SOFA.SOFAnode.Run.Deployment.DeploymentDescriptorImpl)dd).setInstanceId(id);
    ((SOFA.SOFAnode.Run.Deployment.DeploymentDescriptorImpl)dd).setInstanceName(iName);
    releaseVersion = dd.getReleaseVersion();
 //  bcl = SOFA.SOFAnode.Run.SOFABaseClassLoader.getBaseClassLoader(icl, dd.getArchitectureAbsName(), releaseVersion);
 //   ((SOFA.SOFAnode.Run.SOFABaseClassLoader) cl).setVersion(releaseVersion);
    builder = null;
    try {
      Class builderClass = loadComponentClass(dd.getImplementationClass("builder"));
      builder = instantiateBuilder(builderClass);
    } catch (ClassNotFoundException e) {
      throw new ComponentLifecycleException("ClassNotFoundException in creation of component: "+e.getMessage(), e );
    }
    createComponent(stateStore);
    fullName = dd.getArchitectureAbsName()+"["+dd.getReleaseVersion()+"]";
    ((SOFA.Component.ThreadIDRegistryImpl)thrRegistry).setInUpdate(false);
  }


  
  /** Instantiate component builder from its class. */
  protected ComponentBuilder instantiateBuilder(Class builderClass) throws ComponentLifecycleException {
    try {
      ComponentBuilder ret = (ComponentBuilder) builderClass.newInstance();
      ret.initialize(this, dd);
      return ret;
    } catch (InstantiationException e) {
      throw new ComponentLifecycleException("Builder instantiate error: "+e.getMessage());
    } catch (IllegalAccessException e) {
      throw new ComponentLifecycleException("Builder instantiate error: "+e.getMessage());
    }
  }

  public boolean isStopped() {
    return builder.isStopped();
  }

  public boolean isUpdatable() {
    return builder.isUpdatable();
  }

}
