/*
 * EventPassingImpl.java
 *
 * Created on 5. duben 2002, 13:50
 */

package SOFA.Connector.ECG.SGenerator.Connectors;

import SOFA.Connector.ConnectorException;
import SOFA.Connector.LinkException;
import SOFA.Connector.Linkable;
import SOFA.Connector.Reference;
import SOFA.Connector.RoleBase;
import SOFA.Connector.RoleBaseImpl;
import SOFA.Connector.TaggedProfile;
import SOFA.Connector.EEG.Types.CRole;
import SOFA.Connector.EEG.Types.Distributor;
import SOFA.Connector.EEG.Types.Interceptor;
import SOFA.Connector.EEG.Types.SRole;
import SOFA.Connector.EEG.Types.Skeleton;
import SOFA.Connector.EEG.Types.Stub;

/**
 *
 * @author  ghort
 * @version 
 */
public class EventPassingImpl extends RoleBaseImpl implements SOFA.Connector.ECG.Types.EventPassing, SOFA.Connector.EEG.ConnectorManagerInterface {

	protected SRole cRole;
	protected Interceptor cInterceptor;
	protected Skeleton skeleton[];

	protected CRole sRole;
	protected Interceptor sInterceptor;
	protected Distributor distributor;
	protected java.util.LinkedList availableStubs; // of Stub
	
	/** Creates new EventPassingImpl */
	public EventPassingImpl() {
	}

	static public RoleBase createConnector(SOFA.Connector.ECG.DeploymentDescriptor dd, ClassLoader cl) throws ConnectorException {
		EventPassingImpl pc=new EventPassingImpl();
		RoleBase role=null;
		if (dd.unit.equals("Supplier")) {
			try {
				Linkable last;

				SOFA.Connector.EEG.ElementOutputDescriptor sRoleClass=SOFA.Connector.EEG.ElementOutputDescriptor.findFirst(dd.elements,"sRole");
				pc.sRole=(CRole)Class.forName(sRoleClass.impl, true, cl).newInstance();
				pc.sRole.setManagerTie(pc);
				last=pc.sRole;

				SOFA.Connector.EEG.ElementOutputDescriptor sInterceptorClass=SOFA.Connector.EEG.ElementOutputDescriptor.findFirst(dd.elements,"sInterceptor");
				if (sInterceptorClass!=null) {
					pc.sInterceptor=(Interceptor)Class.forName(sInterceptorClass.impl, true, cl).newInstance();
					pc.sInterceptor.setManagerTie(pc);
					last.link(pc.sInterceptor);
					last=pc.sInterceptor;
				}

				SOFA.Connector.EEG.ElementOutputDescriptor distributorClass=SOFA.Connector.EEG.ElementOutputDescriptor.findFirst(dd.elements,"distributor");
				pc.distributor=(Distributor)Class.forName(distributorClass.impl, true, cl).newInstance();
				pc.distributor.setManagerTie(pc);
				last.link(pc.distributor);
				
				pc.availableStubs=new java.util.LinkedList();

				for (int i=0;i<dd.elements.length;i++) {
					if (dd.elements[i].name.equals("stub")) {
						pc.availableStubs.add(Class.forName(dd.elements[i].impl, true, cl).newInstance());
					}
				}

				pc.setDeploymentDescriptor(dd);

				role=pc.sRole;
			} catch (Exception e) {
				throw new ConnectorException("Can't instantiate sRole.",e);
			}
		} else if (dd.unit.equals("Consumer")) {
			try {
				Object last;

				SOFA.Connector.EEG.ElementOutputDescriptor cRoleClass=SOFA.Connector.EEG.ElementOutputDescriptor.findFirst(dd.elements,"cRole");
				pc.cRole=(SRole)Class.forName(cRoleClass.impl, true, cl).newInstance();
				pc.cRole.setManagerTie(pc);
				last=pc.cRole;
				
				SOFA.Connector.EEG.ElementOutputDescriptor cInterceptorClass=SOFA.Connector.EEG.ElementOutputDescriptor.findFirst(dd.elements,"cInterceptor");
				if (cInterceptorClass!=null) {
					pc.cInterceptor=(Interceptor)Class.forName(cInterceptorClass.impl, true, cl).newInstance();
					pc.cInterceptor.setManagerTie(pc);
					pc.cInterceptor.link(last);
					last=pc.cInterceptor;
				}

				int i, j, skelNum=0;

				for (i=0;i<dd.elements.length;i++) {
					if (dd.elements[i].name.equals("skeleton")) {
						skelNum++;
					}
				}

				pc.skeleton=new Skeleton[skelNum];
				java.util.LinkedList profiles=new java.util.LinkedList();

				skelNum=0;
				for (i=0;i<dd.elements.length;i++) {
					if (dd.elements[i].name.equals("skeleton")) {
						pc.skeleton[skelNum]=(Skeleton)Class.forName(dd.elements[i].impl, true, cl).newInstance();
						pc.skeleton[skelNum].setManagerTie(pc);
						pc.skeleton[skelNum].link(last);
						
						TaggedProfile[] prf=pc.skeleton[skelNum].getProfiles();
						for (j=0;j<prf.length;j++) {
							profiles.add(prf[j]);
						}
						skelNum++;
					}
				}

				TaggedProfile[] prf=new TaggedProfile[profiles.size()];
				java.util.Iterator iter=profiles.iterator();
				for (i=0;iter.hasNext();i++) {
					prf[i]=(TaggedProfile)iter.next();
				}
				
				pc.setSOFAReference(new Reference(dd.impl,prf));
				pc.setDeploymentDescriptor(dd);
				
				role=pc.cRole;
			} catch (Exception e) {
				throw new ConnectorException("Can't instantiate cRole.",e);
			}
		} else {
			throw new ConnectorException("Unknown unit '"+dd.unit+"'.");
		}
		return role;
	}

	public void link(Object target) throws LinkException {
		Reference ref=(Reference)target;
		java.util.Iterator iter=availableStubs.iterator();
		
		while (iter.hasNext()) {
			Stub st=(Stub)iter.next();
			String[] profileNames=(st).getSupportedProfiles();
			int i, j;

			for (i=0;i<profileNames.length;i++) {
				for (j=0;j<ref.profiles.length;j++) {
					if (profileNames[i].equals(ref.profiles[j].tag)) {
						try {
							st=(Stub)st.getClass().newInstance();
							st.setManagerTie(this);
							st.link(ref.profiles[j]);
							distributor.link(st);
						} catch (Exception e) {
							throw new LinkException("Can't create stub ("+st.getClass()+").",e);
						}
						return;
					}
				}
			}
		}
	}

        public String getName() {
          return getConnectorName();
        }

}
