/*
 * DistributorImplGen.java
 *
 * Created on 13. duben 2002, 21:08
 */

package SOFA.Connector.EEG.EEM.Elements;

import SOFA.Connector.EEG.CodeWriter.JWriterDirectorException;
import SOFA.Connector.EEG.EEM.Actions.JIfaceCN;
import SOFA.Connector.EEG.EEM.Actions.JImplWriterDirectorBase;
import SOFA.SOFAnode.Made.TIR.CDLType;
import SOFA.SOFAnode.Made.TIR.DefinitionKind;
import SOFA.SOFAnode.Made.TIR.OperationDef;
import SOFA.SOFAnode.Made.TIR.ParamDescr;
import SOFA.SOFAnode.Made.TIR.ParamMode;
import SOFA.SOFAnode.Made.TIR.PrimitiveDef;
import SOFA.SOFAnode.Made.TIR.PrimitiveKind;
import SOFA.SOFAnode.Made.TIR.TIRExceptLock;

/**
 *
 * @author  ghort
 * @version 
 */
public class DistributorImplGen extends JImplWriterDirectorBase {

	/** Creates new DistributorImplGen */
	public DistributorImplGen(String provIfaceCDL, SOFA.Connector.Property[] params) {
		super(provIfaceCDL,params);
	}

	public String[] getImplements() {
		return new String[] {"SOFA.Connector.EEG.Types.Distributor"};
	}
	
	public String getClassBody() {
		return managerTieImpl() + "\n" +
			"\tprotected java.util.HashSet listeners = new java.util.HashSet();\n\n"+
			"\tpublic synchronized void link(Object listener) throws SOFA.Connector.LinkException {\n"+
			"\t\tif (!listeners.add(listener)) {\n"+
			"\t\t\tthrow new SOFA.Connector.LinkException(\"Listener already present.\");\n"+
			"\t\t}\n"+
			"\t}\n\n"+
			"\tpublic synchronized void unlink(Object listener) throws SOFA.Connector.LinkException {\n"+
			"\t\tif (!listeners.remove(listener)) {\n"+
			"\t\t\tthrow new SOFA.Connector.LinkException(\"Listener not found.\");\n"+
			"\t\t}\n"+
			"\t}\n\n"+
                        "\tpublic org.objectweb.fractal.api.Type getFcItfType() { return null; }\n"+
                        "\tpublic boolean isFcInternalItf() { return false; }\n"+
                        "\tpublic String getFcItfName() { throw new SOFA.Connector.ConnectorTransportException(\"Interconection via EventPassing connector - getFcItfName is not supported.\"); }\n"+
                        "\tpublic org.objectweb.fractal.api.Component getFcItfOwner() { throw new SOFA.Connector.ConnectorTransportException(\"Interconection via EventPassing connector - getFcItfOwner is not supported.\"); }\n\n";
	}

	public String getMethodDef(OperationDef mt) throws JWriterDirectorException {
		return JIfaceCN.getMethodDef(mt);
	}
	
	public String getMethodBody(OperationDef method) throws JWriterDirectorException {
		try {
			String reqIfaceMapped=SOFA.Connector.Property.findFirst(params,"reqIfaceMapped");
			
			CDLType retType=method.result();
			if (!(retType.get_def_kind().value()==DefinitionKind.dk_Primitive && 
				((PrimitiveDef)retType).kind().value()==PrimitiveKind.pk_void)) {
				throw new JWriterDirectorException("Can't distribute calls which return value. (method: '"+method.get_identification().name()+"')");
			}
			int i;
			ParamDescr[] params=method.params();

			for (i=0;i<params.length;i++) {
				int mode=params[i].mode().value();
				if (mode==ParamMode.PARAM_INOUT || mode==ParamMode.PARAM_OUT) {
					throw new JWriterDirectorException("Can't distribute calls which which have in/inout parameters. (method: '"+method.get_identification().name()+"')");
				}
			}

			return 
				"\t\tsynchronized (this) {\n"+
				"\t\t\tjava.util.Iterator iter=listeners.iterator();\n"+
				"\t\t\twhile (iter.hasNext()) {\n"+
				"\t\t\t\t"+reqIfaceMapped+" tgt=("+reqIfaceMapped+")iter.next();\n"+
				"\t\t\t\ttgt."+method.get_identification().name()+"("+JIfaceCN.targetCallParams(method)+");\n"+
				"\t\t\t}\n"+
				"\t\t}\n";
		} catch (TIRExceptLock e) {
			throw new JWriterDirectorException("Cannot access TIR.",e);
		} catch (java.rmi.RemoteException e) {
			throw new JWriterDirectorException("Cannot access TIR.",e);
		}
	}
}
