/*
 * ContractRuleEditDialog.java
 *
 * Created on 17. kvten 2004, 22:47
 */

package SOFA.SOFAnet.Browser;

import SOFA.SOFAnet.Repository.ContractRule;
import SOFA.SOFAnet.Repository.NodeNameFilter;
import SOFA.SOFAnet.Repository.BundleNameFilter;
import SOFA.SOFAnet.Repository.NodeNameLicList;
import SOFA.SOFAnet.Repository.Licence;
import java.util.Date;
import java.text.DateFormat;
import java.text.ParseException;
import javax.swing.*;
import java.awt.Component;
import java.awt.Frame;

/**
 *
 * @author  Ladislav Sobr
 */
public class ContractRuleEditDialog extends javax.swing.JDialog
{
  private ContractRule contractRule;
  private BundleNameFilter software;
  private Licence licence;
  private NodeNameLicList pushDestinationNodes;
  private NodeNameFilter pushNodeFilter;
  private NodeNameFilter pullNodeFilter;
  private boolean isOK;
  
  public static boolean showDialog(Component frameComp, ContractRule contractRule, String title)
  {
    Frame frame = JOptionPane.getFrameForComponent(frameComp);
    ContractRuleEditDialog dialog = new ContractRuleEditDialog(frame, true, contractRule, title);
    dialog.setLocationRelativeTo(frameComp);
    dialog.setVisible(true);
    return dialog.isOK();
  }
  
  /** Creates new form ContractRuleEditDialog */
  public ContractRuleEditDialog(java.awt.Frame parent, boolean modal, ContractRule contractRule, String title)
  {
    super(parent, modal);
    this.contractRule = contractRule;
    software = (BundleNameFilter)contractRule.getSoftware().clone();
    licence = (Licence)contractRule.getLicence().clone();
    pushDestinationNodes = contractRule.getPushDestinationNodes();
    pushNodeFilter = contractRule.getPushNodeFilter();
    pullNodeFilter = contractRule.getPullNodeFilter();
    isOK = false;
    
    initComponents();
    setTitle(title);
    
    nameTextField.setText(contractRule.getName());
    descriptionTextField.setText(contractRule.getDescription());

    Date validFrom = contractRule.getValidFrom();
    Date validTo = contractRule.getValidTo();
    DateFormat dateFormat = DateFormat.getDateTimeInstance(DateFormat.MEDIUM, DateFormat.MEDIUM);

    if (validFrom != null) validFromTextField.setText(dateFormat.format(validFrom));
    else validFromTextField.setText("");

    if (validTo != null) validToTextField.setText(dateFormat.format(validTo));
    else validToTextField.setText("");
    
    autodeleteCheckBox.setSelected(contractRule.getAutoDelete());

    int pushCounter = contractRule.getPushCounter();
    
    if (contractRule.isPush())
    {
      pushCheckBox.setSelected(true);
      pushOptionsPanel.setEnabled(true);
      pushCounterCheckBox.setEnabled(true);
      pushDestinationNodesCheckBox.setEnabled(true);
      pushNodeFilterCheckBox.setEnabled(true);
      
      pushCounterTextField.setEnabled(pushCounter != -1);
      pushDestinationNodesButton.setEnabled(pushDestinationNodes != null);
      pushNodeFilterButton.setEnabled(pushNodeFilter != null);
    }
    else
    {
      pushCheckBox.setSelected(false);
      pushOptionsPanel.setEnabled(false);
      pushCounterCheckBox.setEnabled(false);
      pushDestinationNodesCheckBox.setEnabled(false);
      pushNodeFilterCheckBox.setEnabled(false);
      
      pushCounterTextField.setEnabled(false);
      pushDestinationNodesButton.setEnabled(false);
      pushNodeFilterButton.setEnabled(false);
    }

    if (pushCounter != -1)
    {
      pushCounterCheckBox.setSelected(true);
      pushCounterTextField.setText(Integer.toString(pushCounter));
    }
    else
    {
      pushCounterCheckBox.setSelected(false);
      pushCounterTextField.setText("");
    }
    
    pushDestinationNodesCheckBox.setSelected(pushDestinationNodes != null);
    pushNodeFilterCheckBox.setSelected(pushNodeFilter != null);

    int pullCounter = contractRule.getPullCounter();

    if (contractRule.isPull())
    {
      pullCheckBox.setSelected(true);
      pullOptionsPanel.setEnabled(true);
      pullCounterCheckBox.setEnabled(true);
      pullNodeFilterCheckBox.setEnabled(true);
      
      pullCounterTextField.setEnabled(pullCounter != -1);
      pullNodeFilterButton.setEnabled(pullNodeFilter != null);
    }
    else
    {
      pullCheckBox.setSelected(false);
      pullOptionsPanel.setEnabled(false);
      pullCounterCheckBox.setEnabled(false);
      pullNodeFilterCheckBox.setEnabled(false);
      
      pullCounterTextField.setEnabled(false);
      pullNodeFilterButton.setEnabled(false);
    }
    
    if (pullCounter != -1)
    {
      pullCounterCheckBox.setSelected(true);
      pullCounterTextField.setText(Integer.toString(pullCounter));
    }
    else
    {
      pullCounterCheckBox.setSelected(false);
      pullCounterTextField.setText("");
    }
    
    pullNodeFilterCheckBox.setSelected(pullNodeFilter != null);
    
    if (pushDestinationNodes != null) pushDestinationNodes = (NodeNameLicList)pushDestinationNodes.clone();
    else pushDestinationNodes = new NodeNameLicList();
    if (pushNodeFilter != null) pushNodeFilter = (NodeNameFilter)pushNodeFilter.clone();
    else pushNodeFilter = new NodeNameFilter();
    if (pullNodeFilter != null) pullNodeFilter = (NodeNameFilter)pullNodeFilter.clone();
    else pullNodeFilter = new NodeNameFilter();
  }
  
  public boolean isOK()
  {
    return isOK;
  }
  
  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the Form Editor.
   */
  private void initComponents()//GEN-BEGIN:initComponents
  {
    nameAndDescriptionPanel = new javax.swing.JPanel();
    nameLabel = new javax.swing.JLabel();
    nameTextField = new javax.swing.JTextField();
    descriptionLabel = new javax.swing.JLabel();
    descriptionTextField = new javax.swing.JTextField();
    jPanel7 = new javax.swing.JPanel();
    softwareAndLicencePanel = new javax.swing.JPanel();
    softwareButton = new javax.swing.JButton();
    licenceButton = new javax.swing.JButton();
    jPanel2 = new javax.swing.JPanel();
    jPanel3 = new javax.swing.JPanel();
    parametersPanel = new javax.swing.JPanel();
    validFromPanel = new javax.swing.JPanel();
    validFromLabel = new javax.swing.JLabel();
    validFromTextField = new javax.swing.JTextField();
    validToPanel = new javax.swing.JPanel();
    validToLabel = new javax.swing.JLabel();
    validToTextField = new javax.swing.JTextField();
    autodeleteCheckBox = new javax.swing.JCheckBox();
    actionsPanel = new javax.swing.JPanel();
    jPanel5 = new javax.swing.JPanel();
    pushCheckBox = new javax.swing.JCheckBox();
    pullCheckBox = new javax.swing.JCheckBox();
    jPanel4 = new javax.swing.JPanel();
    jPanel9 = new javax.swing.JPanel();
    pushOptionsPanel = new javax.swing.JPanel();
    pushCounterPanel = new javax.swing.JPanel();
    pushCounterCheckBox = new javax.swing.JCheckBox();
    pushCounterTextField = new javax.swing.JTextField();
    pushNodeFilterPanel = new javax.swing.JPanel();
    pushNodeFilterCheckBox = new javax.swing.JCheckBox();
    pushNodeFilterButton = new javax.swing.JButton();
    pushDestinationNodesPanel = new javax.swing.JPanel();
    pushDestinationNodesCheckBox = new javax.swing.JCheckBox();
    pushDestinationNodesButton = new javax.swing.JButton();
    pullOptionsPanel = new javax.swing.JPanel();
    pullCounterPanel = new javax.swing.JPanel();
    pullCounterCheckBox = new javax.swing.JCheckBox();
    pullCounterTextField = new javax.swing.JTextField();
    pullNodeFilterPanel = new javax.swing.JPanel();
    pullNodeFilterCheckBox = new javax.swing.JCheckBox();
    pullNodeFilterButton = new javax.swing.JButton();
    jPanel6 = new javax.swing.JPanel();
    OKButton = new javax.swing.JButton();
    cancelButton = new javax.swing.JButton();

    setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
    setTitle("Contract Rule Editor");
    nameAndDescriptionPanel.setLayout(new java.awt.GridLayout(4, 0));

    nameAndDescriptionPanel.setBorder(new javax.swing.border.TitledBorder("Name and Description of Contract Rule"));
    nameLabel.setText("Name:");
    nameAndDescriptionPanel.add(nameLabel);

    nameTextField.setEditable(false);
    nameAndDescriptionPanel.add(nameTextField);

    descriptionLabel.setText("Description:");
    nameAndDescriptionPanel.add(descriptionLabel);

    nameAndDescriptionPanel.add(descriptionTextField);

    getContentPane().add(nameAndDescriptionPanel, java.awt.BorderLayout.NORTH);

    jPanel7.setLayout(new java.awt.BorderLayout());

    softwareAndLicencePanel.setBorder(new javax.swing.border.TitledBorder("Covered Software And Licence"));
    softwareButton.setText("Edit Software (Bundle Filter)");
    softwareButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        softwareButtonActionPerformed(evt);
      }
    });

    softwareAndLicencePanel.add(softwareButton);

    licenceButton.setText("Edit Licence");
    licenceButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        licenceButtonActionPerformed(evt);
      }
    });

    softwareAndLicencePanel.add(licenceButton);

    jPanel7.add(softwareAndLicencePanel, java.awt.BorderLayout.NORTH);

    jPanel2.setLayout(new java.awt.BorderLayout());

    jPanel3.setLayout(new java.awt.GridLayout(1, 0));

    parametersPanel.setLayout(new java.awt.GridLayout(3, 0));

    parametersPanel.setBorder(new javax.swing.border.TitledBorder("Parameters"));
    validFromPanel.setLayout(new javax.swing.BoxLayout(validFromPanel, javax.swing.BoxLayout.X_AXIS));

    validFromLabel.setText("Valid From:");
    validFromPanel.add(validFromLabel);

    validFromPanel.add(validFromTextField);

    parametersPanel.add(validFromPanel);

    validToPanel.setLayout(new javax.swing.BoxLayout(validToPanel, javax.swing.BoxLayout.X_AXIS));

    validToLabel.setText("Valid To:");
    validToPanel.add(validToLabel);

    validToPanel.add(validToTextField);

    parametersPanel.add(validToPanel);

    autodeleteCheckBox.setText("Automatically Delete When Expired");
    parametersPanel.add(autodeleteCheckBox);

    jPanel3.add(parametersPanel);

    actionsPanel.setLayout(new java.awt.BorderLayout());

    actionsPanel.setBorder(new javax.swing.border.TitledBorder("Covered Actions"));
    actionsPanel.setMinimumSize(new java.awt.Dimension(200, 111));
    actionsPanel.setPreferredSize(new java.awt.Dimension(200, 111));
    jPanel5.setLayout(new java.awt.GridLayout(2, 0));

    pushCheckBox.setText("Push");
    pushCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        pushCheckBoxActionPerformed(evt);
      }
    });

    jPanel5.add(pushCheckBox);

    pullCheckBox.setText("Pull");
    pullCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        pullCheckBoxActionPerformed(evt);
      }
    });

    jPanel5.add(pullCheckBox);

    actionsPanel.add(jPanel5, java.awt.BorderLayout.NORTH);

    jPanel3.add(actionsPanel);

    jPanel2.add(jPanel3, java.awt.BorderLayout.NORTH);

    jPanel4.setLayout(new java.awt.BorderLayout());

    jPanel9.setLayout(new java.awt.GridLayout(1, 0));

    pushOptionsPanel.setLayout(new java.awt.GridLayout(3, 0));

    pushOptionsPanel.setBorder(new javax.swing.border.TitledBorder("Push Options"));
    pushCounterPanel.setLayout(new javax.swing.BoxLayout(pushCounterPanel, javax.swing.BoxLayout.X_AXIS));

    pushCounterCheckBox.setText("Push Counter");
    pushCounterCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        pushCounterCheckBoxActionPerformed(evt);
      }
    });

    pushCounterPanel.add(pushCounterCheckBox);

    pushCounterPanel.add(pushCounterTextField);

    pushOptionsPanel.add(pushCounterPanel);

    pushNodeFilterPanel.setLayout(new javax.swing.BoxLayout(pushNodeFilterPanel, javax.swing.BoxLayout.X_AXIS));

    pushNodeFilterCheckBox.setText("Push Node Filter");
    pushNodeFilterCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        pushNodeFilterCheckBoxActionPerformed(evt);
      }
    });

    pushNodeFilterPanel.add(pushNodeFilterCheckBox);

    pushNodeFilterButton.setText("Edit Push Node Filter");
    pushNodeFilterButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        pushNodeFilterButtonActionPerformed(evt);
      }
    });

    pushNodeFilterPanel.add(pushNodeFilterButton);

    pushOptionsPanel.add(pushNodeFilterPanel);

    pushDestinationNodesPanel.setLayout(new javax.swing.BoxLayout(pushDestinationNodesPanel, javax.swing.BoxLayout.X_AXIS));

    pushDestinationNodesCheckBox.setText("Push Destination Nodes");
    pushDestinationNodesCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        pushDestinationNodesCheckBoxActionPerformed(evt);
      }
    });

    pushDestinationNodesPanel.add(pushDestinationNodesCheckBox);

    pushDestinationNodesButton.setText("Edit Nodes");
    pushDestinationNodesButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        pushDestinationNodesButtonActionPerformed(evt);
      }
    });

    pushDestinationNodesPanel.add(pushDestinationNodesButton);

    pushOptionsPanel.add(pushDestinationNodesPanel);

    jPanel9.add(pushOptionsPanel);

    pullOptionsPanel.setLayout(new java.awt.GridLayout(3, 0));

    pullOptionsPanel.setBorder(new javax.swing.border.TitledBorder("Pull Options"));
    pullCounterPanel.setLayout(new javax.swing.BoxLayout(pullCounterPanel, javax.swing.BoxLayout.X_AXIS));

    pullCounterCheckBox.setText("Pull Counter");
    pullCounterCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        pullCounterCheckBoxActionPerformed(evt);
      }
    });

    pullCounterPanel.add(pullCounterCheckBox);

    pullCounterPanel.add(pullCounterTextField);

    pullOptionsPanel.add(pullCounterPanel);

    pullNodeFilterPanel.setLayout(new javax.swing.BoxLayout(pullNodeFilterPanel, javax.swing.BoxLayout.X_AXIS));

    pullNodeFilterCheckBox.setText("Pull Node Filter");
    pullNodeFilterCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        pullNodeFilterCheckBoxActionPerformed(evt);
      }
    });

    pullNodeFilterPanel.add(pullNodeFilterCheckBox);

    pullNodeFilterButton.setText("Edit Pull Node Filter");
    pullNodeFilterButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        pullNodeFilterButtonActionPerformed(evt);
      }
    });

    pullNodeFilterPanel.add(pullNodeFilterButton);

    pullOptionsPanel.add(pullNodeFilterPanel);

    jPanel9.add(pullOptionsPanel);

    jPanel4.add(jPanel9, java.awt.BorderLayout.NORTH);

    jPanel2.add(jPanel4, java.awt.BorderLayout.CENTER);

    jPanel7.add(jPanel2, java.awt.BorderLayout.CENTER);

    getContentPane().add(jPanel7, java.awt.BorderLayout.CENTER);

    OKButton.setText("OK");
    OKButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        OKButtonActionPerformed(evt);
      }
    });

    jPanel6.add(OKButton);

    cancelButton.setText("Cancel");
    cancelButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        cancelButtonActionPerformed(evt);
      }
    });

    jPanel6.add(cancelButton);

    getContentPane().add(jPanel6, java.awt.BorderLayout.SOUTH);

    pack();
  }//GEN-END:initComponents

  private void pushNodeFilterButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_pushNodeFilterButtonActionPerformed
  {//GEN-HEADEREND:event_pushNodeFilterButtonActionPerformed
    StringListEditDialog.showDialog(this, pushNodeFilter.getPatterns(), "Node Name Pattern", "Push Node Filter Editor");
  }//GEN-LAST:event_pushNodeFilterButtonActionPerformed

  private void pushNodeFilterCheckBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_pushNodeFilterCheckBoxActionPerformed
  {//GEN-HEADEREND:event_pushNodeFilterCheckBoxActionPerformed
    pushNodeFilterButton.setEnabled(pushNodeFilterCheckBox.isSelected());
  }//GEN-LAST:event_pushNodeFilterCheckBoxActionPerformed

  private void pullNodeFilterButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_pullNodeFilterButtonActionPerformed
  {//GEN-HEADEREND:event_pullNodeFilterButtonActionPerformed
    StringListEditDialog.showDialog(this, pullNodeFilter.getPatterns(), "Node Name Pattern", "Pull Node Filter Editor");
  }//GEN-LAST:event_pullNodeFilterButtonActionPerformed

  private void pullNodeFilterCheckBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_pullNodeFilterCheckBoxActionPerformed
  {//GEN-HEADEREND:event_pullNodeFilterCheckBoxActionPerformed
    pullNodeFilterButton.setEnabled(pullNodeFilterCheckBox.isSelected());
  }//GEN-LAST:event_pullNodeFilterCheckBoxActionPerformed

  private void pullCounterCheckBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_pullCounterCheckBoxActionPerformed
  {//GEN-HEADEREND:event_pullCounterCheckBoxActionPerformed
    pullCounterTextField.setEnabled(pullCounterCheckBox.isSelected());
  }//GEN-LAST:event_pullCounterCheckBoxActionPerformed

  private void pushDestinationNodesButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_pushDestinationNodesButtonActionPerformed
  {//GEN-HEADEREND:event_pushDestinationNodesButtonActionPerformed
    NodeNameLicListEditDialog.showDialog(this, pushDestinationNodes, "Push Destination Nodes Editor");
  }//GEN-LAST:event_pushDestinationNodesButtonActionPerformed

  private void pushDestinationNodesCheckBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_pushDestinationNodesCheckBoxActionPerformed
  {//GEN-HEADEREND:event_pushDestinationNodesCheckBoxActionPerformed
    pushDestinationNodesButton.setEnabled(pushDestinationNodesCheckBox.isSelected());
  }//GEN-LAST:event_pushDestinationNodesCheckBoxActionPerformed

  private void pushCounterCheckBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_pushCounterCheckBoxActionPerformed
  {//GEN-HEADEREND:event_pushCounterCheckBoxActionPerformed
    pushCounterTextField.setEnabled(pushCounterCheckBox.isSelected());
  }//GEN-LAST:event_pushCounterCheckBoxActionPerformed

  private void pullCheckBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_pullCheckBoxActionPerformed
  {//GEN-HEADEREND:event_pullCheckBoxActionPerformed
    if (pullCheckBox.isSelected())
    {
      pullOptionsPanel.setEnabled(true);
      pullCounterCheckBox.setEnabled(true);
      pullNodeFilterCheckBox.setEnabled(true);

      pullCounterTextField.setEnabled(pullCounterCheckBox.isSelected());
      pullNodeFilterButton.setEnabled(pullNodeFilterCheckBox.isSelected());
    }
    else
    {
      pullOptionsPanel.setEnabled(false);
      pullCounterCheckBox.setEnabled(false);
      pullNodeFilterCheckBox.setEnabled(false);

      pullCounterTextField.setEnabled(false);
      pullNodeFilterButton.setEnabled(false);
    }
  }//GEN-LAST:event_pullCheckBoxActionPerformed

  private void pushCheckBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_pushCheckBoxActionPerformed
  {//GEN-HEADEREND:event_pushCheckBoxActionPerformed
    if (pushCheckBox.isSelected())
    {
      pushOptionsPanel.setEnabled(true);
      pushCounterCheckBox.setEnabled(true);
      pushDestinationNodesCheckBox.setEnabled(true);
      pushNodeFilterCheckBox.setEnabled(true);

      pushCounterTextField.setEnabled(pushCounterCheckBox.isSelected());
      pushDestinationNodesButton.setEnabled(pushDestinationNodesCheckBox.isSelected());
      pushNodeFilterButton.setEnabled(pushNodeFilterCheckBox.isSelected());
    }
    else
    {
      pushOptionsPanel.setEnabled(false);
      pushCounterCheckBox.setEnabled(false);
      pushDestinationNodesCheckBox.setEnabled(false);
      pushNodeFilterCheckBox.setEnabled(false);

      pushCounterTextField.setEnabled(false);
      pushDestinationNodesButton.setEnabled(false);
      pushNodeFilterButton.setEnabled(false);
    }
  }//GEN-LAST:event_pushCheckBoxActionPerformed

  private void licenceButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_licenceButtonActionPerformed
  {//GEN-HEADEREND:event_licenceButtonActionPerformed
    LicenceEditDialog.showDialog(this, licence, true, "Licence of Contract Rule");
  }//GEN-LAST:event_licenceButtonActionPerformed

  private void softwareButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_softwareButtonActionPerformed
  {//GEN-HEADEREND:event_softwareButtonActionPerformed
    StringListEditDialog.showDialog(this, software.getPatterns(), "Bundle Name Pattern", "Software covered by Contract Rule - Bundle Filter Editor");
  }//GEN-LAST:event_softwareButtonActionPerformed

  private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_cancelButtonActionPerformed
  {//GEN-HEADEREND:event_cancelButtonActionPerformed
    dispose();
  }//GEN-LAST:event_cancelButtonActionPerformed

  private void OKButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_OKButtonActionPerformed
  {//GEN-HEADEREND:event_OKButtonActionPerformed
    Date validFrom = null;
    Date validTo = null;
    try
    {
      DateFormat dateFormat = DateFormat.getInstance();
      String validFromText = validFromTextField.getText().trim().toLowerCase();
      if (validFromText.length() != 0 && validFromText.compareTo("no limitation") != 0) validFrom = dateFormat.parse(validFromText);

      String validToText = validToTextField.getText().trim().toLowerCase();
      if (validToText.length() != 0 && validToText.compareTo("no limitation") != 0) validTo = dateFormat.parse(validToText);
    }
    catch (ParseException e)
    {
      JOptionPane.showMessageDialog(this, "Error occured while parsing 'Valid From' or 'Valid To':\n" + e, "Error", JOptionPane.ERROR_MESSAGE);
      return;
    }

    boolean isPush = pushCheckBox.isSelected();
    boolean isPull = pullCheckBox.isSelected();
    
    int pushCounter = -1;
    int pullCounter = -1;
    
    String pushCounterText = pushCounterTextField.getText().trim();
    String pullCounterText = pullCounterTextField.getText().trim();

    if (isPush && pushCounterCheckBox.isSelected() && pushCounterText.length() != 0)
    {
      try
      {
        pushCounter = Integer.parseInt(pushCounterText);
        if (pushCounter < 0) throw new NumberFormatException("Negative 'Push Counter' is not allowed");
      }
      catch (NumberFormatException e)
      {
        JOptionPane.showMessageDialog(this, "Error occured while parsing 'Push Counter':\n" + e, "Error", JOptionPane.ERROR_MESSAGE);
        return;
      }
    }

    if (isPull && pullCounterCheckBox.isSelected() && pullCounterText.length() != 0)
    {
      try
      {
        pullCounter = Integer.parseInt(pullCounterText);
        if (pullCounter < 0) throw new NumberFormatException("Negative 'Pull Counter' is not allowed");
      }
      catch (NumberFormatException e)
      {
        JOptionPane.showMessageDialog(this, "Error occured while parsing 'Pull Counter':\n" + e, "Error", JOptionPane.ERROR_MESSAGE);
        return;
      }
    }
    
    if (!pushDestinationNodesCheckBox.isSelected()) pushDestinationNodes = null;
    if (!pushNodeFilterCheckBox.isSelected()) pushNodeFilter = null;
    if (!pullNodeFilterCheckBox.isSelected()) pullNodeFilter = null;
    
    contractRule.setDescription(descriptionTextField.getText());
    contractRule.setSoftware(software);
    contractRule.setLicence(licence);
    contractRule.setValidFrom(validFrom);
    contractRule.setValidTo(validTo);
    contractRule.setAutoDelete(autodeleteCheckBox.isSelected());
    
    if (isPush)
    {
      contractRule.setPush(true);
      contractRule.setPushDestinationNodes(pushDestinationNodes);
      contractRule.setPushNodeFilter(pushNodeFilter);
      contractRule.setPushCounter(pushCounter);
    }
    else
    {
      contractRule.setPush(false);
      contractRule.setPushDestinationNodes(null);
      contractRule.setPushNodeFilter(null);
      contractRule.setPushCounter(-1);
    }

    if (isPull)
    {
      contractRule.setPull(true);
      contractRule.setPullNodeFilter(pullNodeFilter);
      contractRule.setPullCounter(pullCounter);
    }
    else
    {
      contractRule.setPull(false);
      contractRule.setPullNodeFilter(null);
      contractRule.setPullCounter(-1);
    }
    
    isOK = true;
    dispose();
  }//GEN-LAST:event_OKButtonActionPerformed
  
  // Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.JButton OKButton;
  private javax.swing.JPanel actionsPanel;
  private javax.swing.JCheckBox autodeleteCheckBox;
  private javax.swing.JButton cancelButton;
  private javax.swing.JLabel descriptionLabel;
  private javax.swing.JTextField descriptionTextField;
  private javax.swing.JPanel jPanel2;
  private javax.swing.JPanel jPanel3;
  private javax.swing.JPanel jPanel4;
  private javax.swing.JPanel jPanel5;
  private javax.swing.JPanel jPanel6;
  private javax.swing.JPanel jPanel7;
  private javax.swing.JPanel jPanel9;
  private javax.swing.JButton licenceButton;
  private javax.swing.JPanel nameAndDescriptionPanel;
  private javax.swing.JLabel nameLabel;
  private javax.swing.JTextField nameTextField;
  private javax.swing.JPanel parametersPanel;
  private javax.swing.JCheckBox pullCheckBox;
  private javax.swing.JCheckBox pullCounterCheckBox;
  private javax.swing.JPanel pullCounterPanel;
  private javax.swing.JTextField pullCounterTextField;
  private javax.swing.JButton pullNodeFilterButton;
  private javax.swing.JCheckBox pullNodeFilterCheckBox;
  private javax.swing.JPanel pullNodeFilterPanel;
  private javax.swing.JPanel pullOptionsPanel;
  private javax.swing.JCheckBox pushCheckBox;
  private javax.swing.JCheckBox pushCounterCheckBox;
  private javax.swing.JPanel pushCounterPanel;
  private javax.swing.JTextField pushCounterTextField;
  private javax.swing.JButton pushDestinationNodesButton;
  private javax.swing.JCheckBox pushDestinationNodesCheckBox;
  private javax.swing.JPanel pushDestinationNodesPanel;
  private javax.swing.JButton pushNodeFilterButton;
  private javax.swing.JCheckBox pushNodeFilterCheckBox;
  private javax.swing.JPanel pushNodeFilterPanel;
  private javax.swing.JPanel pushOptionsPanel;
  private javax.swing.JPanel softwareAndLicencePanel;
  private javax.swing.JButton softwareButton;
  private javax.swing.JLabel validFromLabel;
  private javax.swing.JPanel validFromPanel;
  private javax.swing.JTextField validFromTextField;
  private javax.swing.JLabel validToLabel;
  private javax.swing.JPanel validToPanel;
  private javax.swing.JTextField validToTextField;
  // End of variables declaration//GEN-END:variables
  
}
