/*
 * InputTriggerShareActionPropertiesEditDialog.java
 *
 * Created on 17. kvten 2004, 23:32
 */

package SOFA.SOFAnet.Browser;

import SOFA.SOFAnet.Repository.InputTrigger;
import SOFA.SOFAnet.Repository.Licence;
import SOFA.SOFAnet.Repository.NodeNameFilter;
import SOFA.SOFAnet.Repository.ShareGroups;
import javax.swing.*;
import java.util.*;
import java.awt.Component;
import java.awt.Frame;

/**
 *
 * @author  Ladislav Sobr
 */
public class InputTriggerShareActionPropertiesEditDialog extends javax.swing.JDialog
{
  private ActionShareParameters asParameters;
  private NodeNameFilter asNodeFilter;
  private ShareGroups asShareGroups;
  private boolean isOK;
  
  public static class ActionShareParameters
  {
    private int asNoLicences;
    private ShareGroups asShareGroups;
    private NodeNameFilter asNodeFilter;
    private boolean asEquality;
    
    public ActionShareParameters(InputTrigger inputTrigger)
    {
      asNoLicences = inputTrigger.getAsNoLicences();
      asShareGroups = inputTrigger.getAsShareGroups();
      asNodeFilter = inputTrigger.getAsNodeFilter();
      asEquality = inputTrigger.getAsEquality();
    }
    
    public void save(InputTrigger inputTrigger)
    {
      inputTrigger.setAsNoLicences(asNoLicences);
      inputTrigger.setAsShareGroups(asShareGroups);
      inputTrigger.setAsNodeFilter(asNodeFilter);
      inputTrigger.setAsEquality(asEquality);
    }
    
    public int getAsNoLicences()
    {
      return asNoLicences;
    }

    public ShareGroups getAsShareGroups()
    {
      return asShareGroups;
    }

    public NodeNameFilter getAsNodeFilter()
    {
      return asNodeFilter;
    }

    public boolean getAsEquality()
    {
      return asEquality;
    }
    
    public void setAsNoLicences(int asNoLicences)
    {
      if (asNoLicences <= 0 && asNoLicences != Licence.ALL_LICENCES) asNoLicences = Licence.ONE_IMPLICIT_LICENCE;
      this.asNoLicences = asNoLicences;
    }

    public void setAsShareGroups(ShareGroups asShareGroups)
    {
      this.asShareGroups = asShareGroups;
    }

    public void setAsNodeFilter(NodeNameFilter asNodeFilter)
    {
      this.asNodeFilter = asNodeFilter;
    }

    public void setAsEquality(boolean asEquality)
    {
      this.asEquality = asEquality;
    }
    
  }
  
  public static boolean showDialog(Component frameComp, ActionShareParameters asParameters)
  {
    Frame frame = JOptionPane.getFrameForComponent(frameComp);
    InputTriggerShareActionPropertiesEditDialog dialog = new InputTriggerShareActionPropertiesEditDialog(frame, true, asParameters);
    dialog.setLocationRelativeTo(frameComp);
    dialog.setVisible(true);
    return dialog.isOK();
  }
  
  /** Creates new form InputTriggerShareActionPropertiesEditDialog */
  public InputTriggerShareActionPropertiesEditDialog(java.awt.Frame parent, boolean modal, ActionShareParameters asParameters)
  {
    super(parent, modal);
    this.asParameters = asParameters;
    asNodeFilter = asParameters.getAsNodeFilter();
    asShareGroups = asParameters.getAsShareGroups();
    isOK = false;
    
    initComponents();
    
    int numLic = asParameters.getAsNoLicences();
    if (numLic == Licence.ONE_IMPLICIT_LICENCE)
    {
      asNumberOfLicencesCheckBox.setSelected(false);
      asNumberOfLicencesTextField.setText("");
      asNumberOfLicencesTextField.setEnabled(false);
      asNumberOfLicencesLabel.setEnabled(false);
    }
    else
    if (numLic == Licence.ALL_LICENCES)
    {
      asNumberOfLicencesCheckBox.setSelected(true);
      asNumberOfLicencesTextField.setText("*");
      asNumberOfLicencesTextField.setEnabled(true);
      asNumberOfLicencesLabel.setEnabled(true);
    }
    else
    {
      asNumberOfLicencesCheckBox.setSelected(true);
      asNumberOfLicencesTextField.setText(Integer.toString(numLic));
      asNumberOfLicencesTextField.setEnabled(true);
      asNumberOfLicencesLabel.setEnabled(true);
    }
    
    if (asNodeFilter != null)
    {
      asNodeFilterCheckBox.setSelected(true);
      asNodeFilterButton.setEnabled(true);
    }
    else
    {
      asNodeFilterCheckBox.setSelected(false);
      asNodeFilterButton.setEnabled(false);
    }
    
    if (asShareGroups != null)
    {
      asShareGroupsCheckBox.setSelected(true);
      asShareGroupsButton.setEnabled(true);
    }
    else
    {
      asShareGroupsCheckBox.setSelected(false);
      asShareGroupsButton.setEnabled(false);
    }
    
    asEqualityCheckBox.setSelected(asParameters.getAsEquality());

    if (asNodeFilter != null) asNodeFilter = (NodeNameFilter)asNodeFilter.clone();
    else asNodeFilter = new NodeNameFilter();
    if (asShareGroups != null) asShareGroups = (ShareGroups)asShareGroups.clone();
    else asShareGroups = new ShareGroups();
  }

  public boolean isOK()
  {
    return isOK;
  }
  
  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the Form Editor.
   */
  private void initComponents()//GEN-BEGIN:initComponents
  {
    jPanel1 = new javax.swing.JPanel();
    asNumberOfLicencesPanel = new javax.swing.JPanel();
    asNumberOfLicencesCheckBox = new javax.swing.JCheckBox();
    asNumberOfLicencesTextField = new javax.swing.JTextField();
    asNumberOfLicencesLabel = new javax.swing.JLabel();
    asShareGroupsPanel = new javax.swing.JPanel();
    asShareGroupsCheckBox = new javax.swing.JCheckBox();
    asShareGroupsButton = new javax.swing.JButton();
    asNodeFilterPanel = new javax.swing.JPanel();
    asNodeFilterCheckBox = new javax.swing.JCheckBox();
    asNodeFilterButton = new javax.swing.JButton();
    asEqualityCheckBox = new javax.swing.JCheckBox();
    jPanel2 = new javax.swing.JPanel();
    OKButton = new javax.swing.JButton();
    cancelButton = new javax.swing.JButton();

    setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
    setTitle("Share Action Properties");
    jPanel1.setLayout(new java.awt.GridLayout(4, 0));

    jPanel1.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(5, 5, 5, 5)));
    asNumberOfLicencesPanel.setLayout(new javax.swing.BoxLayout(asNumberOfLicencesPanel, javax.swing.BoxLayout.X_AXIS));

    asNumberOfLicencesCheckBox.setText("Number of Licences");
    asNumberOfLicencesCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        asNumberOfLicencesCheckBoxActionPerformed(evt);
      }
    });

    asNumberOfLicencesPanel.add(asNumberOfLicencesCheckBox);

    asNumberOfLicencesPanel.add(asNumberOfLicencesTextField);

    asNumberOfLicencesLabel.setText("(use '*' or 'all' for all licences)");
    asNumberOfLicencesPanel.add(asNumberOfLicencesLabel);

    jPanel1.add(asNumberOfLicencesPanel);

    asShareGroupsPanel.setLayout(new javax.swing.BoxLayout(asShareGroupsPanel, javax.swing.BoxLayout.X_AXIS));

    asShareGroupsCheckBox.setText("Share Groups");
    asShareGroupsCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        asShareGroupsCheckBoxActionPerformed(evt);
      }
    });

    asShareGroupsPanel.add(asShareGroupsCheckBox);

    asShareGroupsButton.setText("Edit Share Groups");
    asShareGroupsButton.setEnabled(false);
    asShareGroupsButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        asShareGroupsButtonActionPerformed(evt);
      }
    });

    asShareGroupsPanel.add(asShareGroupsButton);

    jPanel1.add(asShareGroupsPanel);

    asNodeFilterPanel.setLayout(new javax.swing.BoxLayout(asNodeFilterPanel, javax.swing.BoxLayout.X_AXIS));

    asNodeFilterCheckBox.setText("Node Filter");
    asNodeFilterCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        asNodeFilterCheckBoxActionPerformed(evt);
      }
    });

    asNodeFilterPanel.add(asNodeFilterCheckBox);

    asNodeFilterButton.setText("Edit Node Filter");
    asNodeFilterButton.setEnabled(false);
    asNodeFilterButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        asNodeFilterButtonActionPerformed(evt);
      }
    });

    asNodeFilterPanel.add(asNodeFilterButton);

    jPanel1.add(asNodeFilterPanel);

    asEqualityCheckBox.setSelected(true);
    asEqualityCheckBox.setText("Equality of Manager and Clients");
    jPanel1.add(asEqualityCheckBox);

    getContentPane().add(jPanel1, java.awt.BorderLayout.NORTH);

    jPanel2.setMinimumSize(new java.awt.Dimension(200, 35));
    jPanel2.setPreferredSize(new java.awt.Dimension(400, 35));
    OKButton.setText("OK");
    OKButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        OKButtonActionPerformed(evt);
      }
    });

    jPanel2.add(OKButton);

    cancelButton.setText("Cancel");
    cancelButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        cancelButtonActionPerformed(evt);
      }
    });

    jPanel2.add(cancelButton);

    getContentPane().add(jPanel2, java.awt.BorderLayout.SOUTH);

    pack();
  }//GEN-END:initComponents

  private void asNumberOfLicencesCheckBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_asNumberOfLicencesCheckBoxActionPerformed
  {//GEN-HEADEREND:event_asNumberOfLicencesCheckBoxActionPerformed
    asNumberOfLicencesTextField.setEnabled(asNumberOfLicencesCheckBox.isSelected());
    asNumberOfLicencesLabel.setEnabled(asNumberOfLicencesCheckBox.isSelected());    
  }//GEN-LAST:event_asNumberOfLicencesCheckBoxActionPerformed

  private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_cancelButtonActionPerformed
  {//GEN-HEADEREND:event_cancelButtonActionPerformed
    dispose();
  }//GEN-LAST:event_cancelButtonActionPerformed

  private void OKButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_OKButtonActionPerformed
  {//GEN-HEADEREND:event_OKButtonActionPerformed
    boolean asNumberOfLicencesSelected = asNumberOfLicencesCheckBox.isSelected();
    String asNumberOfLicencesText = asNumberOfLicencesTextField.getText().trim().toLowerCase();
    boolean asEquality = asEqualityCheckBox.isSelected();
    
    int asNumberOfLicences = Licence.ONE_IMPLICIT_LICENCE;
    if (asNumberOfLicencesSelected && asNumberOfLicencesText.length() != 0)
    {
      if (asNumberOfLicencesText.compareTo("*") == 0 || asNumberOfLicencesText.compareTo("all") == 0) asNumberOfLicences = Licence.ALL_LICENCES;
      else
      {
        try
        {
          asNumberOfLicences = Integer.parseInt(asNumberOfLicencesText);
          if (asNumberOfLicences < 0) throw new NumberFormatException("Negative 'Number of Licences' is not allowed");
        }
        catch (NumberFormatException e)
        {
          JOptionPane.showMessageDialog(this, "Error occured while parsing 'Number of Licences':\n" + e, "Error", JOptionPane.ERROR_MESSAGE);
          return;
        }
      }
    }

    if (!asNodeFilterCheckBox.isSelected()) asNodeFilter = null;
    if (!asShareGroupsCheckBox.isSelected()) asShareGroups = null;
    
    asParameters.setAsNoLicences(asNumberOfLicences);
    asParameters.setAsNodeFilter(asNodeFilter);
    asParameters.setAsShareGroups(asShareGroups);
    asParameters.setAsEquality(asEquality);
    
    isOK = true;
    dispose();
  }//GEN-LAST:event_OKButtonActionPerformed

  private void asNodeFilterButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_asNodeFilterButtonActionPerformed
  {//GEN-HEADEREND:event_asNodeFilterButtonActionPerformed
    StringListEditDialog.showDialog(this, asNodeFilter.getPatterns(), "Node Name Pattern", "Node Filter Editor");
  }//GEN-LAST:event_asNodeFilterButtonActionPerformed

  private void asShareGroupsButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_asShareGroupsButtonActionPerformed
  {//GEN-HEADEREND:event_asShareGroupsButtonActionPerformed
    StringListEditDialog.showDialog(this, asShareGroups.getList(), "Share Group", "Share Groups Editor");
  }//GEN-LAST:event_asShareGroupsButtonActionPerformed

  private void asNodeFilterCheckBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_asNodeFilterCheckBoxActionPerformed
  {//GEN-HEADEREND:event_asNodeFilterCheckBoxActionPerformed
    asNodeFilterButton.setEnabled(asNodeFilterCheckBox.isSelected());
  }//GEN-LAST:event_asNodeFilterCheckBoxActionPerformed

  private void asShareGroupsCheckBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_asShareGroupsCheckBoxActionPerformed
  {//GEN-HEADEREND:event_asShareGroupsCheckBoxActionPerformed
    asShareGroupsButton.setEnabled(asShareGroupsCheckBox.isSelected());
  }//GEN-LAST:event_asShareGroupsCheckBoxActionPerformed
   
  // Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.JButton OKButton;
  private javax.swing.JCheckBox asEqualityCheckBox;
  private javax.swing.JButton asNodeFilterButton;
  private javax.swing.JCheckBox asNodeFilterCheckBox;
  private javax.swing.JPanel asNodeFilterPanel;
  private javax.swing.JCheckBox asNumberOfLicencesCheckBox;
  private javax.swing.JLabel asNumberOfLicencesLabel;
  private javax.swing.JPanel asNumberOfLicencesPanel;
  private javax.swing.JTextField asNumberOfLicencesTextField;
  private javax.swing.JButton asShareGroupsButton;
  private javax.swing.JCheckBox asShareGroupsCheckBox;
  private javax.swing.JPanel asShareGroupsPanel;
  private javax.swing.JButton cancelButton;
  private javax.swing.JPanel jPanel1;
  private javax.swing.JPanel jPanel2;
  // End of variables declaration//GEN-END:variables
  
}
