/*
 * NetNodeBrowser.java
 *
 * Created on 10. kvten 2004, 17:07
 */

package SOFA.SOFAnet.Browser;

import javax.swing.*;
import java.awt.*;

import SOFA.SOFAnet.Admin.RMI.RMIAdminInterface;

import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.UnknownHostException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import SOFA.SOFAnet.Repository.NodeInfo;
import SOFA.Util.VMProperties;

/**
 *
 * @author  Ladislav Sobr
 */
public class NetNodeBrowser extends JFrame
{
  private RMIAdminInterface admin;
  private String nodeName;
  
  private JPanel jBinLicPanel;
  private JPanel jOpsPanel;
  private JPanel jOffersPanel;
  private JPanel jLocalInfosPanel;
  private JPanel jTRPanel;
  private JPanel jInputTriggersPanel;
  private JPanel jOutputTriggersPanel;
  private JPanel jContractsPanel;
  private JTabbedPane jTabbedPane;
  
  /** Creates new form NetNodeBrowser */
  public NetNodeBrowser(String nodeName)
  {
    this.nodeName = nodeName;
    
    initNodeConnection();
    initComponents();
  }
  
  private void initNodeConnection()
  {
    nodeName = nodeName.trim();
    if (!nodeName.toLowerCase().startsWith("sofa://")) nodeName = "sofa://" + nodeName;

    NodeInfo nodeInfo = new NodeInfo();
    try
    {
      nodeInfo.setNodeName(nodeName);
    }
    catch (NodeInfo.InvalidNodeNameException e)
    {
      JOptionPane.showMessageDialog(this, "Invalid name of SOFAnet node.\n" + e, "Error", JOptionPane.ERROR_MESSAGE);
      System.exit(1);
    }
    
    try
    {
      admin = (RMIAdminInterface)Naming.lookup("rmi://" + nodeInfo.getAddressAndPort() + "/SOFAnet/RMIAdminServer");
    }
    catch (NotBoundException e)
    {
      JOptionPane.showMessageDialog(this, "Cannot connect to SOFAnet node.\n" + e, "Error", JOptionPane.ERROR_MESSAGE);
      System.exit(1);
    }
    catch (MalformedURLException e)
    {
      JOptionPane.showMessageDialog(this, "Cannot connect to SOFAnet node.\n" + e, "Error", JOptionPane.ERROR_MESSAGE);
      System.exit(1);
    }
    catch (RemoteException e)
    {
      JOptionPane.showMessageDialog(this, "Cannot connect to SOFAnet node.\n" + e, "Error", JOptionPane.ERROR_MESSAGE);
      System.exit(1);
    }    
  }
  
  private void initComponents()
  {
    jTabbedPane = new JTabbedPane();
    jOpsPanel = new OperationsPanel(admin);
    jBinLicPanel = new BinBundlesAndLicencesPanel(admin);
    jOffersPanel = new BundleOffersPanel(admin);
    jLocalInfosPanel = new LocalInfosPanel(admin);
    jTRPanel = new TRPanel(admin);
    jInputTriggersPanel = new InputTriggersPanel(admin);
    jOutputTriggersPanel = new OutputTriggersPanel(admin);
    jContractsPanel = new ContractsPanel(admin);

    setTitle("SOFAnet Node Browser [" + nodeName + "]");
    
    addWindowListener(new java.awt.event.WindowAdapter()
    {
      public void windowClosing(java.awt.event.WindowEvent evt)
      {
        exitForm(evt);
      }
    });

    jTabbedPane.addTab("Operations", jOpsPanel);
    jTabbedPane.addTab("Binary Bundles and Licences", jBinLicPanel);
    jTabbedPane.addTab("Bundle Offers", jOffersPanel);
    jTabbedPane.addTab("Local Infos", jLocalInfosPanel);
    jTabbedPane.addTab("TR", jTRPanel);
    jTabbedPane.addTab("Input Triggers", jInputTriggersPanel);
    jTabbedPane.addTab("Output Triggers", jOutputTriggersPanel);
    jTabbedPane.addTab("Contracts", jContractsPanel);

    getContentPane().add(jTabbedPane, java.awt.BorderLayout.CENTER);

    pack();
    
    setLocationRelativeTo(null);
  }
  
  
  /** Exit the Application */
  private void exitForm(java.awt.event.WindowEvent evt)
  {
    System.exit(0);
  }
  
  /**
   * @param args the command line arguments
   */
  public static void main(String args[])
  {
    String nodeName = "";
    if (args.length == 0 || args[0].trim().length() == 0)
    {
      try
      {
        nodeName = InetAddress.getLocalHost().getHostName();
      }
      catch (UnknownHostException e)
      {
        System.err.println("Missing name of SOFAnet node and can't get the host name.");
        System.exit(1);
      }
      
      String rmiPort = System.getProperty(VMProperties.RMI_PORT, null);
      
      if (rmiPort != null) nodeName += ":" + rmiPort;
      System.out.println("Missing name of SOFAnet node, using '" + nodeName + "'");
    }
    else nodeName = args[0];
    
    try
    {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    }
    catch (Exception e)
    {
    }
    
    new NetNodeBrowser(nodeName).show();
  }
  
}
