/*
 * RMISearchConfigurationConnectionEditDialog.java
 *
 * Created on 24. kvten 2004, 11:39
 */

package SOFA.SOFAnet.Browser;

import javax.swing.*;
import SOFA.SOFAnet.Repository.ShareGroups;
import SOFA.SOFAnet.Repository.NodeInfo;
import SOFA.SOFAnet.Search.RMI.RMISearchConfiguration;
import java.util.*;
import java.awt.Component;
import java.awt.Frame;

/**
 *
 * @author  Ladislav Sobr
 */
public class RMISearchConfigurationConnectionEditDialog extends javax.swing.JDialog
{
  private RMISearchConfiguration.Connection connection;
  private ShareGroups shareGroups;
  private boolean isOK;

  public static boolean showDialog(Component frameComp, RMISearchConfiguration.Connection connection)
  {
    Frame frame = JOptionPane.getFrameForComponent(frameComp);
    RMISearchConfigurationConnectionEditDialog dialog = new RMISearchConfigurationConnectionEditDialog(frame, true, connection);
    dialog.setLocationRelativeTo(frameComp);
    dialog.setVisible(true);
    return dialog.isOK();
  }
  
  /** Creates new form RMISearchConfigurationConnectionEditDialog */
  public RMISearchConfigurationConnectionEditDialog(java.awt.Frame parent, boolean modal, RMISearchConfiguration.Connection connection)
  {
    super(parent, modal);
    this.connection = connection;
    shareGroups = connection.getShareGroups();
    isOK = false;
    initComponents();

    nodeNameTextField.setText(connection.getNodeName());
    
    if (shareGroups != null)
    {
      shareGroupsCheckBox.setSelected(true);
      shareGroupsButton.setEnabled(true);
    }
    else
    {
      shareGroupsCheckBox.setSelected(false);
      shareGroupsButton.setEnabled(false);
    }
    
    if (shareGroups != null) shareGroups = (ShareGroups)shareGroups.clone();
    else shareGroups = new ShareGroups();
  }
  
  public boolean isOK()
  {
    return isOK;
  }
  
  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the Form Editor.
   */
  private void initComponents()//GEN-BEGIN:initComponents
  {
    jPanel1 = new javax.swing.JPanel();
    nodeNameLabel = new javax.swing.JLabel();
    nodeNameTextField = new javax.swing.JTextField();
    jPanel7 = new javax.swing.JPanel();
    shareGroupsCheckBox = new javax.swing.JCheckBox();
    shareGroupsButton = new javax.swing.JButton();
    jPanel3 = new javax.swing.JPanel();
    OKButton = new javax.swing.JButton();
    cancelButton = new javax.swing.JButton();

    setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
    setTitle("SearchNet Connection");
    jPanel1.setLayout(new java.awt.GridLayout(3, 0));

    jPanel1.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(5, 5, 5, 5)));
    nodeNameLabel.setText("Node Name:");
    jPanel1.add(nodeNameLabel);

    jPanel1.add(nodeNameTextField);

    jPanel7.setLayout(new javax.swing.BoxLayout(jPanel7, javax.swing.BoxLayout.X_AXIS));

    shareGroupsCheckBox.setText("Share Groups");
    shareGroupsCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        shareGroupsCheckBoxActionPerformed(evt);
      }
    });

    jPanel7.add(shareGroupsCheckBox);

    shareGroupsButton.setText("Edit Share Groups");
    shareGroupsButton.setEnabled(false);
    shareGroupsButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        shareGroupsButtonActionPerformed(evt);
      }
    });

    jPanel7.add(shareGroupsButton);

    jPanel1.add(jPanel7);

    getContentPane().add(jPanel1, java.awt.BorderLayout.NORTH);

    jPanel3.setPreferredSize(new java.awt.Dimension(300, 35));
    OKButton.setText("OK");
    OKButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        OKButtonActionPerformed(evt);
      }
    });

    jPanel3.add(OKButton);

    cancelButton.setText("Cancel");
    cancelButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        cancelButtonActionPerformed(evt);
      }
    });

    jPanel3.add(cancelButton);

    getContentPane().add(jPanel3, java.awt.BorderLayout.SOUTH);

    pack();
  }//GEN-END:initComponents

  private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_cancelButtonActionPerformed
  {//GEN-HEADEREND:event_cancelButtonActionPerformed
    dispose();
  }//GEN-LAST:event_cancelButtonActionPerformed

  private void OKButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_OKButtonActionPerformed
  {//GEN-HEADEREND:event_OKButtonActionPerformed
    String nodeName = nodeNameTextField.getText().trim();
    
    if (nodeName.length() == 0)
    {
      JOptionPane.showMessageDialog(this, "Empty node name", "Error", JOptionPane.ERROR_MESSAGE);
      return;
    }
    
    if (!shareGroupsCheckBox.isSelected()) shareGroups = null;
    
    connection.setNodeName(nodeName);
    connection.setShareGroups(shareGroups);
    
    isOK = true;
    dispose();
  }//GEN-LAST:event_OKButtonActionPerformed

  private void shareGroupsButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_shareGroupsButtonActionPerformed
  {//GEN-HEADEREND:event_shareGroupsButtonActionPerformed
    StringListEditDialog.showDialog(this, shareGroups.getList(), "Share Group", "Share Groups Editor");
  }//GEN-LAST:event_shareGroupsButtonActionPerformed

  private void shareGroupsCheckBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_shareGroupsCheckBoxActionPerformed
  {//GEN-HEADEREND:event_shareGroupsCheckBoxActionPerformed
    shareGroupsButton.setEnabled(shareGroupsCheckBox.isSelected());
  }//GEN-LAST:event_shareGroupsCheckBoxActionPerformed
  
  // Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.JButton OKButton;
  private javax.swing.JButton cancelButton;
  private javax.swing.JPanel jPanel1;
  private javax.swing.JPanel jPanel3;
  private javax.swing.JPanel jPanel7;
  private javax.swing.JLabel nodeNameLabel;
  private javax.swing.JTextField nodeNameTextField;
  private javax.swing.JButton shareGroupsButton;
  private javax.swing.JCheckBox shareGroupsCheckBox;
  // End of variables declaration//GEN-END:variables
  
}
