/*
 * TRPanel.java
 *
 * Created on 17. kvten 2004, 17:23
 */

package SOFA.SOFAnet.Browser;

import javax.swing.*;
import SOFA.SOFAnet.Admin.RMI.*;
import SOFA.SOFAnode.TR.ComponentInfo;
import SOFA.SOFAnode.TR.Impl.ComponentInfoImpl;
import java.rmi.RemoteException;
import java.util.Arrays;

/**
 *
 * @author  Ladislav Sobr
 */
public class TRPanel extends javax.swing.JPanel
{
  private RMIAdminInterface admin;
  private DefaultListModel content;
  
  private static class CompInfo implements Comparable
  {
    public ComponentInfo componentInfo;
    public String componentName;
    
    CompInfo(ComponentInfo componentInfo)
    {
      this.componentInfo = componentInfo;
      this.componentName = componentInfo.getName() + "[" + componentInfo.getImplementationVersion() + "]";
    }
    
    public boolean equals(Object o)
    {
      if (this == o) return true;
      if (o instanceof CompInfo)
      {
        CompInfo compInfo = (CompInfo)o;
        return componentName.compareTo(compInfo.componentName) == 0;
      }
      else return false;
    }

    public int compareTo(Object o)
    {
      CompInfo compInfo = (CompInfo)o;
      return componentName.compareTo(compInfo.componentName);
    }
    
    public String toString()
    {
      return componentName;
    }
  }
  
  /** Creates new form TRPanel */
  public TRPanel(RMIAdminInterface admin)
  {
    this.admin = admin;
    content = new DefaultListModel();
    initComponents();
  }
  
  private void reload()
  {
    content.clear();
    
    ComponentInfo[] componentInfos = null;
    try
    {
      componentInfos = admin.componentsInTRList();
    }
    catch (RemoteException e)
    {
      JOptionPane.showMessageDialog(this, "RMI Error occured while accessing SOFA net node:\n" + e, "Error", JOptionPane.ERROR_MESSAGE);
      return;
    }
    
    CompInfo[] compInfos = new CompInfo[componentInfos.length];
    for (int i = 0; i < compInfos.length; i++) compInfos[i] = new CompInfo(componentInfos[i]);
    Arrays.sort(compInfos);
    
    content.ensureCapacity(compInfos.length);
    for (int i = 0; i < compInfos.length; i++) content.addElement(compInfos[i]);
  }
  
  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the Form Editor.
   */
  private void initComponents()//GEN-BEGIN:initComponents
  {
    jPanel2 = new javax.swing.JPanel();
    jPanel4 = new javax.swing.JPanel();
    createUserBundleButton = new javax.swing.JButton();
    createComponentBundleButton = new javax.swing.JButton();
    deleteButton = new javax.swing.JButton();
    jPanel1 = new javax.swing.JPanel();
    jPanel3 = new javax.swing.JPanel();
    refreshButton = new javax.swing.JButton();
    jScrollPane1 = new javax.swing.JScrollPane();
    contentList = new javax.swing.JList();

    setLayout(new java.awt.BorderLayout());

    addComponentListener(new java.awt.event.ComponentAdapter()
    {
      public void componentShown(java.awt.event.ComponentEvent evt)
      {
        formComponentShown(evt);
      }
    });

    jPanel2.setLayout(new java.awt.BorderLayout());

    jPanel2.setPreferredSize(new java.awt.Dimension(200, 154));
    jPanel4.setLayout(new java.awt.GridLayout(3, 0));

    jPanel4.setBorder(new javax.swing.border.TitledBorder("Components Specific"));
    createUserBundleButton.setText("Create User Bundle");
    createUserBundleButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        createUserBundleButtonActionPerformed(evt);
      }
    });

    jPanel4.add(createUserBundleButton);

    createComponentBundleButton.setText("Create Component Bundle");
    createComponentBundleButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        createComponentBundleButtonActionPerformed(evt);
      }
    });

    jPanel4.add(createComponentBundleButton);

    deleteButton.setText("Delete Components");
    deleteButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        deleteButtonActionPerformed(evt);
      }
    });

    jPanel4.add(deleteButton);

    jPanel2.add(jPanel4, java.awt.BorderLayout.NORTH);

    jPanel1.setLayout(new java.awt.BorderLayout());

    jPanel1.setBorder(new javax.swing.border.TitledBorder("General"));
    jPanel3.setLayout(new java.awt.GridLayout(1, 0));

    refreshButton.setText("Refresh");
    refreshButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        refreshButtonActionPerformed(evt);
      }
    });

    jPanel3.add(refreshButton);

    jPanel1.add(jPanel3, java.awt.BorderLayout.NORTH);

    jPanel2.add(jPanel1, java.awt.BorderLayout.CENTER);

    add(jPanel2, java.awt.BorderLayout.EAST);

    contentList.setModel(content);
    contentList.setVisibleRowCount(-1);
    jScrollPane1.setViewportView(contentList);

    add(jScrollPane1, java.awt.BorderLayout.CENTER);

  }//GEN-END:initComponents

  private void refreshButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_refreshButtonActionPerformed
  {//GEN-HEADEREND:event_refreshButtonActionPerformed
    reload();
  }//GEN-LAST:event_refreshButtonActionPerformed

  private void deleteButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_deleteButtonActionPerformed
  {//GEN-HEADEREND:event_deleteButtonActionPerformed
    Object[] selectedCompInfos = contentList.getSelectedValues();
    ComponentInfo[] componentInfos = null;
    if (selectedCompInfos != null)
    {
      componentInfos = new ComponentInfo[selectedCompInfos.length];
      for (int i = 0; i < selectedCompInfos.length; i++) componentInfos[i] = ((CompInfo)selectedCompInfos[i]).componentInfo;
      if (DeleteComponentsDialog.showDialog(this, admin, componentInfos)) reload();
    }    
  }//GEN-LAST:event_deleteButtonActionPerformed

  private void createComponentBundleButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_createComponentBundleButtonActionPerformed
  {//GEN-HEADEREND:event_createComponentBundleButtonActionPerformed
    CompInfo selectedCompInfo = (CompInfo)contentList.getSelectedValue();
    String componentName = "";
    if (selectedCompInfo != null) componentName = selectedCompInfo.componentName;
    CreateComponentBundleDialog.showDialog(this, admin, componentName);
  }//GEN-LAST:event_createComponentBundleButtonActionPerformed

  private void createUserBundleButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_createUserBundleButtonActionPerformed
  {//GEN-HEADEREND:event_createUserBundleButtonActionPerformed
    Object[] selectedCompInfos = contentList.getSelectedValues();
    ComponentInfo[] componentInfos = null;
    if (selectedCompInfos != null)
    {
      componentInfos = new ComponentInfo[selectedCompInfos.length];
      for (int i = 0; i < selectedCompInfos.length; i++) componentInfos[i] = ((CompInfo)selectedCompInfos[i]).componentInfo;
    }
    CreateUserBundleDialog.showDialog(this, admin, "", componentInfos);
  }//GEN-LAST:event_createUserBundleButtonActionPerformed

  private void formComponentShown(java.awt.event.ComponentEvent evt)//GEN-FIRST:event_formComponentShown
  {//GEN-HEADEREND:event_formComponentShown
    reload();
  }//GEN-LAST:event_formComponentShown
  
  
  // Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.JList contentList;
  private javax.swing.JButton createComponentBundleButton;
  private javax.swing.JButton createUserBundleButton;
  private javax.swing.JButton deleteButton;
  private javax.swing.JPanel jPanel1;
  private javax.swing.JPanel jPanel2;
  private javax.swing.JPanel jPanel3;
  private javax.swing.JPanel jPanel4;
  private javax.swing.JScrollPane jScrollPane1;
  private javax.swing.JButton refreshButton;
  // End of variables declaration//GEN-END:variables
  
}
