/*
 * Reporter.java
 *
 * Created on 30. duben 2004, 18:27
 */

package SOFA.SOFAnet.Core;

import java.io.*;

/**
 * This "static" class implements info/warning/error output of SOFAnet
 * @author  Ladislav Sobr
 */
public class Reporter
{
  private static PrintStream errorStream = System.err;
  private static PrintStream warningStream = System.err;
  private static PrintStream infoStream = System.out;
  private static boolean showErrors = true;
  private static boolean showWarnings = true;
  private static boolean showInfos = true;
  private static boolean showDetailedExceptions = false;
 
  /** Do not allow instances of this class */
  private Reporter()
  {
  }

  public static void setErrorStream(PrintStream printStream)
  {
    errorStream = printStream;
  }
  
  public static void setErrorStream(OutputStream outputStream)
  {
    errorStream = new PrintStream(outputStream);
  }
  
  public static void setWarningStream(PrintStream printStream)
  {
    warningStream = printStream;
  }
  
  public static void setWarningStream(OutputStream outputStream)
  {
    warningStream = new PrintStream(outputStream);
  }
  
  public static void setInfoStream(PrintStream printStream)
  {
    infoStream = printStream;
  }
  
  public static void setInfoStream(OutputStream outputStream)
  {
    infoStream = new PrintStream(outputStream);
  }
  
  public static void error(String message)
  {
    printErrorWarning(true, message, null);
  }
  
  public static void error(String message, Throwable cause)
  {
    printErrorWarning(true, message, cause);
  }

  public static void error(Throwable cause)
  {
    printErrorWarning(true, null, cause);
  }
  
  public static void warning(String message)
  {
    printErrorWarning(false, message, null);
  }
  
  public static void warning(String message, Throwable cause)
  {
    printErrorWarning(false, message, cause);
  }

  public static void warning(Throwable cause)
  {
    printErrorWarning(false, null, cause);
  }
  
  public static void info(String message)
  {
    printInfo(message);
  }
  
  public static void startInfo(String message)
  {
    printInfo("+" + message);
  }
  
  public static void stopInfo(String message)
  {
    printInfo("-" + message);
  }

  public static void setShowErrors(boolean showErrors)
  {
    Reporter.showErrors = showErrors;
  }
  
  public static void setShowWarnings(boolean showWarnings)
  {
    Reporter.showWarnings = showWarnings;
  }
  
  public static void setShowInfos(boolean showInfos)
  {
    Reporter.showInfos = showInfos;
  }
  
  public static void setShowDetailedExceptions(boolean showDetailedExceptions)
  {
    Reporter.showDetailedExceptions = showDetailedExceptions;
  }
  
  private static void printErrorWarning(boolean error, String message, Throwable cause)
  {
    String header;
    PrintStream stream;
    if (error)
    {
      if (!showErrors) return;
      stream = errorStream;
      header = "Error";
    }
    else
    {
      if (!showWarnings) return;
      stream = warningStream;
      header = "Warning";
    }
    
    if (message != null)
    {
      stream.println(header + ": " + message);
      
      if (cause != null)
      {
        stream.println("  cause(exc): " + cause.toString());
        if (showDetailedExceptions)
        {
          stream.println("++++++++");
          cause.printStackTrace(errorStream);
          stream.println("--------");
        }
      }
    }
    else
    {
      if (cause != null)
      {
        stream.println(header + "(exc.): " + cause.toString());
        if (showDetailedExceptions)
        {
          stream.println("++++++++");
          cause.printStackTrace(errorStream);
          stream.println("--------");
        }
      }
    }
  }
  
  private static void printInfo(String message)
  {
    if (!showInfos) return;
    
    if (message != null) infoStream.println(message);
  }
}
