/*
 * ReturnSharedAcceptanceTester.java
 *
 * Created on 21. duben 2004, 19:00
 */

package SOFA.SOFAnet.Core;

import SOFA.SOFAnet.Repository.NodeNameList;
import SOFA.SOFAnet.Transport.*;
import java.lang.Object;
import java.util.*;

/**
 * The object of this class is used to test which share client can return it's licence to the share manager.
 * The inquiring of the share clients is done paralelly.
 *
 * @author  Ladislav Sobr
 */
class ReturnSharedAcceptanceTester
{
  private final String bundleName;
  private final NodeNameList clients;
  private final TransportInterface transport;
  
  private Object lock;
  private int counter;
  private final int num;
  private int result;

  private class OneNodeTester extends Thread
  {
    private String nodeName;
    private int index;
    
    OneNodeTester(String nodeName, int index)
    {
      this.nodeName = nodeName;
      this.index = index;
    }

    public void run()
    {
      IOParams ioParams = new IOParams();
      ioParams.setBundleName(bundleName);
      ioParams.setNodeName(nodeName);
      
      boolean can = false;
      try
      {
        transport.canReturnShared(ioParams);
        if (ioParams.getErrCode() == 0) can = true;
      }
      catch (TransportException e)
      {
        can = false;
        Reporter.warning("Accessing to SOFAnode '" + nodeName + "' failed", e);
      }
      
      synchronized (lock)
      {
        counter++;
        if (can) result = index;
        if (can || counter == num) lock.notify();
      }
    }
  }
  
  /** Creates a new instance of ReturnSharedAcceptanceTester
   *
   * @param bundleName name of bundle
   * @param clients list of clients that should be asked
   * @param transport interface of transport layer
   */
  public ReturnSharedAcceptanceTester(String bundleName, NodeNameList clients, TransportInterface transport)
  {
    this.bundleName = bundleName;
    this.clients = clients;
    this.transport = transport;
    
    lock = new Object();
    counter = 0;
    num = clients.getList().size();
    result = -1;
  }

  /**
   * Runs tester. Can be called only once per instance!
   * @return index of client (in clients list) who can return shared bundle; -1 if no such client exists
   */
  public int go()
  {
    OneNodeTester[] testers = new OneNodeTester[num];
    
    int i = 0;
    Iterator it = clients.getList().iterator();
    while (it.hasNext())
    {
      String nodeName = (String)it.next();
      
      testers[i] = new OneNodeTester(nodeName, i);
      i++;
    }
    
    synchronized (lock)
    {
      for (i = 0; i < num; i++) testers[i].start();
      try
      {
        lock.wait(10000);     //10 second timeout
      }
      catch (InterruptedException e)
      {
      }
      
      return result;
    }
  }
  
}
