/*
 * RMILocalCenter.java
 *
 * Created on 3. kvten 2004, 16:33
 */

package SOFA.SOFAnet.Local.RMI;

import SOFA.SOFAnet.Core.LocalOps;
import SOFA.SOFAnet.Repository.NodeInfo;
import java.rmi.RemoteException;
import java.rmi.Naming;
import java.net.MalformedURLException;

/**
 * Managing class of RMI implementation of "local interface" of the SOFAnet.
 * This interface is used by SOFA runtime.
 *
 * @author  Ladislav Sobr
 */
public class RMILocalCenter
{
  private LocalOps localOps;

  private RMILocalServer server;
  
  boolean started;
  private String myNodeName;
  
  /** Creates a new instance of RMILocalCenter */
  public RMILocalCenter()
  {
    started = false;
    server = null;
    myNodeName = NodeInfo.getLocalNodeName();
  }

  public void setLocalOps(LocalOps localOps)
  {
    this.localOps = localOps;
  }

  /** Starts RMI local subsystem */
  public void start() throws RMILocalException
  {
    started = true;

    //run RMI local server
    
    NodeInfo myNodeInfo = new NodeInfo();
    try
    {
      myNodeInfo.setNodeName(myNodeName);
    }
    catch (NodeInfo.InvalidNodeNameException e)
    {
      throw new RMILocalException("Invalid name of local SOFA node: " + myNodeName, e);
    }
    
    try
    {
      server = new RMILocalServer(localOps);
      Naming.rebind("//" + myNodeInfo.getAddressAndPort() + "/SOFAnet/RMILocalServer", server);
    }
    catch (RemoteException e)
    {
      throw new RMILocalException("RMI local server failed to start", e);
    }
    catch (MalformedURLException e)
    {
      throw new RMILocalException("RMI local server failed to start", e);
    }
  }  

  boolean isStarted()
  {
    return started;
  }
}
