/*
 * Repository.java
 *
 * Created on 19. bezen 2004, 21:12
 */

package SOFA.SOFAnet.Repository;

import java.io.*;
import SOFA.SOFAnet.Core.Reporter;

/**
 * Repository of node of SOFAnet.
 *
 * @author  Ladislav Sobr
 */
public class Repository
{
  private final File infoBaseDir;
  
  private BinBundles binBundles;
  private BundleContents installedBundleContents;
  private InputTriggers inputTriggers;
  private OutputTriggers outputTriggers;
  private Contracts contracts;
  private LocalInfos localInfos;
  private Licences licences;
  private BundleOffers bundleOffers;
  private BinBundles shareClientCache;
  private BundleContents sharedBundleContents;
  
  //REMARK: Do not change the order of the repository-members.
  //The order is important to prevent deadlock,
  //because everybody must lock the repository-members in this order!
  
  
  /**
   * Creates a new instance of Repository
   *
   * @param infoBaseDir directory for general information (must exist)
   * @param binBundlesDir directory for binary bundles (must exist)
   * @param shareClientCacheDir directory for binary bundles in cache of share client (must exist)
   */
  public Repository(File infoBaseDir, File binBundlesDir, File shareClientCacheDir)
  {
    this.infoBaseDir = infoBaseDir;
    
    File installedBundlesDir = new File(infoBaseDir, "installed_bundles");
    File inputTriggersDir = new File(infoBaseDir, "input_triggers");
    File outputTriggersDir = new File(infoBaseDir, "output_triggers");
    File contractsDir = new File(infoBaseDir, "contracts");
    File localInfosDir = new File(infoBaseDir, "local_infos");
    File licencesDir = new File(infoBaseDir, "licences");
    File bundleOffersDir = new File(infoBaseDir, "bundle_offers");
    File sharedBundlesDir = new File(infoBaseDir, "shared_bundles");
    
    if (infoBaseDir.exists() && infoBaseDir.isDirectory())
    {
      if (!installedBundlesDir.exists()) installedBundlesDir.mkdir();
      if (!inputTriggersDir.exists()) inputTriggersDir.mkdir();
      if (!outputTriggersDir.exists()) outputTriggersDir.mkdir();
      if (!contractsDir.exists()) contractsDir.mkdir();
      if (!localInfosDir.exists()) localInfosDir.mkdir();
      if (!licencesDir.exists()) licencesDir.mkdir();
      if (!bundleOffersDir.exists()) bundleOffersDir.mkdir();
      if (!sharedBundlesDir.exists()) sharedBundlesDir.mkdir();
    }
    else
    {
      Reporter.error("Cannot open directory '" + infoBaseDir);
    }
    
    binBundles = new BinBundles(binBundlesDir);
    installedBundleContents = new BundleContents(installedBundlesDir);
    inputTriggers = new InputTriggers(inputTriggersDir);
    outputTriggers = new OutputTriggers(outputTriggersDir);
    contracts = new Contracts(contractsDir);
    localInfos = new LocalInfos(localInfosDir);
    licences = new Licences(licencesDir);
    bundleOffers = new BundleOffers(bundleOffersDir);
    shareClientCache = new BinBundles(shareClientCacheDir);
    sharedBundleContents = new BundleContents(sharedBundlesDir);
  }
  
  public File getInfoBaseDir()
  {
    return infoBaseDir;
  }
    
  public BinBundles getBinBundles()
  {
    return binBundles;
  }

  public BundleContents getInstalledBundleContents()
  {
    return installedBundleContents;
  }
  
  public InputTriggers getInputTriggers()
  {
    return inputTriggers;
  }
  
  public OutputTriggers getOutputTriggers()
  {
    return outputTriggers;
  }
  
  public Contracts getContracts()
  {
    return contracts;
  }
  
  public LocalInfos getLocalInfos()
  {
    return localInfos;
  }
  
  public Licences getLicences()
  {
    return licences;
  }
  
  public BundleOffers getBundleOffers()
  {
    return bundleOffers;
  }
  
  public BinBundles getShareClientCache()
  {
    return shareClientCache;
  }

  public BundleContents getSharedBundleContents()
  {
    return sharedBundleContents;
  }
  
}
