/*
 * ShareGroups.java
 *
 * Created on 18. bezen 2004, 23:45
 */

package SOFA.SOFAnet.Repository;

import java.util.*;
import org.w3c.dom.*;
import org.xml.sax.SAXException;
import java.io.Serializable;

/**
 * Implemention of list of share groups (strings).
 *
 * @author  Ladislav Sobr
 */
public class ShareGroups implements Cloneable, Serializable
{
  private List list;        // empty list means "all groups"
  
  /** Creates a new instance of ShareGroups */
  public ShareGroups()
  {
    list = Collections.synchronizedList(new LinkedList());
  }
  
  public Object clone()
  {
    ShareGroups clone = null;
    try
    { 
      clone = (ShareGroups)super.clone();
    }
    catch (CloneNotSupportedException e)
    { 
      throw new InternalError();
    }
    clone.list = Collections.synchronizedList(new LinkedList());
    clone.list.addAll(list);
    return clone;
  }
  

  public List getList()
  {
    return list;
  }

  public void add(String group)
  {
    list.add(group);
  }

  public void add(ShareGroups shareGroups)
  {
    if (shareGroups != null)
    {
      List li = shareGroups.getList();
      synchronized (li)
      {
        Iterator it = li.iterator();
        while (it.hasNext())
        {
          String group = (String)it.next();
          list.add(group);
        }
      }
    }
  }
  
  public boolean contains(String shareGroup)
  {
    return list.contains(shareGroup);
  }
      
  /**
   * Loads the ShareGroups from the XML DOM tree.
   * <p>
   * XML format:
   * <p>
   * <pre>
   *  *&lt;share_group name="string"/&gt;
   * </pre>
   */
  public void loadFromXML(Element element) throws SAXException
  {
    synchronized (list)
    {
      NodeList nl = element.getChildNodes();
      for (int i = 0; i < nl.getLength(); i++)
      {
        Node node = nl.item(i);
        if (node.getNodeType() == Node.ELEMENT_NODE && node.getNodeName().compareTo("share_group") == 0)
        {
          Element el = (Element)node;
          String name = el.getAttribute("name");
          list.add(name);
        }
      }
    }
  }
  
  /**
   * Saves the ShareGroups to XML DOM tree.
   */
  public void saveToXML(Element element)
  {
    synchronized (list)
    {
      Document doc = element.getOwnerDocument();

      Iterator it = list.iterator();
      while (it.hasNext())
      {
        String name = (String)it.next();

        Element el = doc.createElement("share_group");
        el.setAttribute("name", name);
        element.appendChild(el);
      }
    }
  }
  
}
