/*
 * StorageBase.java
 *
 * Created on 22. listopad 2003, 19:38
 */

package SOFA.SOFAnet.Repository;

import java.lang.*;
import java.io.*;
import java.util.*;
import SOFA.SOFAnet.Core.Reporter;

/**
 * Base class for storages implemented as directories with files.
 *
 * @author  Ladislav Sobr
 */
abstract public class StorageBase
{
  public Map map;
  protected File baseDir;
  protected String extension;
  protected boolean loadOnInit;
  
  
  /**
   * Creates a new instance of StorageBase
   *
   * @param baseDir directory with storage
   * @param extension extension of files that belongs to storage
   * @param loadOnInit if true, the content of all items in repository is loaded on init (otherwise items are created in memory but not fully loaded)
   */
  public StorageBase(File baseDir, String extension, boolean loadOnInit)
  {
    this.baseDir = baseDir;
    this.extension = extension;
    this.loadOnInit = loadOnInit;
    map = Collections.synchronizedMap(new HashMap());
  }
  
  abstract protected StorageItem newItem(String name, File file);
  
  /**
   * Initialization of storage.
   */
  public void init()
  {
    File [] files = baseDir.listFiles();
    if (files == null)
    {
      Reporter.error("Cannot open directory '" + baseDir + "'");
      return;
    }
    
    for (int i = 0; i < files.length; i++)
    {
      if (!files[i].isDirectory())
      {
        String name = files[i].getName();
        if (name.endsWith("." + extension))
        {
          name = name.substring(0, name.length() - extension.length() - 1);
          StorageItem item = newItem(name, files[i].getAbsoluteFile());
          if (item != null)
          {
            boolean toBeDeleted = false;
            if (loadOnInit)
            {
              item.loadFromStorage();
              toBeDeleted = item.toBeDeleted();
            }
            if (toBeDeleted) item.deleteFromStorage();
            else map.put(item.getName(), item);
          }
        }
      }
    }
  }

}
