/*
 * XMLStorageItem.java
 *
 * Created on 1. prosinec 2003, 21:21
 */

package SOFA.SOFAnet.Repository;

import java.io.*;
import java.util.*;
import SOFA.Util.XML;
import org.w3c.dom.*;
import org.xml.sax.SAXException;
import SOFA.SOFAnet.Core.Reporter;

/**
 * Implementation of storage item (StorageItem interface) as XML file.
 *
 * @author  Ladislav Sobr
 */
public abstract class XMLStorageItem implements StorageItem, Serializable
{
  protected String name;
  protected File file;
  protected boolean loaded;
  protected String rootTag;
  
  /** Creates a new instance of XMLStorageItem */
  public XMLStorageItem(String name, File file, String rootTag)
  {
    loaded = false;
    this.name = name;
    this.file = file;
    this.rootTag = rootTag;
  }
  
  public boolean isLoaded()
  {
    return loaded;
  }
  
  public void setLoaded(boolean loaded)
  {
    if (this.loaded != loaded)
    {
      this.loaded = loaded;
      reset();
      if (loaded) loadFromStorage();
    }
  }
  
  public void loadFromStorage()
  {
    try
    {
      Document doc = XML.parse(file);
      Element root = doc.getDocumentElement();
      loadFromXML(root);
      loaded = true;
    }
    catch (IOException e)
    {
      Reporter.error("Load of '" + rootTag + "' XML format from file '" + file + "' failed", e);
      reset();
      loaded = false;
    }
    catch (SAXException e)
    {
      Reporter.error("Parsing of '" + rootTag + "' XML format from file '" + file + "' failed" + e);
      reset();
      loaded = false;
    }
  }
  
  public void saveToStorage()
  {
    loaded = true;
    if (file == null) return;
    try
    {
      Document doc = XML.newDocument();
      Element root = doc.createElement(rootTag);
      doc.appendChild(root);

      saveToXML(root);

      XML.dump(file, root);
    }
    catch (IOException e)
    {
      Reporter.error("Saving of '" + rootTag + "' XML format to file '" + file + "' failed", e);
    }
  }
  
  public void deleteFromStorage()
  {
    file.delete();
  }
  
  public boolean toBeDeleted()
  {
    return false;
  }

  public String getName()
  {
    return name;
  }
  
  public File getFile()
  {
    return file;
  }
  
  public abstract void loadFromXML(Element element) throws SAXException;
  public abstract void saveToXML(Element element);
  protected abstract void reset();
}
