/*
 * RMISearchConfiguration.java
 *
 * Created on 21. kvten 2004, 13:33
 */

package SOFA.SOFAnet.Search.RMI;

import java.util.*;
import java.io.Serializable;
import SOFA.SOFAnet.Repository.NodeNameFilter;
import SOFA.SOFAnet.Repository.ShareGroups;

/**
 *
 * @author  Ladislav Sobr
 */
public class RMISearchConfiguration implements Cloneable, Serializable
{
  private NodeNameFilter nodeFilter;
  private List connections;
  
  public static class Connection implements Cloneable, Serializable
  {
    private String nodeName;
    private ShareGroups shareGroups;
    
    public Connection(String nodeName, ShareGroups shareGroups)
    {
      this.nodeName = nodeName;
      this.shareGroups = shareGroups;
    }

    public Object clone()
    {
      try
      {
        return super.clone();
      }
      catch (CloneNotSupportedException e)
      {
        throw new InternalError(); 
      }
    }
    
    public String toString()
    {
      return nodeName;
    }
    
    public String getNodeName()
    {
      return nodeName;
    }
    
    public ShareGroups getShareGroups()
    {
      return shareGroups;
    }
    
    public void setNodeName(String nodeName)
    {
      this.nodeName = nodeName;
    }
    
    public void setShareGroups(ShareGroups shareGroups)
    {
      this.shareGroups = shareGroups;
    }
  }
  
  /** Creates a new instance of RMISearchConfiguration */
  public RMISearchConfiguration()
  {
    reset();
  }

  public void reset()
  {
    nodeFilter = null;
    connections = Collections.synchronizedList(new LinkedList());
  }
  
  public Object clone()
  {
    RMISearchConfiguration clone = null;
    try
    { 
      clone = (RMISearchConfiguration)super.clone();
    }
    catch (CloneNotSupportedException e)
    { 
      throw new InternalError();
    }
    clone.connections = Collections.synchronizedList(new LinkedList());
    clone.connections.addAll(connections);
    return clone;
  }
  
  public NodeNameFilter getNodeFilter()
  {
    return nodeFilter;
  }
  
  public List getConnections()
  {
    return connections;
  }
  
  public void setNodeFilter(NodeNameFilter nodeFilter)
  {
    this.nodeFilter = nodeFilter;
  }
  
  public void setConnections(List connections)
  {
    this.connections = connections;
  }
}
