/*
 * SearchInterface.java
 *
 * Created on 28. duben 2004, 15:55
 */

package SOFA.SOFAnet.Search;

/**
 * Client interface of search subsytem
 *
 * @author  Ladislav Sobr
 */
public interface SearchInterface
{
  /**
   * Performs search request in the search network.
   * <p>
   * Input:
   * <ul>
   * <li>  searchPattern   &nbsp;&nbsp;<i>   request itself - what are we looking for</i>
   * <li>  firstOnly       &nbsp;&nbsp;<i>   wheter take only one (first) satisfying reply or all replies came in time limit</i>
   * <li>  shareGroups     &nbsp;&nbsp;<i>   name of share groups that we should search through (null means "all")</i>
   * <li>  timeout         &nbsp;&nbsp;<i>   maximal time in milliseconds to wait for the reply(ies)</i>
   * </ul>
   * <p>
   * Output:
   * <ul>
   * <li>  errCode
   *    <ul>
   *    <li>    0 == OK
   *    <li>    1 == error occured
   *    </ul>
   * <li>  replies         &nbsp;&nbsp;<i>   list of replies to the request (only if errCode == 0)</i>
   * </ul>
   *
   * @param request Input/Output data for the call
   */
  void request(SearchRequest request) throws SearchException;

  /**
   * Replies to search request.
   * <p>
   * Input:
   * <ul>
   * <li>  requestID       &nbsp;&nbsp;<i>  identification of request that this reply belongs to (includes requesting node name)</i>
   * <li>  replies         &nbsp;&nbsp;<i>  list of replies</i>
   * </ul>
   * <p>
   * Output: none
   *
   * @param reply Input data for the call
   */
  void reply(SearchReply reply) throws SearchException;
}
