/*
 * SearchPattern.java
 *
 * Created on 28. duben 2004, 15:42
 */

package SOFA.SOFAnet.Search;

import java.io.Serializable;

/**
 * Describes real search request - what is looked for.
 * <p>
 * Meaning of parameters:
 * <ul>
 * <li>  pattern:
 *   <ul>
 *   <li>  false: "str" contains exact bundle(or component) name
 *   <li>  true: "str" contains pattern of bundle(or component) name
 *   </ul>
 * <li>  component:
 *   <ul>
 *   <li>  false: look only for bundles, not their components
 *   <li>  true: look also for components in bundles ("str" must contain name/pattern of a single component)
 *   </ul>
 * <li>  completeComponent (only if component == true):
 *   <ul>
 *   <li>  CC_TRY: try to find complete component, but it needn't be
 *   <li>  CC_NO: look only for single components
 *   <li>  CC_ONLY: found bundle must surely contain complete component
 *   </ul>
 * </ul>
 *
 * @author  Ladislav Sobr
 */
public class SearchPattern implements Serializable
{
  final static public int CC_TRY = 0;
  final static public int CC_NO = 1;
  final static public int CC_ONLY = 2;
  
  private boolean pattern;
  private boolean component;
  private int completeComponent;
  
  private String str;
  
  /** Creates a new instance of SearchPattern */
  public SearchPattern()
  {
    pattern = false;
    component = false;
    completeComponent = CC_TRY;
  }
  
  public boolean isPattern()
  {
    return pattern;
  }
  
  public boolean isComponent()
  {
    return component;
  }
  
  public int getCompleteComponent()
  {
    return completeComponent;
  }
  
  public String getStr()
  {
    return str;
  }
  
  public void setPattern(boolean pattern)
  {
    this.pattern = pattern;
  }
  
  public void setComponent(boolean component)
  {
    this.component = component;
  }
  
  public void setCompleteComponent(int completeComponent)
  {
    if (completeComponent != CC_NO && completeComponent != CC_ONLY) completeComponent = CC_TRY;
    this.completeComponent = completeComponent;
  }
  
  public void setStr(String str)
  {
    this.str = str;
  }
}
