/*
 * SearchRequest.java
 *
 * Created on 28. duben 2004, 15:18
 */

package SOFA.SOFAnet.Search;

import java.util.List;

/**
 * Data structure for I/O of search request.
 *
 * @author  Ladislav Sobr
 */
public class SearchRequest
{
  private SearchPattern searchPattern;  //request itself
  
  //other input concerning search network
  private boolean firstOnly;            //wheter take only one (first) satisfying reply
  private String shareGroup;            //name of share group that we should search through (empty string means "all")
  private int timeout;                  //maximal time in milliseconds to wait for the reply(ies)
  
  //output
  private int errCode;
  private List replies;                 //list of SearchReplyItem objects
  
  /** Creates a new instance of SearchRequest */
  public SearchRequest()
  {
    firstOnly = true;
    shareGroup = "";
    timeout = 5000;
    errCode = -1;
    replies = null;
  }

  public SearchPattern getSearchPattern()
  {
    return searchPattern;
  }
  
  public boolean getFirstOnly()
  {
    return firstOnly;
  }
  
  public String getShareGroup()
  {
    return shareGroup;
  }
  
  public int getTimeout()
  {
    return timeout;
  }
  
  public int getErrCode()
  {
    return errCode;
  }
  
  public List getReplies()
  {
    return replies;
  }
  
  public void setSearchPattern(SearchPattern searchPattern)
  {
    this.searchPattern = searchPattern;
  }
  
  public void setFirstOnly(boolean firstOnly)
  {
    this.firstOnly = firstOnly;
  }
  
  public void setShareGroup(String shareGroup)
  {
    this.shareGroup = shareGroup;
  }
  
  public void setTimeout(int timeout)
  {
    this.timeout = timeout;
  }
  
  public void setErrCode(int errCode)
  {
    this.errCode = errCode;
  }
  
  public void setReplies(List replies)
  {
    this.replies = replies;
  }
}
