/*
 * RMITransportCenter.java
 *
 * Created on 3. kvten 2004, 16:41
 */

package SOFA.SOFAnet.Transport.RMI;

import SOFA.SOFAnet.Transport.*;
import SOFA.SOFAnet.Core.NetOps;
import SOFA.SOFAnet.Core.ShareOps;
import SOFA.SOFAnet.Repository.NodeInfo;
import java.rmi.RemoteException;
import java.rmi.Naming;
import java.net.MalformedURLException;

/**
 * Managing class of RMI implementation of transport subsystem of the SOFAnet.
 *
 * @author  Ladislav Sobr
 */
public class RMITransportCenter
{
  private NetOps netOps;
  private ShareOps shareOps;

  private RMITransportClient client;
  private RMITransportServer server;
  
  boolean started;
  private String myNodeName;
  
  /** Creates a new instance of RMITransportCenter */
  public RMITransportCenter()
  {
    started = false;
    
    client = new RMITransportClient();
    server = null;
    
    myNodeName = NodeInfo.getLocalNodeName();
  }
  
  public void setNetOps(NetOps netOps)
  {
    this.netOps = netOps;
    client.setNetOps(netOps);
  }
  
  public void setShareOps(ShareOps shareOps)
  {
    this.shareOps = shareOps;
    client.setShareOps(shareOps);
  }
  
  public TransportInterface getTransportInterface()
  {
    return client;
  }
  
  /** Starts RMI transport subsystem */
  public void start() throws RMITransportException
  {
    started = true;
    
    //run RMI transport server
   
    NodeInfo myNodeInfo = new NodeInfo();
    try
    {
      myNodeInfo.setNodeName(myNodeName);
    }
    catch (NodeInfo.InvalidNodeNameException e)
    {
      throw new RMITransportException("Invalid name of local SOFA node: " + myNodeName, e);
    }
    
    try
    {
      server = new RMITransportServer(netOps, shareOps);
      Naming.rebind("//" + myNodeInfo.getAddressAndPort() + "/SOFAnet/RMITransportServer", server);
    }
    catch (RemoteException e)
    {
      throw new RMITransportException("RMI transport server failed to start", e);
    }
    catch (MalformedURLException e)
    {
      throw new RMITransportException("RMI transport server failed to start", e);
    }
  }  

  boolean isStarted()
  {
    return started;
  }
}
