/*
 * RMITransportServerHolder.java
 *
 * Created on 8. duben 2004, 14:34
 */

package SOFA.SOFAnet.Transport.RMI;

import java.net.InetAddress;
import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import SOFA.SOFAnet.Repository.NodeInfo;

/**
 * The object of this class is responsible for connecting to remote RMI server and holding its interface.
 *
 * @author  Ladislav Sobr
 */
public class RMITransportServerHolder
{
  private RMITransportInterface server;
  private NodeInfo nodeInfo;
    
  /** Creates a new instance of RMITransportServerHolder */
  public RMITransportServerHolder()
  {
  }
  
  /** Setups the node to connect to */
  public void setNode(NodeInfo nodeInfo)
  {
    this.nodeInfo = nodeInfo;
  }
  
  /** Returns interface to the RMI server on the remote node, never returns null */
  public synchronized RMITransportInterface getServer() throws RMITransportException
  {
    if (nodeInfo == null) throw new RMITransportException("Not specified node");
    
    if (server != null) return server;

    try
    {
      server = (RMITransportInterface)Naming.lookup("rmi://" + nodeInfo.getAddressAndPort() + "/SOFAnet/RMITransportServer");
    }
    catch (NotBoundException e)
    {
      server = null;
      throw new RMITransportException(e.getMessage(), e);
    }
    catch (MalformedURLException e)
    {
      server = null;
      throw new RMITransportException(e.getMessage(), e);
    }
    catch (RemoteException e)
    {
      server = null;
      throw new RMITransportException(e.getMessage(), e);
    }    
    
    return server;
  }
  
  /** Releases interface of the remote RMI server */
  public void releaseServer()
  {
    server = null;
  }
  
}
