/**
 * InOut2InOutConnector.java is a part of the SOFA project.
 * This file was created by pepan on 18.4.2003.
 */
package SOFA.SOFAnode.InOut.Connector;

import SOFA.Connector.ConnectorException;
import SOFA.Connector.LinkException;
import SOFA.Connector.Linkable;
import SOFA.Connector.Reference;
import SOFA.Connector.RoleBase;
import SOFA.Connector.TaggedProfile;
import SOFA.SOFAnode.InOut.InOut2InOut;
import SOFA.Util.VMProperties;

/** Contains methods for creating a connector between two {@link InOut2InOut} interfaces.
 *
 * @author Petr Panuska
 */
public class InOut2InOutConnector {
  /**
   * Creates the server side of the connector.
   * @param inOut InOut which is going to be connectable.
   * @return the server side of the connector.
   */
  public static RoleBase createSrv (InOut2InOut inOut) throws ConnectorException {
    RoleBase role = new InOut2InOutSRole();
    SOFA.Connector.ECG.DeploymentDescriptor cDD = new SOFA.Connector.ECG.DeploymentDescriptor();
    cDD.unit = "Server";
    role.setDeploymentDescriptor(cDD);

    Linkable skelRMI;
    String rmiName;
    String rmihost = System.getProperty(VMProperties.RMI_HOST, "localhost");
    String rmiport = System.getProperty(VMProperties.RMI_PORT, "1099");
    String inOutName = System.getProperty(VMProperties.TR_NAME, "localInOut");
    inOutName = "/SOFA/SOFAnode/InOut/Connector/InOut2InOutRMISkel/" + inOutName;
    rmiName = "//" + rmihost + ":" + rmiport + inOutName;
    try {
      skelRMI = new InOut2InOutRMISkel(inOutName);
    } catch (Exception e) {
      throw new ConnectorException("Can't instantiate InOut2InOutRMISkel.\n" + rmiName + "\n", e);
    }
    skelRMI.link(role);

    // ???
    Reference ref = new Reference("SOFA.Connector.ECG.SGenerator.Connectors.DeplDockConnector",
      new TaggedProfile[]{new SOFA.Connector.ECG.Profiles.LocalProfile(role), new SOFA.Connector.ECG.Profiles.RMIProfile(rmiName)});
    role.setSOFAReference(ref);
    role.link(inOut);
    return role;
  }

  /**
   * Creates the client side of the connector.
   * @param ref the reference to InOut.
   * @return the client side of the connector.
   */
  public static RoleBase createClt (Reference ref) throws ConnectorException {
    RoleBase role = new SOFA.SOFAnode.InOut.Connector.InOut2InOutCRole();
    try {
      role.link(ref);
    } catch (LinkException e) {
      throw new ConnectorException("Can't link connector to InOut.\n" + ref + "\n", e);
    }
    return role;
  }

}
