/* $Id: CompException.java,v 1.2 2004/05/20 14:23:51 bures Exp $ */
package SOFA.SOFAnode.Made.CDL;
import java.rmi.RemoteException;

import SOFA.SOFAnode.Made.TIR.CDLRepository;
import SOFA.SOFAnode.Made.TIR.Contained;
import SOFA.SOFAnode.Made.TIR.Container;
import SOFA.SOFAnode.Made.TIR.EnumDef;
import SOFA.SOFAnode.Made.TIR.ExceptionDef;
import SOFA.SOFAnode.Made.TIR.StructDef;
import SOFA.SOFAnode.Made.TIR.StructMember;
import SOFA.SOFAnode.Made.TIR.TIRExceptCreate;
import SOFA.SOFAnode.Made.TIR.TIRExceptLock;
import SOFA.SOFAnode.Made.TIR.UnionDef;

class CompException extends CompContainedContainerIm {
  public CompException(ID id, CompContainer in, CompRepository inRep) {
    super(ObjectsKind.o_Exception, id, in, inRep);
  }

  public boolean addObject(CompContained o) {
    ((CompContained) o).getIdent().version = id.version;
    return super.addObject(o);
  }

  public void addToNormal(Container obj, CDLRepository normRep, java.util.ArrayList delayed) throws CDLExceptToNormal, CDLExceptRemote, CDLExceptLock {
    if (id.isin == IDKind.versiontag) {
      try {
        ((Contained)obj).tag(id.tag);
      } catch (RemoteException e) {
        throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
      } catch (TIRExceptLock e) {
        throw new CDLExceptLock("Repository is locked.");
      } catch (TIRExceptCreate ecr) {
        throw new CDLExceptToNormal("Can't set tag for "+ fullName());
      }
    }
    cont.toFirst();
    try {
      for (int i=0;i<cont.size(); i++) {
        CompContained akt = (CompContained) cont.aktual();
        switch (akt.objectKind()) {
        // *****************
        case ObjectsKind.o_Struct:
          try {
            StructDef str = ((ExceptionDef) obj).create_struct(akt.getIdent().id);
            if (str == null) 
              throw new CDLExceptToNormal("Can't create struct "+((CompContained)akt).fullName());
            ((CompStruct) akt).addToNormal(str, normRep, delayed);
          } catch (TIRExceptCreate ecr) {
            throw new CDLExceptToNormal("Can't create struct "+((CompContained)akt).fullName());
          }
          break;
        // *****************
        case ObjectsKind.o_StructMember:
          try {
            StructMember mem = ((ExceptionDef) obj).create_member(akt.getIdent().name, ((CompStructMember)akt).tp.toNormal(normRep));
            if (mem == null) 
              throw new CDLExceptToNormal("Can't create struct member "+((CompContained)akt).fullName());
          } catch (TIRExceptCreate ecr) {
            throw new CDLExceptToNormal("Can't create struct member "+((CompContained)akt).fullName());
          }
          break;
        // *****************
        case ObjectsKind.o_Enum:
          try {
            EnumDef en = ((ExceptionDef) obj).create_enum(akt.getIdent().id);
            if (en == null) 
              throw new CDLExceptToNormal("Can't create enum "+((CompContained)akt).fullName());
            ((CompEnum) akt).addToNormal(en);
          } catch (TIRExceptCreate ecr) {
            throw new CDLExceptToNormal("Can't create enum "+((CompContained)akt).fullName());
          }
          break;
        // *****************
        case ObjectsKind.o_Union:
          try {
            UnionDef un = ((ExceptionDef) obj).create_union(akt.getIdent().id, ((CompUnion) akt).switch_type.toNormal(normRep));
            if (un == null) 
              throw new CDLExceptToNormal("Can't create union "+((CompContained)akt).fullName());
            ((CompUnion) akt).addToNormal(un, normRep, delayed);
          } catch (TIRExceptCreate ecr) {
            throw new CDLExceptToNormal("Can't create union "+((CompContained)akt).fullName());
          }
          break;

        default:
          throw new CDLExceptToNormal("Unexcepted kind of object"); 
        }
        cont.toNext();
      }
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    } catch (TIRExceptLock e) {
      throw new CDLExceptLock("Repository is locked.");
    }
  }

  public void checkConsist(EnumList props, CompRepository rep) throws CDLExceptCheck, CDLExceptLock, CDLExceptRemote {
    CompContained[] cont = contents();
    for (int i=0;i<cont.length;i++) {
      cont[i].checkConsist(props, rep);
    }
  }
}
