/* $Id: CompExprOper.java,v 1.2 2004/05/20 14:23:51 bures Exp $ */
package SOFA.SOFAnode.Made.CDL;
import java.rmi.RemoteException;

import SOFA.SOFAnode.Made.TIR.CDLRepository;
import SOFA.SOFAnode.Made.TIR.ExprBinOperationDef;
import SOFA.SOFAnode.Made.TIR.ExprConstant;
import SOFA.SOFAnode.Made.TIR.ExprOperDef;
import SOFA.SOFAnode.Made.TIR.ExprOperKind;
import SOFA.SOFAnode.Made.TIR.ExprProperty;
import SOFA.SOFAnode.Made.TIR.ExprUnOperationDef;

/** ancestor of all expression types */
abstract class CompExprOper {
  public int kind;

  public CompExprOper(int t) { kind = t; }

  /** return type (kind) of expression */
  public abstract int type();

  /** tests if expression contains property */
  public abstract boolean isProp();

  /** returns list of names of properties in expression */
  public abstract EnumList nameProp(CompRepository rep) throws CDLExceptRemote, CDLExceptLock;

  /** return >= 0 if expression is positive 
             = -1 if contains property, 
             = -2 non positive or non int expression */
  //public abstract int isPositive();

  public abstract ExprOperDef toNormal(CDLRepository rep) throws CDLExceptToNormal, CDLExceptRemote, CDLExceptLock; 

    /** test if "exprProp" is subset of "prop" */
  public static boolean testPropInExpr(EnumList prop, EnumList exprProp) {
    if (prop==null || exprProp==null)
      return true;
    exprProp.toFirst();
    for (int i=0;i<exprProp.size();i++) {
      if (!prop.isIn((String) exprProp.aktual()))
        return false;
      exprProp.toNext();
    }
    return true;
  }

  /** if expreession is suitable for type */
  public static boolean isExprType(CompType t, CompExprOper e) {
    if (e.type()==CompExprKind.e_default)
      return true;
    int ttype;
    switch (t.objectKind()) {
    case ObjectsKind.o_none:
      if (((CompReffer) t).basereffer == ObjectsKind.o_Primitive) {
        ttype = ((CompReffer) t).baserefferprim;
        switch (ttype) {
        case CompPrimKind.p_short:
        case CompPrimKind.p_long:
        case CompPrimKind.p_longlong:
        case CompPrimKind.p_ushort:
        case CompPrimKind.p_ulong:
        case CompPrimKind.p_ulonglong:
          if (e.type() == CompExprKind.e_int || e.type() == CompExprKind.e_all)
            return true;
          else
            return false;
        case CompPrimKind.p_float:
        case CompPrimKind.p_double:
        case CompPrimKind.p_longdouble:
          if (e.type() == CompExprKind.e_float || e.type() == CompExprKind.e_all)
            return true;
          else
            return false;
        case CompPrimKind.p_string:
          if (e.type() == CompExprKind.e_string || e.type() == CompExprKind.e_all)
            return true;
          else
            return false;
        case CompPrimKind.p_wstring:
          if (e.type() == CompExprKind.e_wstring || e.type() == CompExprKind.e_all)
            return true;
          else
            return false;
        case CompPrimKind.p_char:
          if (e.type() == CompExprKind.e_char || e.type() == CompExprKind.e_all)
            return true;
          else
            return false;
        case CompPrimKind.p_wchar:
          if (e.type() == CompExprKind.e_wchar || e.type() == CompExprKind.e_all)
            return true;
          else
            return false;
        case CompPrimKind.p_boolean:
          if (e.type() == CompExprKind.e_boolean || e.type() == CompExprKind.e_all)
            return true;
          else
            return false;
        }
      } else {
        switch (((CompReffer) t).basereffer) {
        case ObjectsKind.o_String:
          if (e.type() == CompExprKind.e_string || e.type() == CompExprKind.e_all)
            return true;
          else
            return false;
        case ObjectsKind.o_Wstring:
          if (e.type() == CompExprKind.e_wstring || e.type() == CompExprKind.e_all)
            return true;
          else
            return false;
        case ObjectsKind.o_Enum:
          if (e.type() == CompExprKind.e_enum || e.type() == CompExprKind.e_all)
            return true;
          else
            return false;
        case ObjectsKind.o_Fixed:
          if (e.type() == CompExprKind.e_fixed || e.type() == CompExprKind.e_all)
            return true;
          else
            return false;
        }
      }
      break;
    case ObjectsKind.o_Primitive:
      ttype = ((CompPrimitive) t).kind;
      switch (ttype) {
      case CompPrimKind.p_short:
      case CompPrimKind.p_long:
      case CompPrimKind.p_longlong:
      case CompPrimKind.p_ushort:
      case CompPrimKind.p_ulong:
      case CompPrimKind.p_ulonglong:
        if (e.type() == CompExprKind.e_int || e.type() == CompExprKind.e_all)
          return true;
        else
          return false;
      case CompPrimKind.p_float:
      case CompPrimKind.p_double:
      case CompPrimKind.p_longdouble:
        if (e.type() == CompExprKind.e_float || e.type() == CompExprKind.e_all)
          return true;
        else
          return false;
      case CompPrimKind.p_string:
        if (e.type() == CompExprKind.e_string || e.type() == CompExprKind.e_all)
          return true;
        else
          return false;
      case CompPrimKind.p_wstring:
        if (e.type() == CompExprKind.e_wstring || e.type() == CompExprKind.e_all)
          return true;
        else
          return false;
      case CompPrimKind.p_char:
        if (e.type() == CompExprKind.e_char || e.type() == CompExprKind.e_all)
          return true;
        else
          return false;
      case CompPrimKind.p_wchar:
        if (e.type() == CompExprKind.e_wchar || e.type() == CompExprKind.e_all)
          return true;
        else
          return false;
      case CompPrimKind.p_boolean:
        if (e.type() == CompExprKind.e_boolean || e.type() == CompExprKind.e_all)
          return true;
        else
          return false;
      }
      break;
    case ObjectsKind.o_Enum:
      if (e.type() == CompExprKind.e_enum)
        return true;
      else
        return false;
    case ObjectsKind.o_String:
      if (e.type() == CompExprKind.e_string)
        return true;
      else
        return false;
    case ObjectsKind.o_Wstring:
      if (e.type() == CompExprKind.e_wstring)
        return true;
      else
        return false;
    case ObjectsKind.o_Fixed:
      if (e.type() == CompExprKind.e_fixed)
        return true;
      else
        return false;
    default:
      return false;
    }
    return false;
  }

  /** search properties in "normal" expressions */
  public static EnumList scanNormalExpr(ExprOperDef e) throws CDLExceptRemote {
    try {
      int i;
      switch (e.get_eok_kind().value()) {
      case ExprOperKind.eok_or:
      case ExprOperKind.eok_xor:
      case ExprOperKind.eok_and:
      case ExprOperKind.eok_shr:
      case ExprOperKind.eok_shl:
      case ExprOperKind.eok_plus:
      case ExprOperKind.eok_minus:
      case ExprOperKind.eok_mul:
      case ExprOperKind.eok_div:
      case ExprOperKind.eok_mod:
        EnumList l1 = scanNormalExpr(((ExprBinOperationDef) e).operand1());
        EnumList l2 = scanNormalExpr(((ExprBinOperationDef) e).operand2());
        for (i=0;i<l2.size();i++) {
          l1.addName((String) l2.aktual());
          l2.toNext();
        }
        return l1;
      case ExprOperKind.eok_unminus:
      case ExprOperKind.eok_unplus:
      case ExprOperKind.eok_untilde:
        return scanNormalExpr(((ExprUnOperationDef) e).operand());
      case ExprOperKind.eok_short:
      case ExprOperKind.eok_long:
      case ExprOperKind.eok_longlong:
      case ExprOperKind.eok_unsigshort:
      case ExprOperKind.eok_unsiglong:
      case ExprOperKind.eok_unsiglonglong:
      case ExprOperKind.eok_float:
      case ExprOperKind.eok_double:
      case ExprOperKind.eok_longdouble:
      case ExprOperKind.eok_fixed:
      case ExprOperKind.eok_valuegen:
      case ExprOperKind.eok_enum:
      case ExprOperKind.eok_octet:
      case ExprOperKind.eok_char:
      case ExprOperKind.eok_wchar:
      case ExprOperKind.eok_string:
      case ExprOperKind.eok_wstring:
      case ExprOperKind.eok_default:
        return new EnumList();
      case ExprOperKind.eok_constant:
        return scanNormalExpr(((ExprConstant) e).ref_const().value());
      case ExprOperKind.eok_property:
        EnumList l = new EnumList();
        l.addName(((ExprProperty) e).ref_prop());
        return l;
      }
    } catch (RemoteException ex) {
      throw new CDLExceptRemote(ex.getMessage());
    }
    return new EnumList();
  }
}
