/* $Id: CGFileWriter.java,v 1.2 2004/05/20 14:23:51 bures Exp $ */
package SOFA.SOFAnode.Made.CodeGen;

import java.io.File;
import java.io.IOException;

/** Class for writing files with generated code. Use only <tt>writeln</tt> methods for writing new line!
  *
  * @author Petr Hnetynka
  */
public class CGFileWriter extends java.io.FileWriter {
  private int indent;
  private boolean doIndent;


  public CGFileWriter(String name) throws IOException {
    super(name);
    indent = 0;
    doIndent = true;
  }

  public CGFileWriter(File file) throws IOException {
    super(file);
    indent = 0;
    doIndent = true;
  }
 
  /** Line separator in files. */
  public static final String newLineSep = "\n";  // !!! change to property line.separator ??? may be

  /** Number of spaces of one level of indentation */
  public static final int indentSize = 2;


  /** Current indentaion level */
  public int curIndent() {
    return indent;
  }

  /** Increase intentaion level */
  public void incIndent() {
    indent++;
  }

  /** Decrease intentaion level */
  public void decIndent() {
    if (indent != 0)
      indent--;
  }

  private void makeIndent() throws IOException {
    if (doIndent) {
      for (int i=0; i<indent * indentSize; i++) {
        super.write(' ');
      }
      doIndent = false;
    }
  }

  /** Writes new line.
    *
    * @exception IOException     If an I/O error occurs
    */
  public void iwriteln() throws IOException {
    makeIndent();
    super.write(newLineSep);
    flush();
    doIndent = true;
  }

  /** Writes text and new line.
    *
    * @param text   string 
    * @exception IOException     If an I/O error occurs
    */
  public void iwriteln(String text) throws IOException {
    makeIndent();
    super.write(text);
    super.write(newLineSep);
    flush();
    doIndent = true;
  }

  /** Writes spaces.
    * @param number  number of spaces
    * @exception IOException     If an I/O error occurs
    */
  public void iwritespace(int number) throws IOException {
    makeIndent();
    for (int i=0;i<number;i++) {
      super.write(" ");
    }
  }

  public void iwrite(char[] cbuf, int off, int len) throws IOException {
    makeIndent();
    super.write(cbuf, off, len);
  }

  public void iwrite(int c) throws IOException {
    makeIndent();
    super.write(c);
  }

  public void iwrite(String str, int off, int len) throws IOException {
    makeIndent();
    super.write(str, off, len);
  }
 
  public void iwrite(char[] cbuf) throws IOException {
    makeIndent();
    super.write(cbuf);
  }

  public void iwrite(String str) throws IOException {
    makeIndent();
    super.write(str);
  }
}
