/* $Id: EnumDefImpl.java,v 1.2 2004/05/20 14:23:52 bures Exp $ */
package SOFA.SOFAnode.Made.TIR.Impl;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.rmi.RemoteException;

import SOFA.SOFAnode.Made.TIR.Container;
import SOFA.SOFAnode.Made.TIR.DefinitionKind;
import SOFA.SOFAnode.Made.TIR.EnumDef;
import SOFA.SOFAnode.Made.TIR.EnumMember;
import SOFA.SOFAnode.Made.TIR.Identification;
import SOFA.SOFAnode.Made.TIR.Repository;
import SOFA.SOFAnode.Made.TIR.StateKind;
import SOFA.SOFAnode.Made.TIR.TIRExceptCommit;
import SOFA.SOFAnode.Made.TIR.TIRExceptCreate;
import SOFA.SOFAnode.Made.TIR.TIRExceptLock;
import SOFA.SOFAnode.Made.TIR.TIRObject;

public class EnumDefImpl extends ContainedImpl implements EnumDef {
  DefinitionKindImpl defKindImpl;
  StateKindImpl stKindImpl; 
  int numOfMembs;
  LiItem firstMemb;
  LiItem lastMemb;

  EnumDefImpl normal;
  boolean hasCont;
  long workId;

  public EnumDefImpl(Identification ident, Container in, Repository inrep, long workId) throws RemoteException {
    super(ident,in,inrep);
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Enum);
    stKindImpl = new StateKindImpl(StateKind.sk_work);
    numOfMembs = 0;
    firstMemb = lastMemb = null;
    normal = null;
    hasCont = false;
    this.workId = workId;
  }
 
  public EnumDefImpl(Container in, Repository inrep) throws RemoteException {
    super(null,in,inrep);
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Enum);
    stKindImpl = new StateKindImpl(StateKind.sk_normal);
    numOfMembs = 0;
    firstMemb = lastMemb = null;
    normal = null;
    hasCont = false;
    this.workId = -1L;
  }

  public EnumDefImpl(EnumDefImpl a, Container in, Repository inrep, long workId) throws RemoteException {
    super(a.id,in,inrep);
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Enum);
    stKindImpl = new StateKindImpl(StateKind.sk_work);
    numOfMembs = 0;
    firstMemb = lastMemb = null;
    normal = a;
    hasCont = false;
    this.workId = workId;
  }

/* from interface TIRObject */
  public DefinitionKind get_def_kind() throws RemoteException {
    return (DefinitionKind) defKindImpl;
  }

  /* from interface TIRObject */
  public StateKind get_state() throws RemoteException {
    return (StateKind) stKindImpl;
  }

  public void tag(String t) throws RemoteException, TIRExceptCreate, TIRExceptLock {
    if (stKindImpl.value()!=StateKind.sk_work)
      throw new TIRExceptCreate("you can call create method on work object");
    super.tag(t);
  }

  private void addListItem(TIRObject what) {
    LiItem item = new LiItem(what);
    if (firstMemb == null) {
      firstMemb = lastMemb = item;
    } else {
      LiItem akt = firstMemb;
      while (akt.next != null) akt = akt.next;
      akt.next = item;
      item.prev = akt;
      lastMemb = item;
    }
    numOfMembs++;
  }

  private void fromNormObj() throws RemoteException, TIRExceptLock {
    if (normal==null)
      return;
    if (((WorkRepositoryImpl)rep).lock.isLocked())
      throw new TIRExceptLock("Repository is locked");
    int i;
    LiItem akt;
    akt = normal.firstMemb;
    for(i=0;i<normal.numOfMembs;i++) {
      addListItem(new EnumMemberImpl((EnumMemberImpl)akt.obj));
      akt = akt.next;
    }
    hasCont = true;
  }

  public EnumMember[] members() throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal && !hasCont)
      fromNormObj();
    EnumMember[] ret = new EnumMember [numOfMembs];
    LiItem akt = firstMemb;
    for(int i=0;i<numOfMembs;i++) {
      ret[i] = (EnumMember) akt.obj;
      akt = akt.next;
    }
    return ret;
  }

  public EnumMember create_member(String name) throws RemoteException, TIRExceptCreate, TIRExceptLock {
    if (!isNew())
      throw new TIRExceptCreate("you can call create method on work object");
    if (name!=null && name.compareTo("")!=0) {
      EnumMember[] a = members();
      boolean found = false;
      for (int i=0;i<a.length;i++) {
        if (a[i].name().compareTo(name)==0) {
          found = true;
          break;
        }
      }
      if (found)
        throw new TIRExceptCreate("Member with this name exists in enum.");
      EnumMember ret = new EnumMemberImpl(new String(name));
      addListItem(ret);
      return ret;
    } else 
      return null;
  }

  public void save(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      boolean closeFile = false;
      if (st.curOutFile==null) {  // saving in module
        st.curOutFile = new DataOutputStream(new FileOutputStream(st.currentFile));
	closeFile = true;
      } else {   // saving in structure (or union)
        ;
      }
      ((IdentificationImpl)id).save(st.curOutFile);   // saving identification
      st.curOutFile.writeInt(numOfMembs);
      LiItem akt = firstMemb;
      for(int i=0;i<numOfMembs;i++) {  // saving all members
        ((TIRImplObject)(akt.obj)).save(st);
	akt = akt.next;
      }
      if (closeFile)  { // close file 
        st.curOutFile.close();
	st.curOutFile = null;
      }
    } catch (IOException e) {
      throw new TIRExceptStorage("Can't write in file "+st.currentFile+".");
    }
  }

  public void load(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      boolean closeFile = false;
      if (st.curInFile==null) {
        st.curInFile = new DataInputStream(new FileInputStream(st.currentFile));
        closeFile = true;
      } else {
        ;
      }
      id = new IdentificationImpl();
      ((IdentificationImpl)id).load(st.curInFile);
      int num = st.curInFile.readInt();
      EnumMemberImpl memb;
      for (int i=0;i<num;i++) {
        memb = new EnumMemberImpl();
	memb.load(st);
	addListItem(memb);
      }
      if (closeFile)  { // close file 
        st.curInFile.close();
	st.curInFile = null;
      }
    } catch (IOException e) {
      throw new TIRExceptStorage("Can't read in file "+st.currentFile+".");
    }
  }

  public void postLoad(RepositoryImpl r)  throws RemoteException, TIRExceptStorage {
    ;
  }

  public boolean isNew() {
    return ((stKindImpl.value()==StateKind.sk_work) && normal==null);
  }

  public void canCommit() throws RemoteException, TIRExceptCommit {;}

  public void doCommit(Container in, Repository rep) throws RemoteException {
    if (stKindImpl.value()==StateKind.sk_normal)
      return;
    if (isNew()) {  
      stKindImpl.toNormal();
      LiItem akt = firstMemb;
      for (int i=0;i<numOfMembs;i++) {
        ((TIRImplObject)akt.obj).doCommit(in,rep);
        akt = akt.next;     
      }
      parent = in;
      this.rep = rep;
    } else {
      normal = null;
    }
  }

  public void doAbort(long workId) throws RemoteException {}
}
