/* $Id: ExprUnOperationDefImpl.java,v 1.2 2004/05/20 14:23:52 bures Exp $ */
package SOFA.SOFAnode.Made.TIR.Impl;
import java.io.IOException;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import SOFA.SOFAnode.Made.TIR.Container;
import SOFA.SOFAnode.Made.TIR.DefinitionKind;
import SOFA.SOFAnode.Made.TIR.ExprDouble;
import SOFA.SOFAnode.Made.TIR.ExprFixed;
import SOFA.SOFAnode.Made.TIR.ExprFloat;
import SOFA.SOFAnode.Made.TIR.ExprLong;
import SOFA.SOFAnode.Made.TIR.ExprLongLong;
import SOFA.SOFAnode.Made.TIR.ExprOctet;
import SOFA.SOFAnode.Made.TIR.ExprOperDef;
import SOFA.SOFAnode.Made.TIR.ExprOperKind;
import SOFA.SOFAnode.Made.TIR.ExprShort;
import SOFA.SOFAnode.Made.TIR.ExprUnOperationDef;
import SOFA.SOFAnode.Made.TIR.ExprUnsigLong;
import SOFA.SOFAnode.Made.TIR.ExprUnsigLongLong;
import SOFA.SOFAnode.Made.TIR.ExprUnsigShort;
import SOFA.SOFAnode.Made.TIR.Repository;
import SOFA.SOFAnode.Made.TIR.StateKind;
import SOFA.SOFAnode.Made.TIR.TIRExceptCalculate;
import SOFA.SOFAnode.Made.TIR.TIRExceptCommit;

public class ExprUnOperationDefImpl extends UnicastRemoteObject implements ExprUnOperationDef, TIRImplObject {
  ExprOperKindImpl eok;
  DefinitionKindImpl defKindImpl;
  StateKindImpl stKindImpl;
  ExprOperDef oper;

  public ExprUnOperationDefImpl(int kind, ExprOperDef op) throws RemoteException {
    eok = new ExprOperKindImpl(kind);
    oper = op;
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Expression);
    stKindImpl = new StateKindImpl(StateKind.sk_work);
  }

  public ExprUnOperationDefImpl(int kind) throws RemoteException {
    eok = new ExprOperKindImpl(kind);
    oper = null;
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Expression);
    stKindImpl = new StateKindImpl(StateKind.sk_normal);
  }

  public ExprUnOperationDefImpl() throws RemoteException {
    eok = null;
    oper = null;
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Expression);
    stKindImpl = new StateKindImpl(StateKind.sk_normal);
  }

  /* from interface TIRObject */
  public DefinitionKind get_def_kind() throws RemoteException {
    return (DefinitionKind) defKindImpl;
  }

  /* from interface TIRObject */
  public StateKind get_state() throws RemoteException {
    return (StateKind) stKindImpl;
  }

  public ExprOperDef operand() throws RemoteException {
    return oper;
  }
  
  public ExprOperKind get_eok_kind() throws RemoteException {
    return (ExprOperKind) eok;
  }
  
  public ExprOperDef calculate() throws RemoteException, TIRExceptCalculate {
    ExprOperDef a = oper.calculate();
    ExprOperDef ret = null;
    switch (a.get_eok_kind().value()) {
      case ExprOperKind.eok_short:
	if (eok.value()==ExprOperKind.eok_unminus) {
	  ret = new ExprShortImpl((short)-((ExprShort)a).value());
	} else {
          if (eok.value()==ExprOperKind.eok_unplus) {
	    ret = new ExprShortImpl(((ExprShort)a).value());
	  } else {
            if (eok.value()==ExprOperKind.eok_untilde) {
	      ret = new ExprShortImpl((short)~((ExprShort)a).value());
	    } else {
              throw new TIRExceptCalculate("Can't calculate.");
            }  
	  }
	}
	break;
      case ExprOperKind.eok_long:
	if (eok.value()==ExprOperKind.eok_unminus) {
	  ret = new ExprLongImpl(-((ExprLong)a).value());
	} else {
          if (eok.value()==ExprOperKind.eok_unplus) {
	    ret = new ExprLongImpl(((ExprLong)a).value());
	  } else {
            if (eok.value()==ExprOperKind.eok_untilde) {
              ret = new ExprLongImpl(~((ExprLong)a).value());
	    } else {
              throw new TIRExceptCalculate("Can't calculate.");
            }
	  }
	}
	break;
      case ExprOperKind.eok_longlong:
	if (eok.value()==ExprOperKind.eok_unminus) {
	  ret = new ExprLongLongImpl(-((ExprLongLong)a).value());
	} else {
          if (eok.value()==ExprOperKind.eok_unplus) {
	    ret = new ExprLongLongImpl(((ExprLongLong)a).value());
	  } else {
            if (eok.value()==ExprOperKind.eok_untilde) {
              ret = new ExprLongLongImpl(~((ExprLongLong)a).value());
	    } else {
              throw new TIRExceptCalculate("Can't calculate.");
            }
	  }
	}
	break;
      case ExprOperKind.eok_unsigshort:
	if (eok.value()==ExprOperKind.eok_unminus) {
	  ret = new ExprUnsigShortImpl((short)-((ExprUnsigShort)a).value());
	} else {
          if (eok.value()==ExprOperKind.eok_unplus) {
	    ret = new ExprUnsigShortImpl(((ExprUnsigShort)a).value());
	  } else {
            if (eok.value()==ExprOperKind.eok_untilde) {
              ret = new ExprUnsigShortImpl((short)~((ExprUnsigShort)a).value());
	    } else {
              throw new TIRExceptCalculate("Can't calculate.");
            }  
	  }
	}
	break;
      case ExprOperKind.eok_unsiglong:
	if (eok.value()==ExprOperKind.eok_unminus) {
	  ret = new ExprUnsigLongImpl(-((ExprUnsigLong)a).value());
	} else {
          if (eok.value()==ExprOperKind.eok_unplus) {
	    ret = new ExprUnsigLongImpl(((ExprUnsigLong)a).value());
	  } else {
            if (eok.value()==ExprOperKind.eok_untilde) {
              ret = new ExprUnsigLongImpl(~((ExprUnsigLong)a).value());
	    } else {
              throw new TIRExceptCalculate("Can't calculate.");
            }  
	  }
	}
	break;
      case ExprOperKind.eok_unsiglonglong:
	if (eok.value()==ExprOperKind.eok_unminus) {
	  ret = new ExprUnsigLongLongImpl(-((ExprUnsigLongLong)a).value());
	} else {
          if (eok.value()==ExprOperKind.eok_unplus) {
	    ret = new ExprUnsigLongLongImpl(((ExprUnsigLongLong)a).value());
	  } else {
            if (eok.value()==ExprOperKind.eok_untilde) {
              ret = new ExprUnsigLongLongImpl(~((ExprUnsigLongLong)a).value());
	    } else {
              throw new TIRExceptCalculate("Can't calculate.");
            }
	  }
	}
	break;
      case ExprOperKind.eok_octet:
	if (eok.value()==ExprOperKind.eok_unminus) {
	  ret = new ExprOctetImpl((byte)-((ExprOctet)a).value());
	} else {
          if (eok.value()==ExprOperKind.eok_unplus) {
	    ret = new ExprOctetImpl(((ExprOctet)a).value());
	  } else {
            if (eok.value()==ExprOperKind.eok_untilde) {
              ret = new ExprOctetImpl((byte)~((ExprOctet)a).value());
	    } else {
              throw new TIRExceptCalculate("Can't calculate.");
            }
	  }
	}
	break;
      case ExprOperKind.eok_float:
	if (eok.value()==ExprOperKind.eok_unminus) {
	  ret = new ExprFloatImpl(-((ExprFloat)a).value());
	} else {
          if (eok.value()==ExprOperKind.eok_unplus) {
	    ret = new ExprFloatImpl(((ExprFloat)a).value());
	  } else {
            throw new TIRExceptCalculate("Can't calculate.");
	  }
	}
	break;
      case ExprOperKind.eok_double:
	if (eok.value()==ExprOperKind.eok_unminus) {
	  ret = new ExprDoubleImpl(-((ExprDouble)a).value());
	} else {
          if (eok.value()==ExprOperKind.eok_unplus) {
	    ret = new ExprDoubleImpl(((ExprDouble)a).value());
	  } else {
            throw new TIRExceptCalculate("Can't calculate.");
	  }
	}
	break;
      case ExprOperKind.eok_fixed:
	if (eok.value()==ExprOperKind.eok_unminus) {
	  ret = new ExprFixedImpl(((ExprFixed)a).value().negate(),((ExprFixed)a).digits(),((ExprFixed)a).scale());
	} else {
          if (eok.value()==ExprOperKind.eok_unplus) {
	    ret = new ExprFixedImpl(((ExprFixed)a).value(),((ExprFixed)a).digits(),((ExprFixed)a).scale());
	  } else {
            throw new TIRExceptCalculate("Can't calculate.");
	  }
	}
	break;
      default: throw new TIRExceptCalculate("Can't calculate.");
    }
    return ret;
  }

  public void save(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      st.curOutFile.writeInt(eok.value());
      ((TIRImplObject)oper).save(st);
    } catch (IOException e) {
      throw new TIRExceptStorage("Can't write in directory "+st.current+".");
    }
  }
  
  public void load(Storage st) throws RemoteException, TIRExceptStorage {
    oper = Storage.readExprOperDef(st);
  }

  public void postLoad(RepositoryImpl r) throws RemoteException, TIRExceptStorage {
    ((TIRImplObject) oper).postLoad(r);
  }

  public boolean isNew() {
    return (stKindImpl.value()==StateKind.sk_work);
  }

  public void canCommit() throws RemoteException, TIRExceptCommit {;}

  public void doCommit(Container in, Repository rep) throws RemoteException {
    if (stKindImpl.value()==StateKind.sk_work) {
      stKindImpl.toNormal();
      ((TIRImplObject)oper).doCommit(in,rep);
    }
  }

  public void doAbort(long workId) throws RemoteException {}
}
