/* $Id: ProtocolDefImpl.java,v 1.2 2004/05/20 14:23:52 bures Exp $ */
package SOFA.SOFAnode.Made.TIR.Impl;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import SOFA.SOFAnode.Made.TIR.Container;
import SOFA.SOFAnode.Made.TIR.DefinitionKind;
import SOFA.SOFAnode.Made.TIR.ProtocolBinOperationDef;
import SOFA.SOFAnode.Made.TIR.ProtocolDef;
import SOFA.SOFAnode.Made.TIR.ProtocolMetaOperandDef;
import SOFA.SOFAnode.Made.TIR.ProtocolNestedOperandDef;
import SOFA.SOFAnode.Made.TIR.ProtocolNullOperandDef;
import SOFA.SOFAnode.Made.TIR.ProtocolOperDef;
import SOFA.SOFAnode.Made.TIR.ProtocolOperKind;
import SOFA.SOFAnode.Made.TIR.ProtocolOperandDef;
import SOFA.SOFAnode.Made.TIR.ProtocolUnOperationDef;
import SOFA.SOFAnode.Made.TIR.Repository;
import SOFA.SOFAnode.Made.TIR.StateKind;
import SOFA.SOFAnode.Made.TIR.TIRExceptCommit;
import SOFA.SOFAnode.Made.TIR.TIRExceptCreate;

public class ProtocolDefImpl extends UnicastRemoteObject implements ProtocolDef, TIRImplObject {
  ProtocolOperDef op;
  DefinitionKindImpl defKindImpl;
  StateKindImpl stKindImpl;

  public ProtocolDefImpl(ProtocolOperDef o) throws RemoteException {
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Protocol);
    stKindImpl = new StateKindImpl(StateKind.sk_work);
    op = o;   
  }

  public ProtocolDefImpl(int state) throws RemoteException {
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Protocol);
    stKindImpl = new StateKindImpl(state);
    op = null;   
  }

  /* from interface TIRObject */
  public DefinitionKind get_def_kind() throws RemoteException {
    return (DefinitionKind) defKindImpl;
  }

  /* from interface TIRObject */
  public StateKind get_state() throws RemoteException {
    return (StateKind) stKindImpl;
  }
  
  public ProtocolOperDef protocol() throws RemoteException {
    return op;
  }
  
  public void protocol(ProtocolOperDef pr) throws RemoteException, TIRExceptCreate {
    if (stKindImpl.value()==StateKind.sk_normal)
      throw new TIRExceptCreate("you can call create method on work object");
    op = pr;
  }

  public ProtocolBinOperationDef create_binoperation(int pokind, ProtocolOperDef op1, ProtocolOperDef op2) throws RemoteException, TIRExceptCreate {
    if (stKindImpl.value()==StateKind.sk_normal)
      throw new TIRExceptCreate("you can call create method on work object");
    if (op1==null || op2==null)
      return null;
    if (op1.get_state().value()==StateKind.sk_work &&
        op2.get_state().value()==StateKind.sk_work ) {
      return new ProtocolBinOperationDefImpl(pokind,op1,op2);
    } else
      throw new TIRExceptCreate("Given objects aren't work.");
  }

  public ProtocolUnOperationDef create_unoperation(int pokind, ProtocolOperDef op) throws RemoteException, TIRExceptCreate {
    if (stKindImpl.value()==StateKind.sk_normal)
      throw new TIRExceptCreate("you can call create method on work object");
    if (op==null)
      return null;
    if (op.get_state().value()==StateKind.sk_work) {
      return new ProtocolUnOperationDefImpl(pokind,op);
    } else
      throw new TIRExceptCreate("Given object isn't work.");
  }

  public ProtocolNestedOperandDef create_nestedoperand(int mode, String[] op, ProtocolOperDef nestOper) throws RemoteException, TIRExceptCreate {
    if (stKindImpl.value()==StateKind.sk_normal)
      throw new TIRExceptCreate("you can call create method on work object");
    if (op==null || nestOper==null)
      return null;
    if (op.length==0)
      return null;
    if (nestOper.get_state().value()==StateKind.sk_work) {
      return new ProtocolNestedOperandDefImpl(op, new ProtocolModeImpl(mode), nestOper);
    } else
      throw new TIRExceptCreate("Given object isn't work.");
  }
  
  public ProtocolOperandDef create_operand(int mode, String[] op) throws RemoteException, TIRExceptCreate {
    if (stKindImpl.value()==StateKind.sk_normal)
      throw new TIRExceptCreate("you can call create method on work object");
    if (op==null)
      return null;
    if (op.length==0)
      return null;
    return new ProtocolOperandDefImpl(op, new ProtocolModeImpl(mode));
  } 
  
  public ProtocolNullOperandDef create_nulloperand() throws RemoteException, TIRExceptCreate {
    if (stKindImpl.value()==StateKind.sk_normal)
      throw new TIRExceptCreate("you can call create method on work object");
    return new ProtocolNullOperandDefImpl(StateKind.sk_work);
  }
  
  public ProtocolMetaOperandDef create_metaoperand(String name) throws RemoteException, TIRExceptCreate {
    if (stKindImpl.value()==StateKind.sk_normal)
      throw new TIRExceptCreate("you can call create method on work object");
    if (name==null)
      return null;
    return new ProtocolMetaOperandDefImpl(name);
  }

  public void save(Storage st) throws RemoteException, TIRExceptStorage {
    Storage.writeProtocolOperDef(st, op);
  }

  public void load(Storage st) throws RemoteException, TIRExceptStorage {
    op = Storage.readProtocolOperDef(st);
  }

  public void postLoad(RepositoryImpl r) throws RemoteException, TIRExceptStorage {
    ((TIRImplObject) op).postLoad(r);
  }

  public boolean isNew() {
    return (stKindImpl.value()==StateKind.sk_work);
  }

  public void canCommit() throws RemoteException, TIRExceptCommit {;}

  public void doCommit(Container in, Repository rep) throws RemoteException {
    if (stKindImpl.value()==StateKind.sk_normal)
      return;
    stKindImpl.toNormal();
    op = toNormal(op);
    ((TIRImplObject) op).doCommit(in,rep);
  }

  public void doAbort(long workId) throws RemoteException {}

  public String to_text() throws RemoteException {
    return ((SProtocolOperDef)op).to_text();
  }

  static ProtocolOperDef toNormal(ProtocolOperDef pr) throws RemoteException {
    if (pr == null)
      return null;
    String[] sold;
    String[] snew;
    ProtocolOperDef o1;
    ProtocolOperDef o2;
    int i;
    switch (pr.get_po_kind().value()) {
    case ProtocolOperKind.pok_andparallel:
    case ProtocolOperKind.pok_orparallel:
    case ProtocolOperKind.pok_sequence:
    case ProtocolOperKind.pok_alternative:
    case ProtocolOperKind.pok_restriction:
      o1 = toNormal(((ProtocolBinOperationDef) pr).operand1());
      o2 = toNormal(((ProtocolBinOperationDef) pr).operand2());
      return new ProtocolBinOperationDefImpl(pr.get_po_kind().value(),o1,o2);
    case ProtocolOperKind.pok_repetition:
      o1 = toNormal(((ProtocolUnOperationDef) pr).operand());
      return new ProtocolUnOperationDefImpl(pr.get_po_kind().value(),o1);
    case ProtocolOperKind.pok_nestedoperand:
      o1 = toNormal(((ProtocolNestedOperandDef) pr).nested_operand());
      sold = ((ProtocolNestedOperandDef) pr).operand();
      snew = new String [sold.length];
      for(i=0;i<sold.length;i++) {
        snew[i] = new String(sold[i]);
      }
      return new ProtocolNestedOperandDefImpl(snew, new ProtocolModeImpl(((ProtocolNestedOperandDef) pr).operand_mode().value()),o1);
    case ProtocolOperKind.pok_operand:
      sold = ((ProtocolOperandDef) pr).operand();
      snew = new String [sold.length];
      for(i=0;i<sold.length;i++) {
        snew[i] = new String(sold[i]);
      }
      return new ProtocolOperandDefImpl(snew, new ProtocolModeImpl(((ProtocolOperandDef) pr).operand_mode().value()));
    case ProtocolOperKind.pok_nulloperand:
      return new ProtocolNullOperandDefImpl(StateKind.sk_normal);
    case ProtocolOperKind.pok_metaoperand:
      return new ProtocolMetaOperandDefImpl(new String( ((ProtocolMetaOperandDef)pr).name() ));
    }
    return null;
  }
}
