/* $Id: ProvideDefImpl.java,v 1.2 2004/05/20 14:23:52 bures Exp $ */
package SOFA.SOFAnode.Made.TIR.Impl;
import java.io.IOException;
import java.rmi.RemoteException;

import SOFA.SOFAnode.Made.TIR.AbsoluteName;
import SOFA.SOFAnode.Made.TIR.CDLType;
import SOFA.SOFAnode.Made.TIR.Contained;
import SOFA.SOFAnode.Made.TIR.Container;
import SOFA.SOFAnode.Made.TIR.DefinitionKind;
import SOFA.SOFAnode.Made.TIR.Identification;
import SOFA.SOFAnode.Made.TIR.ProvideDef;
import SOFA.SOFAnode.Made.TIR.Repository;
import SOFA.SOFAnode.Made.TIR.StateKind;
import SOFA.SOFAnode.Made.TIR.TIRExceptCommit;
import SOFA.SOFAnode.Made.TIR.TIRExceptCreate;
import SOFA.SOFAnode.Made.TIR.TIRExceptLock;

public class ProvideDefImpl extends ContainedImpl implements ProvideDef, TIRImplObject, SContained {
  CDLType prov;
  DefinitionKindImpl defKindImpl;
  StateKindImpl stKindImpl;

  ProvideDefImpl normal;
  long workId;

  public ProvideDefImpl(Identification ident, Container in, Repository inrep, CDLType pr, long workId) throws RemoteException {
    super(ident, in,inrep);
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Provides);
    stKindImpl = new StateKindImpl(StateKind.sk_work);
    prov = pr;
    normal = null;  
    this.workId = workId;
  }

  public ProvideDefImpl(Container in, Repository inrep) throws RemoteException {
    super(null,in,inrep);
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Provides);
    stKindImpl = new StateKindImpl(StateKind.sk_normal);
    prov = null;
    normal = null;
    this.workId = -1L;
  }

  public ProvideDefImpl(ProvideDefImpl a, Container in, Repository inrep, long workId) throws RemoteException {
    super(a.id,in,inrep);
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Provides);
    stKindImpl = new StateKindImpl(StateKind.sk_work);
    prov = a.prov;
    normal = a;
    this.workId = workId;
  }

  public CDLType provide() throws RemoteException {
    return prov;
  }

  /* from interface TIRObject */
  public DefinitionKind get_def_kind() throws RemoteException {
    return (DefinitionKind) defKindImpl;
  }

  /* from interface TIRObject */
  public StateKind get_state() throws RemoteException {
    return (StateKind) stKindImpl;
  }

  public void tag(String t) throws RemoteException, TIRExceptCreate, TIRExceptLock {
    if (stKindImpl.value()!=StateKind.sk_work)
      throw new TIRExceptCreate("you can call create method on work object");
    super.tag(t);
  }

  public void save(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      int i;
      ((IdentificationImpl) id).save(st.curOutFile);
      Storage.writeCDLType(st, prov);
    } catch (IOException e) {
      throw new TIRExceptStorage("Can't write in file "+st.currentFile+".");
    }
  }

  public void load(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      int i;
      id = new IdentificationImpl();
      ((IdentificationImpl) id).load(st.curInFile);
      prov = Storage.readCDLType(st);
    } catch (IOException e) {
      throw new TIRExceptStorage("Can't write in file "+st.currentFile+".");
    }
  }

  public void postLoad(RepositoryImpl r) throws RemoteException, TIRExceptStorage {
    try {

      Container con;
      Contained[] rt;
      Contained ret;
      IdentificationImpl id;
      AbsoluteName nm;
      boolean found;
      if (prov.get_def_kind().value() == DefinitionKind.dk_none) {   // is tp NotYetInRep?
        nm = ((NotYetInRep) prov).nm;
  	con = r;  // begin search from repository
        for (int i=0;i<nm.size()-1;i++) {  // last element is name of object
  	  id = new IdentificationImpl(nm.elementAt(0), "::"+nm.elementAt(i),"");
          rt = ((SContainer)con).slookup_name(id);
  	  if (rt.length==0) throw new TIRExceptStorage("Can't find type " + nm.name());
            if (rt.length==1) {
              con = (Container) rt[0];
  	  } else { 
  	    found = false;
            id.version(((NotYetInRep) prov).id.version());
            int k = 0;
  	    for (k=0;k<rt.length;k++) {
  	      if (id.is_short_equal(rt[k].get_identification())) {
  	        found = true;
  		break;  // -- break for
  	      }
  	    } // -- end of for
  	    if (!found) throw new TIRExceptStorage("Can't find type " + nm.name());
  	    con = (Container) rt[k];
  	  }
  	}
  	if ((ret = ((SContainer)con).slookup(((NotYetInRep) prov).id))==null)
  	  throw new TIRExceptStorage("Can't find type " + nm.name());
  	prov = (CDLType) ret;
	if (prov.get_def_kind().value()==DefinitionKind.dk_Array ||
	    prov.get_def_kind().value()==DefinitionKind.dk_Sequence)
	  ((TIRImplObject) prov).postLoad(r);  
      }

    } catch (TIRExceptLock e) {;}
  }

  public boolean isNew() {
    return ((stKindImpl.value()==StateKind.sk_work) && normal==null);
  }

  public void canCommit() throws RemoteException, TIRExceptCommit {;}

  public void doCommit(Container in, Repository rep) throws RemoteException {
    if (stKindImpl.value()==StateKind.sk_normal)
      return;
    if (isNew()) {  
      stKindImpl.toNormal();
      switch (prov.get_def_kind().value()) {
        case DefinitionKind.dk_Array:
        case DefinitionKind.dk_Sequence:
        case DefinitionKind.dk_String:
        case DefinitionKind.dk_Wstring:
        case DefinitionKind.dk_Fixed:
          ((TIRImplObject)prov).doCommit(in,rep);
           break;
        default:
          Identification idl = ((Contained)prov).get_identification();
          prov = new NotYetInRep(idl.lang_absolute_name().name(), idl.name(), idl.version());
          break;
      }  
    } else
      normal = null;
  }

  public void doAbort(long workId) throws RemoteException { }
}
