/* $Id: WorkRepositoryImpl.java,v 1.2 2004/05/20 14:23:52 bures Exp $ */
package SOFA.SOFAnode.Made.TIR.Impl;
import java.rmi.RemoteException;
import java.rmi.server.ServerNotActiveException;
import java.rmi.server.UnicastRemoteObject;

import SOFA.SOFAnode.Made.TIR.ConstantDef;
import SOFA.SOFAnode.Made.TIR.Contained;
import SOFA.SOFAnode.Made.TIR.Container;
import SOFA.SOFAnode.Made.TIR.DefinitionKind;
import SOFA.SOFAnode.Made.TIR.EnumDef;
import SOFA.SOFAnode.Made.TIR.EnumMember;
import SOFA.SOFAnode.Made.TIR.ExprBinOperationDef;
import SOFA.SOFAnode.Made.TIR.ExprBoolean;
import SOFA.SOFAnode.Made.TIR.ExprChar;
import SOFA.SOFAnode.Made.TIR.ExprConstant;
import SOFA.SOFAnode.Made.TIR.ExprDefault;
import SOFA.SOFAnode.Made.TIR.ExprDouble;
import SOFA.SOFAnode.Made.TIR.ExprEnum;
import SOFA.SOFAnode.Made.TIR.ExprFixed;
import SOFA.SOFAnode.Made.TIR.ExprFloat;
import SOFA.SOFAnode.Made.TIR.ExprLong;
import SOFA.SOFAnode.Made.TIR.ExprLongDouble;
import SOFA.SOFAnode.Made.TIR.ExprLongLong;
import SOFA.SOFAnode.Made.TIR.ExprOctet;
import SOFA.SOFAnode.Made.TIR.ExprOperDef;
import SOFA.SOFAnode.Made.TIR.ExprProperty;
import SOFA.SOFAnode.Made.TIR.ExprShort;
import SOFA.SOFAnode.Made.TIR.ExprString;
import SOFA.SOFAnode.Made.TIR.ExprUnOperationDef;
import SOFA.SOFAnode.Made.TIR.ExprUnsigLong;
import SOFA.SOFAnode.Made.TIR.ExprUnsigLongLong;
import SOFA.SOFAnode.Made.TIR.ExprUnsigShort;
import SOFA.SOFAnode.Made.TIR.ExprWchar;
import SOFA.SOFAnode.Made.TIR.ExprWstring;
import SOFA.SOFAnode.Made.TIR.Identification;
import SOFA.SOFAnode.Made.TIR.ManageProfiles;
import SOFA.SOFAnode.Made.TIR.ManageRepository;
import SOFA.SOFAnode.Made.TIR.PrimitiveDef;
import SOFA.SOFAnode.Made.TIR.ProfileDef;
import SOFA.SOFAnode.Made.TIR.Repository;
import SOFA.SOFAnode.Made.TIR.StateKind;
import SOFA.SOFAnode.Made.TIR.TIRExceptCommit;
import SOFA.SOFAnode.Made.TIR.TIRExceptCreate;
import SOFA.SOFAnode.Made.TIR.TIRExceptLock;
import SOFA.SOFAnode.Made.TIR.TIRExceptSave;
import SOFA.SOFAnode.Made.TIR.TIRObject;
import SOFA.SOFAnode.Made.TIR.ValueGen;
import SOFA.SOFAnode.Made.TIR.WorkRepository;

public class WorkRepositoryImpl extends ContainerImpl implements WorkRepository, Repository {
  
  Lock lock; 
  RepositoryImpl normalRep;
  DefinitionKindImpl defKindImpl;
  StateKindImpl stKindImpl;
  //boolean hasCont; 
  PrimitiveDef[] primitives;

  ProfileDef actProfile;
  ManageProfilesImpl mProfiles;

  boolean commited;
  boolean aborted;

  long workId;         // id for create locking (id of "transaction")
  String workIdString; 
  boolean lockForMe;   // if normal rep. is locked for create for me

  LiIdItem firstId;  // for nextversions id
  LiIdItem lastId;
  int numOfIds;

  public WorkRepositoryImpl(RepositoryImpl nR, ProfileDef prof) throws RemoteException, TIRExceptCreate, TIRExceptLock {
    normalRep = nR;
    normalRep.lockProfiles.lock();
    lock = nR.lock;
    mProfiles = new ManageProfilesImpl(normalRep.mProfiles, this);
    if (prof == null) {
      actProfile = null;
    } else {
      ProfileDef a = mProfiles.lookup(prof.name());
      if (a==null) {   // no profile if looked up   ? it never should be
        normalRep.lockProfiles.unlock();
        throw new TIRExceptCreate("Bad profile");
      }
      if (((SProfileDef)a).isLocked()) {  // test, if profile is locked
        normalRep.lockProfiles.unlock();
        throw new TIRExceptCreate("Profile is locked");
      }
     
      if (!((SProfileDef)a).lock()) {  // lock profile
        normalRep.lockProfiles.unlock();
        throw new TIRExceptLock("Profile can not lock.");
      }  
      
      actProfile = a;
    }
    normalRep.lockProfiles.unlock();
    workId = nR.getUniqueNumber();
    try {
      workIdString = UnicastRemoteObject.getClientHost();
    } catch (ServerNotActiveException e) {
      workIdString = "unknown";
    }
    workIdString = workIdString + " : " + workId;
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_WorkRepository);
    stKindImpl = new StateKindImpl(StateKind.sk_work);
    //hasCont = false;
    primitives = new PrimitiveDef [17];
    for(int i=0;i<17;i++) {
      primitives[i] = new PrimitiveDefImpl(i,StateKind.sk_work);
    }
    commited = false;
    aborted = false;
    firstId = lastId = null;
    numOfIds = 0;
  }

  public Contained lookup(Identification id) throws RemoteException, TIRExceptLock {
    if ((stKindImpl.value()==StateKind.sk_work) && (!hasCont)) {
      fromNormRep();
    }
      Contained ret = null;
      LiItem akt = firstChild;
      for(int i=0;i<numOfItems;i++) {
        if (((SIdentification) ((Contained)akt.obj).get_identification()).language().compareTo(id.name())==0) {
          ret = (Contained) akt.obj;
	 break;
        } else {
          akt = akt.next;
        }
      }
      return ret;
  }

  public Contained lookup(String name, String version) throws RemoteException, TIRExceptLock {
    if ((stKindImpl.value()==StateKind.sk_work) && (!hasCont)) {
      fromNormRep();
    }
      Contained ret = null;
      LiItem akt = firstChild;
      for(int i=0;i<numOfItems;i++) {
        if (((SIdentification) ((Contained)akt.obj).get_identification()).language().compareTo(name)==0) {
          ret = (Contained) akt.obj;
	 break;
        } else {
          akt = akt.next;
        }
      }
      return ret;
  }

  public Contained[] contents(DefinitionKind type) throws RemoteException, TIRExceptLock {
    if ((stKindImpl.value()==StateKind.sk_work) && (!hasCont)) {
      fromNormRep();
    }
    return super.contents(type);   
  }

  public Contained[] lookup_name(Identification id) throws RemoteException, TIRExceptLock {
    if ((stKindImpl.value()==StateKind.sk_work) && (!hasCont)) {
      fromNormRep();
    }
    return super.lookup_name(id);
  }

  public Contained[] lookup_name(String name) throws RemoteException, TIRExceptLock {
    if ((stKindImpl.value()==StateKind.sk_work) && (!hasCont)) {
      fromNormRep();
    }
    return super.lookup_name(name);
  }

  public Contained lookup_tag(String name, String tag) throws RemoteException, TIRExceptLock {
    if ((stKindImpl.value()==StateKind.sk_work) && (!hasCont)) {
      fromNormRep();
    }
    return super.lookup_tag(name, tag);
  }

  public Contained[] lookup_branchtag(String name, String brtag) throws RemoteException, TIRExceptLock {
    if ((stKindImpl.value()==StateKind.sk_work) && (!hasCont)) {
      fromNormRep();
    }
    return super.lookup_branchtag(name, brtag);
  }

  public Contained lookup_lastinbranch(String name, String brtag) throws RemoteException, TIRExceptLock {
    if ((stKindImpl.value()==StateKind.sk_work) && (!hasCont)) {
      fromNormRep();
    }
    return super.lookup_lastinbranch(name, brtag);
  }

  public Contained lookup_lastfromversion(String name, String version) throws RemoteException, TIRExceptLock {
    if ((stKindImpl.value()==StateKind.sk_work) && (!hasCont)) {
      fromNormRep();
    }
    return super.lookup_lastfromversion(name, version);
  }

  public Contained[] scontents(DefinitionKind type) throws RemoteException, TIRExceptLock {
    if ((stKindImpl.value()==StateKind.sk_work) && (!hasCont)) {
      fromNormRep();
    }
    return super.scontents(type);
  }

  public Contained slookup(Identification id) throws RemoteException, TIRExceptLock {
    if ((stKindImpl.value()==StateKind.sk_work) && (!hasCont)) {
      fromNormRep();
    }
      Contained ret = null;
      LiItem akt = firstChild;
      for(int i=0;i<numOfItems;i++) {
        if (((SIdentification) ((Contained)akt.obj).get_identification()).language().compareTo(id.name())==0) {
          ret = (Contained) akt.obj;
	 break;
        } else {
          akt = akt.next;
        }
      }
      return ret;
  }

  public Contained slookup(String name, String version) throws RemoteException, TIRExceptLock {
    if ((stKindImpl.value()==StateKind.sk_work) && (!hasCont)) {
      fromNormRep();
    }
      Contained ret = null;
      LiItem akt = firstChild;
      for(int i=0;i<numOfItems;i++) {
        if (((SIdentification) ((Contained)akt.obj).get_identification()).language().compareTo(name)==0) {
          ret = (Contained) akt.obj;
	 break;
        } else {
          akt = akt.next;
        }
      }
      return ret;
  }

  public Contained[] slookup_name(Identification id) throws RemoteException, TIRExceptLock {
    if ((stKindImpl.value()==StateKind.sk_work) && (!hasCont)) {
      fromNormRep();
    }
      Contained[] ret = null;
      LiItem akt = firstChild;
      for(int i=0;i<numOfItems;i++) {
        if (((SIdentification) ((Contained)akt.obj).get_identification()).language().compareTo(id.name())==0) {
	  ret = new Contained [1];
          ret[0] = (Contained) akt.obj;
	 break;
        } else {
          akt = akt.next;
        }
      }
      return ret;
  }

  public Contained[] slookup_name(String name) throws RemoteException, TIRExceptLock {
    if ((stKindImpl.value()==StateKind.sk_work) && (!hasCont)) {
      fromNormRep();
    }
      Contained[] ret = null;
      LiItem akt = firstChild;
      for(int i=0;i<numOfItems;i++) {
        if (((SIdentification) ((Contained)akt.obj).get_identification()).language().compareTo(name)==0) {
	  ret = new Contained [1];
          ret[0] = (Contained) akt.obj;
	 break;
        } else {
          akt = akt.next;
        }
      }
      return ret;
  }

  public DefinitionKind get_spec_def_kind(int kind) throws RemoteException {
    return new DefinitionKindImpl(kind);
  }
  
  static public TIRObject newWorkFromNormal(TIRObject norm, Repository r, Container in, long workId) throws RemoteException {
    TIRObject ret=null;
    int val = norm.get_def_kind().value(); 
    switch (val) {
      case DefinitionKind.dk_CDLRepository:
        ret = new CDLRepositoryImpl((CDLRepositoryImpl) norm, r, workId);
        break;
      case DefinitionKind.dk_Module:
        ret = new ModuleDefImpl((ModuleDefImpl) norm,in,r,workId);
	break;
      case DefinitionKind.dk_Provider: 
        ret = new ProviderDefImpl((ProviderDefImpl)norm,in,r,workId);
	break;
      case DefinitionKind.dk_Struct:
        ret = new StructDefImpl((StructDefImpl)norm,in,r,workId);
	break;
      case DefinitionKind.dk_Exception:
        ret = new ExceptionDefImpl((ExceptionDefImpl)norm,in,r,workId);
	break;
      case DefinitionKind.dk_Enum:
        ret = new EnumDefImpl((EnumDefImpl)norm,in,r, workId);
	break;
      case DefinitionKind.dk_Union:
        ret = new UnionDefImpl((UnionDefImpl)norm,in,r,workId);
	break;
      case DefinitionKind.dk_Constant:
        ret = new ConstantDefImpl((ConstantDefImpl)norm,in,r, workId);
	break;
      case DefinitionKind.dk_Typedef:
        ret = new TypedefDefImpl((TypedefDefImpl)norm,in,r,workId);
	break;
      case DefinitionKind.dk_Interface:
        ret = new InterfaceDefImpl((InterfaceDefImpl)norm,in,r,workId);
	break;
      case DefinitionKind.dk_Frame:
        ret = new FrameDefImpl((FrameDefImpl)norm,in,r, workId);
	break;
      case DefinitionKind.dk_AModule:
        ret = new AModuleDefImpl((AModuleDefImpl)norm,in,r,workId);
	break;
      case DefinitionKind.dk_Architecture:
        ret = new ArchitectureDefImpl((ArchitectureDefImpl)norm,in,r, workId);
	break;
      case DefinitionKind.dk_Attribute:
        ret = new AttributeDefImpl((AttributeDefImpl)norm,in,r,workId);
	break;
      case DefinitionKind.dk_Operation:
        ret = new OperationDefImpl((OperationDefImpl)norm,in,r,workId);
	break;
      case DefinitionKind.dk_Inst:
        ret = new InstDefImpl((InstDefImpl)norm,in,r,workId);
	break;
      case DefinitionKind.dk_Property:
        ret = new PropertyDefImpl((PropertyDefImpl)norm,in,r,workId);
	break;
      case DefinitionKind.dk_Provides:
        ret = new ProvideDefImpl((ProvideDefImpl)norm,in,r,workId);
	break;
      case DefinitionKind.dk_Requires:
        ret = new RequireDefImpl((RequireDefImpl)norm,in,r,workId);
	break;
      case DefinitionKind.dk_StructMember:
        ret = new StructMemberImpl((StructMemberImpl)norm,in,r);
	break;
      case DefinitionKind.dk_UnionMember:
        ret = new UnionMemberImpl((UnionMemberImpl)norm,in,r);
	break;
      case DefinitionKind.dk_Bind:
        ret = new BindDefImpl((BindDefImpl)norm,in,r);
	break;
      case DefinitionKind.dk_EnumMember:
        ret = new EnumMemberImpl((EnumMemberImpl)norm);
	break;
    }
    return ret;
  }

  private void fromNormRep() throws RemoteException, TIRExceptLock {
    if (normalRep.lock.isLocked())
      throw new TIRExceptLock("Repository is locked");
    int i;
    LiItem akt;
    // if ((stKindImpl.value()==StateKind.sk_work) && (!hasCont)) {
      akt = normalRep.firstChild;
      for(i=0;i<normalRep.numOfItems;i++) {
        addListItem(newWorkFromNormal(akt.obj,this,this, workId));
	akt = akt.next;
      }
      hasCont = true;
    // }   
  }

  /* from interface TIRObject */
  public DefinitionKind get_def_kind() throws RemoteException {
    return (DefinitionKind) defKindImpl;
  }

  /* from interface TIRObject */
  public StateKind get_state() throws RemoteException {
    return (StateKind) stKindImpl;
  }

  public void canCommit() {;}
  public void doCommit(Container in, Repository rep) {;}
  public void doAbort(long workId) {;}
  public ManageRepository getManage() throws RemoteException { return null; }
  public WorkRepository[] getActiveWork() throws RemoteException { return null; }

  synchronized public boolean commit() throws RemoteException, TIRExceptCommit {
    if (commited || aborted)
      return false;
    lock.lock();
    LiItem akt;
    int i,j;

    akt = firstChild;
    for (i=0;i<numOfItems;i++) {
      if (((TIRImplObject)akt.obj).isNew()) {
        LiItem nak = normalRep.firstChild;
        for (j=0;j<normalRep.numOfItems;j++) {
          if (((SIdentification) ((Contained)nak.obj).get_identification()).is_short_name_equal(((Contained)akt.obj).get_identification())) {
            if (akt.obj.get_def_kind().value()!=nak.obj.get_def_kind().value())
              throw new TIRExceptCommit("Object with name \""+((SContained)nak.obj).get_absolute_name().name()+"\"exists.");
            if (((SIdentification) ((Contained)nak.obj).get_identification()).is_short_equal(((Contained)akt.obj).get_identification()))  
              throw new TIRExceptCommit("Object with identification \""+((Contained)nak.obj).get_identification()+"\"in container \"::\" exists.");
          }
          nak = nak.next;
        }
      }
      ((TIRImplObject)akt.obj).canCommit();
      akt = akt.next;
    }
    mProfiles.canCommit();
    
    akt = firstChild;
    boolean nn = false;
    for(i=0;i<numOfItems;i++) {
      if (((TIRImplObject)akt.obj).isNew()) 
        nn = true;
      ((TIRImplObject)akt.obj).doCommit(normalRep,normalRep);
      if (nn) {
        normalRep.addListItem(akt.obj);
      }  
      akt = akt.next;
      nn = false;
    }
    try {
      normalRep.postLoad(normalRep);
    } catch (TIRExceptStorage e) {
      throw new TIRExceptCommit("Some unexcepted error. Repository is in unstable state posibly.\n"+
                                 e.getMessage());
    }
    mProfiles.doCommit(normalRep, normalRep);
    
    if (lockForMe) {
      normalRep.lockFor = -1L;
      normalRep.lockCreate.unlock();
      lockForMe = false;
    }

    LiIdItem aktId = firstId;
    for(i=0;i<numOfIds;i++) {
      ((SIdentification)aktId.obj).setNextVersion();
      ((SIdentification)aktId.obj).unlock();
      aktId = aktId.next;
    }
    normalRep.removeWork(workId);
    commited = true;
    try {
      normalRep.commitSave();
    } catch (TIRExceptSave e) {
      lock.unlock();
      throw new TIRExceptCommit("Unsuccessful saving of repository.\n"+e.getMessage());     
    }
    lock.unlock();
    return true;
  }
  
  synchronized public boolean abort() throws RemoteException { 
    if (commited || aborted)
      return false;
    int i;  
    lock.lock();
    LiItem akt = firstChild;
    for(i=0;i<numOfItems;i++) {
      // !!! repaired after add CDLRepository
      // if (((TIRImplObject)akt.obj).isNew()) 
      ((TIRImplObject)akt.obj).doAbort(workId);
      akt = akt.next;
    }
    mProfiles.doAbort(workId);

    if (lockForMe) {
      normalRep.lockFor = -1L;
      normalRep.lockCreate.unlock();
      lockForMe = false;
    }
    
    LiIdItem aktId = firstId;
    for(i=0;i<numOfIds;i++) {
      ((SIdentification)aktId.obj).unlock();
      aktId = aktId.next;
    }
    normalRep.removeWork(workId);
    aborted = true;
    lock.unlock();
    return true;
  }

  /* from interface Repository */
  public Identification create_identification(String lang, String name, String version) throws RemoteException, TIRExceptCreate {
    if (name==null || version==null || lang ==null)
      return null;
    if (name.length()<3)
      throw new TIRExceptCreate("Bad name for identification.");
    if (name.charAt(0)!=':' && name.charAt(0)!=':')
      throw new TIRExceptCreate("You must give absolute name.");
    if (lang.compareTo("cdl")!=0)  // !!! change this
      throw new TIRExceptCreate("Bad language specification.");
    return (Identification) new IdentificationImpl(new String(lang), new String(name), new String(version));
  }

  /* from interface Repository */
  public Identification create_nextversion(Identification id) throws RemoteException, TIRExceptCreate  {
    //test same IP 
    String ver = id.version();
    StringBuffer ip = new StringBuffer();
    int i = 0;
    while ((i<ver.length()) && (ver.charAt(i)!='!')) {
      ip.append(ver.charAt(i));
      i++;
    }
  
    // update version number
    if (normalRep.sofaNodeName.compareTo(ip.toString())!=0)
      throw new TIRExceptCreate("You must create branch first.");
    if (((SIdentification)id).hasNextVersion())
      throw new TIRExceptCreate("Next version from this exists.");
    if (((SIdentification)id).isLocked())
      throw new TIRExceptCreate("Identification is locked for next version.");
    ((SIdentification)id).lock(workId);
    int last = ver.lastIndexOf('!');
    if (last != -1) {
      long a = 0;
      int j = last;
      last++;
      while ((last < ver.length())&&(Character.isDigit(ver.charAt(last)))) {
        a *= 10;
        a += (int) (ver.charAt(last)-'0');
	last++;
      }
      a++;
      
      String rver = new String(ver.substring(0,j+1) + Long.toString(a));
      Identification newId = new IdentificationImpl(((SIdentification) id).language(), id.absolute_name().name(), rver);

      LiIdItem item = new LiIdItem(id);
      if (firstId == null) {
        firstId = lastId = item;
      } else {
        LiIdItem akt = firstId;
        while (akt.next != null) akt = akt.next;
        akt.next = item;
        item.prev = akt;
        lastId = item;
      }
      numOfIds++;
      
      return newId;
    } else {
      ((SIdentification)id).unlock();
      throw new TIRExceptCreate("Version parse error.");
    }
  }

  public Identification create_initialversion(String lang, String name) throws RemoteException, TIRExceptCreate {
    if (name==null || lang == null)
      return null;
    if (name.length()<3)
      throw new TIRExceptCreate("Bad name for identification.");
    if (name.charAt(0)!=':' && name.charAt(0)!=':')
      throw new TIRExceptCreate("You must give absolute name.");
    if (lang.compareTo("cdl")!=0)  // !!! change this
      throw new TIRExceptCreate("Bad language specification.");
    return new IdentificationImpl(new String(lang), new String(name), normalRep.sofaNodeName+"!0");
  }

  public Identification create_branch(Identification from, String branchTag) throws RemoteException, TIRExceptCreate {
    if (branchTag==null || branchTag.length()==0)
      throw new TIRExceptCreate("You must give branchTag.");
    String ver = from.version();
    String rver = normalRep.sofaNodeName+"!"+ver+"!"+((SIdentification)from).nextNumBranch()+"!1";
    ((SIdentification)from).incNextNumBranch();
    return new IdentificationImpl(((SIdentification) from).language(), from.absolute_name().name(), rver, branchTag);
  }

  public PrimitiveDef get_primitive(int kind) throws RemoteException, TIRExceptCreate {
    if (kind>=0 && kind<=17) 
      return primitives[kind];
    else 
      return null;
  }  

/*
  public ModuleDef create_module(Identification id) throws RemoteException, TIRExceptCreate, TIRExceptLock {
    if (id!=null) {
      if (id.absolute_name().size()!=1)
        throw new TIRExceptCreate("Wrong identification for this object.");
      if (!getCreateLock())
        throw new TIRExceptLock("This part of repository is locked for creating.");
      if (!hasCont) {
        fromNormRep();
      }	
      id = new IdentificationImpl (id.absolute_name().name(),""); 
      Contained[] a = lookup_name(id);
      if (a.length != 0) {
        if (a[0].get_def_kind().value() != DefinitionKind.dk_Module)
	  throw new TIRExceptCreate("Name \""+id.name()+"\" exists in repository.");
	boolean found = false;  
	for (int i=0;i<a.length;i++) {
	  if (((SIdentification) a[i].get_identification()).is_short_equal(id)) {
	    found = true;
	    break;
	  }
	}
	if (found) 
	  throw new TIRExceptCreate("Object with same identification exists in repository.");
      }
      // creating new object
      ModuleDef ret = new ModuleDefImpl(id, this, this, StateKind.sk_work, workId);
      addListItem(ret); // adding object to list
      return ret;       // return object
    } else
      return null;
  }
  
  public ConstantDef create_constant(Identification id, CDLType type, ExprOperDef value) throws RemoteException, TIRExceptCreate, TIRExceptLock {
    if (id==null || type==null || value==null)
      return null;
    else {
      if (type.get_state().value()==StateKind.sk_work && 
          value.get_state().value()==StateKind.sk_work) {
        if (id.absolute_name().size()!=1)
          throw new TIRExceptCreate("Wrong identification for this object.");
        if (!getCreateLock())
          throw new TIRExceptLock("This part of repository is locked for creating.");
        if (!hasCont)
          fromNormRep();
        id = new IdentificationImpl(id);  
        Contained[] a = lookup_name(id.name());
        if (a.length != 0) {
          if (a[0].get_def_kind().value() != DefinitionKind.dk_Constant)
	    throw new TIRExceptCreate("Name \""+id.name()+"\" exists in repository.");
	  boolean found = false;  
	  for (int i=0;i<a.length;i++) {
	    if (((SIdentification) a[i].get_identification()).is_short_equal(id)) {
	      found = true;
	      break;
	    }
	  }
	  if (found) 
	    throw new TIRExceptCreate("Object with same identification exists in repository.");
        }
        // creating new object
        ConstantDef ret = new ConstantDefImpl(id, this, this,type, value, workId);
        addListItem(ret); // adding object to list
        return ret;       // return object
      } else {
        throw new TIRExceptCreate("Given objects aren't work.");
      }
    }
  }
  
  public StructDef create_struct(Identification id) throws RemoteException, TIRExceptCreate, TIRExceptLock {
    if (id!=null) {
      if (id.absolute_name().size()!=1)
        throw new TIRExceptCreate("Wrong identification for this object.");
      if (!getCreateLock())
        throw new TIRExceptLock("This part of repository is locked for creating.");
      if (!hasCont)
        fromNormRep();
      id = new IdentificationImpl(id);
      Contained[] a = lookup_name(id.name());
      if (a.length != 0) {
        if (a[0].get_def_kind().value() != DefinitionKind.dk_Struct)
	  throw new TIRExceptCreate("Name \""+id.name()+"\" exists in repository.");
	boolean found = false;  
	for (int i=0;i<a.length;i++) {
	  if (((SIdentification) a[i].get_identification()).is_short_equal(id)) {
	    found = true;
	    break;
	  }
	}
	if (found) 
	  throw new TIRExceptCreate("Object with same identification exists in repository.");
      }
      // creating new object
      StructDef ret = new StructDefImpl(id, this, this,workId);
      addListItem(ret); // adding object to list
      return ret;       // return object
    } else
      return null;
  }
    
  public UnionDef create_union(Identification id, CDLType switch_type) throws RemoteException, TIRExceptCreate, TIRExceptLock {
    if (id==null || switch_type==null)
      return null;
    else {
      if (switch_type.get_state().value()==StateKind.sk_work ) { 
        if (id.absolute_name().size()!=1)
          throw new TIRExceptCreate("Wrong identification for this object.");
        if (!getCreateLock())
          throw new TIRExceptLock("This part of repository is locked for creating.");
        if (!hasCont)
          fromNormRep();
        id = new IdentificationImpl(id);
        Contained[] a = lookup_name(id.name());
        if (a.length != 0) {
          if (a[0].get_def_kind().value() != DefinitionKind.dk_Union)
	    throw new TIRExceptCreate("Name \""+id.name()+"\" exists in repository.");
	  boolean found = false;  
	  for (int i=0;i<a.length;i++) {
	    if (((SIdentification) a[i].get_identification()).is_short_equal(id)) {
	      found = true;
	      break;
	    }
	  }
	  if (found) 
	    throw new TIRExceptCreate("Object with same identification exists in repository.");
        }
        // creating new object
        UnionDef ret = new UnionDefImpl(id, this, this, switch_type,workId);
        addListItem(ret); // adding object to list
        return ret;       // return object
      } else {
        throw new TIRExceptCreate("Given object isn't work.");
      }
    }
  }
    
  public EnumDef create_enum(Identification id) throws RemoteException, TIRExceptCreate, TIRExceptLock {
    if (id!=null) {
      if (id.absolute_name().size()!=1)
        throw new TIRExceptCreate("Wrong identification for this object.");
      if (!getCreateLock())
        throw new TIRExceptLock("This part of repository is locked for creating.");
      if (!hasCont)
        fromNormRep();
      id = new IdentificationImpl(id);
      Contained[] a = lookup_name(id.name());
      if (a.length != 0) {
        if (a[0].get_def_kind().value() != DefinitionKind.dk_Enum)
	  throw new TIRExceptCreate("Name \""+id.name()+"\" exists in repository.");
	boolean found = false;  
	for (int i=0;i<a.length;i++) {
	  if (((SIdentification) a[i].get_identification()).is_short_equal(id)) {
	    found = true;
	    break;
	  }
	}
	if (found) 
	  throw new TIRExceptCreate("Object with same identification exists in repository.");
      }
      // creating new object
      EnumDef ret = new EnumDefImpl(id, this, this, workId);
      addListItem(ret); // adding object to list
      return ret;       // return object
    } else
      return null;
  }
  
  public TypedefDef create_typedef(Identification id, CDLType orig_type) throws RemoteException, TIRExceptCreate, TIRExceptLock {
    if (id==null || orig_type==null)
      return null;
    else {
      if (orig_type.get_state().value()==StateKind.sk_work ) { 
        if (id.absolute_name().size()!=1)
          throw new TIRExceptCreate("Wrong identification for this object.");
        if (!getCreateLock())
          throw new TIRExceptLock("This part of repository is locked for creating.");
        if (!hasCont)
          fromNormRep();
        id = new IdentificationImpl(id);
        Contained[] a = lookup_name(id.name());
        if (a.length != 0) {
          if (a[0].get_def_kind().value() != DefinitionKind.dk_Typedef)
	    throw new TIRExceptCreate("Name \""+id.name()+"\" exists in repository.");
	  boolean found = false;  
	  for (int i=0;i<a.length;i++) {
	    if (((SIdentification) a[i].get_identification()).is_short_equal(id)) {
	      found = true;
	      break;
	    }
	  }
	  if (found) 
	    throw new TIRExceptCreate("Object with same identification exists in repository.");
        }
        // creating new object
        TypedefDef ret = new TypedefDefImpl(id, this, this, orig_type, workId);
        addListItem(ret); // adding object to list
        return ret;       // return object
      } else {
        throw new TIRExceptCreate("Given object isn't work.");
      }
    }
  }
  
  public ExceptionDef create_exception(Identification id) throws RemoteException, TIRExceptCreate, TIRExceptLock {
    if (id!=null) {
      if (id.absolute_name().size()!=1)
        throw new TIRExceptCreate("Wrong identification for this object.");
      if (!getCreateLock())
        throw new TIRExceptLock("This part of repository is locked for creating.");
      if (!hasCont)
        fromNormRep();
      id = new IdentificationImpl(id);
      Contained[] a = lookup_name(id.name());
      if (a.length != 0) {
        if (a[0].get_def_kind().value() != DefinitionKind.dk_Exception)
	  throw new TIRExceptCreate("Name \""+id.name()+"\" exists in repository.");
	boolean found = false;  
	for (int i=0;i<a.length;i++) {
	  if (((SIdentification) a[i].get_identification()).is_short_equal(id)) {
	    found = true;
	    break;
	  }
	}
	if (found) 
	  throw new TIRExceptCreate("Object with same identification exists in repository.");
      }
      // creating new object
      ExceptionDef ret = new ExceptionDefImpl(id, this, this, workId);
      addListItem(ret); // adding object to list
      return ret;       // return object
    } else
      return null;
  }

  public InterfaceDef create_interface(Identification id, InterfaceDef[] base_interfaces) throws RemoteException, TIRExceptCreate, TIRExceptLock {
    int i;
    if (id==null || base_interfaces==null) {
      return null;
    } else {  
      if (id.absolute_name().size()!=1)
        throw new TIRExceptCreate("Wrong identification for this object.");
      if (!getCreateLock())
        throw new TIRExceptLock("This part of repository is locked for creating.");
      if (!hasCont)
        fromNormRep();
      id = new IdentificationImpl(id);
      Contained[] a = lookup_name(id.name());
      if (a.length != 0) {
        if (a[0].get_def_kind().value() != DefinitionKind.dk_Interface)
	  throw new TIRExceptCreate("Name \""+id.name()+"\" exists in repository.");
	boolean found = false;  
	for (i=0;i<a.length;i++) {
	  if (((SIdentification) a[i].get_identification()).is_short_equal(id)) {
	    found = true;
	    break;
	  }
	}
	if (found) 
	  throw new TIRExceptCreate("Object with same identification exists in repository.");
      }
      // creating new object
      InterfaceDef[] bi = new InterfaceDef [base_interfaces.length];
      for (i=0;i<base_interfaces.length;i++) {
        if (base_interfaces[i].get_state().value()==StateKind.sk_work)
	  bi[i]=base_interfaces[i];
	else
	  throw new TIRExceptCreate("Given objects aren't work.");
      }
      InterfaceDef ret = new InterfaceDefImpl(id, this, this, bi, workId);
      addListItem(ret); // adding object to list
      return ret;       // return object
    } 
  }
  
  public FrameDef create_frame(Identification id) throws RemoteException, TIRExceptCreate, TIRExceptLock {
    if (id!=null) {
      if (id.absolute_name().size()!=1)
        throw new TIRExceptCreate("Wrong identification for this object.");
      if (!getCreateLock())
        throw new TIRExceptLock("This part of repository is locked for creating.");
      if (!hasCont)
        fromNormRep();
      id = new IdentificationImpl(id);
      Contained[] a = lookup_name(id.name());
      if (a.length != 0) {
        if (a[0].get_def_kind().value() != DefinitionKind.dk_Frame)
	  throw new TIRExceptCreate("Name \""+id.name()+"\" exists in repository.");
	boolean found = false;  
	for (int i=0;i<a.length;i++) {
	  if (((SIdentification) a[i].get_identification()).is_short_equal(id)) {
	    found = true;
	    break;
	  }
	}
	if (found) 
	  throw new TIRExceptCreate("Object with same identification exists in repository.");
      }
      // creating new object
      FrameDef ret = new FrameDefImpl(id, this, this, workId);
      addListItem(ret); // adding object to list
      return ret;       // return object
    } else
      return null;
  }   

  public ProviderDef  create_provider(Identification id) throws RemoteException, TIRExceptCreate, TIRExceptLock {
    if (id!=null) {
      if (id.absolute_name().size()!=1)
        throw new TIRExceptCreate("Wrong identification for this object.");
      if (!getCreateLock())
        throw new TIRExceptLock("This part of repository is locked for creating.");
      if (!hasCont)
        fromNormRep();
      id = new IdentificationImpl (id.absolute_name().name(),"");
      Contained[] a = lookup_name(id.name());
      if (a.length != 0) {
        if (a[0].get_def_kind().value() != DefinitionKind.dk_Provider)
	  throw new TIRExceptCreate("Name \""+id.name()+"\" exists in repository.");
	boolean found = false;  
	for (int i=0;i<a.length;i++) {
	  if (((SIdentification) a[i].get_identification()).is_short_equal(id)) {
	    found = true;
	    break;
	  }
	}
	if (found) 
	  throw new TIRExceptCreate("Object with same identification exists in repository.");
      }
      // creating new object
      ProviderDef ret = new ProviderDefImpl(id, this, this, StateKind.sk_work, workId);
      addListItem(ret); // adding object to list
      return ret;       // return object
    } else
      return null;
  } */

  public ExprBoolean create_exprboolean(boolean val) throws RemoteException, TIRExceptCreate {
    return new ExprBooleanImpl(val);
  }
  public ExprShort create_exprshort(short val) throws RemoteException, TIRExceptCreate {
    return new ExprShortImpl(val);
  }
  public ExprOctet create_exproctet(byte val) throws RemoteException, TIRExceptCreate {
    return new ExprOctetImpl(val);
  }

  public ExprLong create_exprlong(int val) throws RemoteException, TIRExceptCreate {
    return new ExprLongImpl(val);
  }
  public ExprLongLong create_exprlonglong(long val) throws RemoteException, TIRExceptCreate {
    return new ExprLongLongImpl(val);
  }
  public ExprUnsigShort create_exprunsigshort(short val) throws RemoteException, TIRExceptCreate {
    return new ExprUnsigShortImpl(val);
  }
  public ExprUnsigLong  create_exprunsiglong(int val) throws RemoteException, TIRExceptCreate {
    return new ExprUnsigLongImpl(val);
  }
  public ExprUnsigLongLong create_exprunsiglonglong(long val) throws RemoteException, TIRExceptCreate {
    return new ExprUnsigLongLongImpl(val);
  }
  public ExprFloat create_exprfloat(float val) throws RemoteException, TIRExceptCreate {
    return new ExprFloatImpl(val);
  }
  public ExprDouble create_exprdouble(double val) throws RemoteException, TIRExceptCreate {
    return new ExprDoubleImpl(val);
  }
  public ExprLongDouble create_exprlongdouble(double val) throws RemoteException, TIRExceptCreate {
    throw new TIRExceptCreate("No mapping for type \"long double\".");
  }
  public ExprFixed create_exprfixed(java.math.BigDecimal val, int digits, int scale) throws RemoteException, TIRExceptCreate {
    return new ExprFixedImpl(val, digits, scale);
  }
  public ExprChar    create_exprchar(char val) throws RemoteException, TIRExceptCreate {
    return new ExprCharImpl(val);
  }
  public ExprWchar   create_exprwchar(char val) throws RemoteException, TIRExceptCreate {
    return new ExprWcharImpl(val);
  }
  public ExprString  create_exprstring(String val) throws RemoteException, TIRExceptCreate {
    return new ExprStringImpl(val);
  }
  public ExprWstring create_exprwstring(String val) throws RemoteException, TIRExceptCreate {
    return new ExprWstringImpl(val);
  }
  public ExprConstant create_exprconstant(ConstantDef val) throws RemoteException, TIRExceptCreate {
    return new ExprConstantImpl(val);
  }
  
  public ExprProperty create_exprproperty(String pr) throws RemoteException, TIRExceptCreate {
    if (pr!=null && pr.length()!=0)
      return new ExprPropertyImpl(pr);
    return null;  
  }

  public ExprEnum create_exprenum(EnumDef en, String enMember) throws RemoteException, TIRExceptCreate, TIRExceptLock {
    if (en==null || enMember==null || enMember.length()==0)
      return null;
    EnumMember[] emm = en.members();
    boolean found = false;
    for(int i=0;i<emm.length;i++) {
      if (emm[i].name().compareTo(enMember)==0) {
        found = true;
        break;
      }
    }
    if (!found)
      throw new TIRExceptCreate("No member with name "+enMember+" in this enum.");
    return new ExprEnumImpl(en, enMember);
  }
  
  public ExprDefault  create_exprdefault() throws  RemoteException, TIRExceptCreate {
    return new ExprDefaultImpl(StateKind.sk_work);
  }
  
  public ExprUnOperationDef create_exprunoperation(int kind, ExprOperDef op) throws RemoteException, TIRExceptCreate {
    if (op.get_state().value()==StateKind.sk_work ) {
      return new ExprUnOperationDefImpl(kind, op);
    } else {
      throw new TIRExceptCreate("Given object isn't work.");
    }
  }
  public ExprBinOperationDef create_exprbinoperation(int kind, ExprOperDef op1, ExprOperDef op2) throws RemoteException, TIRExceptCreate {
    if (op1.get_state().value()==StateKind.sk_work &&
        op2.get_state().value()==StateKind.sk_work) {
       return new ExprBinOperationDefImpl(kind, op1,op2);
    } else {
      throw new TIRExceptCreate("Given objects aren't work.");
    }
  }
  public ValueGen create_valuegen() throws RemoteException, TIRExceptCreate {
    return new ValueGenImpl(StateKind.sk_work);
  }

  public WorkRepository beginChanges(ProfileDef prof) throws RemoteException {
    return null;
  }

  public void postLoad(RepositoryImpl r) throws RemoteException, TIRExceptStorage {;}
  public void save(Storage st) throws RemoteException, TIRExceptStorage {;} 
  public void load(Storage st) throws RemoteException, TIRExceptStorage {;}
  public void save() throws RemoteException, TIRExceptSave {;}

  public boolean isNew() {
    return true;
  }

  /* return workId for which is locked */
  public long isLockedForCreate()  throws RemoteException {
    if (normalRep.lockCreate.isLocked()) {
      return normalRep.lockFor;
    }
    return -1L;
  }

  /* search if exists "create lock" */
  public long canLock()  throws RemoteException {
    long a;
    if ((a=isLockedForCreate())!=-1L)  // if rep. is locked
      return a;
    // search lock in children
    LiItem akt = firstChild;
    for(int i=0;i<numOfItems;i++) {
      switch (akt.obj.get_def_kind().value()) {
        case DefinitionKind.dk_Module: 
          if ((a=((ModuleDefImpl)akt.obj).isLockedForCreate())!=-1L)
            return a;
          else {
            if ((a=((ModuleDefImpl)akt.obj).canLockChild())!=-1L)
              return a;
          }
          break;
        case DefinitionKind.dk_Provider:
          if ((a=((ProviderDefImpl)akt.obj).isLockedForCreate())!=-1L)
            return a;
          else {
            if ((a=((ProviderDefImpl)akt.obj).canLockChild())!=-1L)
              return a;
          }
          break;
      }
    }
    return -1L;
  }

  /* it is called from create_* methods */
  boolean getCreateLock() throws RemoteException {
    if (lockForMe)
      return true;
    if (normalRep.lockCreate.isLocked())
      return false;
    normalRep.lockGetCreate.lock();
    long a = canLock();
    if ((a==-1L) || (a==workId)) {
      lockForMe = true;
      normalRep.lockCreate.lock();
      normalRep.lockFor = workId;
    }  
    normalRep.lockGetCreate.unlock();
    return true;
  }

  public String workIdent() throws RemoteException {
    return workIdString;
  }

  public ProfileDef profile() throws RemoteException {
    return actProfile;
  }

  public ManageProfiles getProfiles() throws RemoteException {
    return mProfiles;
  }
}
