/* $Id: CorrectOrdinaryAppl.java,v 1.1 2004/03/22 19:31:49 hnetynka Exp $ */
package SOFA.SOFAnode.Made.Tools;

/** Program that corrects ordinary (non component-based) application in order to it can call
  * SOFA components. It augments java interfaces of components with correct versions.
  * <br>
  * <b>Arguments:</b><br>
  * Components that application uses and all class files of the application (all can be mixed).
  * 
  * @author Petr Hnetynka
  */
public class CorrectOrdinaryAppl {


  /** Main program. */
  public static void main(String[] argv) {

    java.util.Hashtable trTable = new java.util.Hashtable();
    java.util.Hashtable verTable = new java.util.Hashtable();
    java.util.ArrayList classes = new java.util.ArrayList();
    
    for (int i=0; i<argv.length; i++) {
      if (argv[i].indexOf('[') != -1) {
        // component name
        SOFA.SOFAnode.Run.Deployment.DeploymentDescriptor dd = getDeplDescr(argv[i], "index.dc");
        java.util.Hashtable ent = dd.getCDLEntities();
        for (java.util.Enumeration e = ent.keys(); e.hasMoreElements(); ) {
          String javaName = (String) e.nextElement();
          String fullName = (String) ent.get(javaName);
          String specVersion = fullName.substring(fullName.indexOf('?')+1);
          trTable.put(InstallToTR.toVMName(javaName), InstallToTR.toVMName(javaName+InstallToTR.specVersionToFileName(specVersion)));
          verTable.put(InstallToTR.toVMName(javaName), InstallToTR.specVersionToFileName(specVersion));
        }
      } else if (argv[i].endsWith(".class")) {
        // class name
        classes.add(argv[i]);
      } else {
        // unknown name
        System.out.println("Unrecognized type of name: "+argv[i]);
        System.exit(1);
      }
    }
    
    SOFA.Tools.Resolver.Resolver resolver = new SOFA.Tools.Resolver.Resolver(trTable);
    for (int i=0; i<classes.size(); i++) {
      try {
        String cur = (String) classes.get(i);
        System.out.print("Correcting class \""+cur+"\"...");
        String className = new SOFA.Tools.Resolver.ClassInfo(cur).className();
        java.io.FileInputStream fis = new java.io.FileInputStream(cur);
        byte [] b = resolver.process(fis);
        fis.close();
        String ver = (String) verTable.get(className);
        if (ver != null) {
          String old = cur;
          cur = cur.replaceFirst("\\.class", ver+".class");
          new java.io.File(old).delete();
        } 
        java.io.FileOutputStream fos = new java.io.FileOutputStream(cur);
        fos.write(b);
        fos.close();
        System.out.println("OK");
      } catch (java.io.IOException e) {
        System.out.println("IOException: "+e.getMessage());
        System.exit(1);
      }
    }
  }

  static SOFA.SOFAnode.Run.Deployment.DeploymentDescriptor getDeplDescr(String applname, String ddname) {
    try {
      String trString = System.getProperty("sofa.tr.url","file:/sofa/tr/");
      String path = trString+"impl";
      path = path + fullNameToFileName(applname, ddname);
      java.net.URL url = new java.net.URL(path);
java.net.URLConnection con = url.openConnection();
      java.io.InputStream is = con.getInputStream();
      SOFA.SOFAnode.Run.Deployment.DeploymentDescriptor dd = new SOFA.SOFAnode.Run.Deployment.DeploymentDescriptorImpl(is);
      is.close();
      return dd;
    } catch (java.net.MalformedURLException e) {
      System.out.println("MalformedURLException: "+e.getMessage());
      e.printStackTrace();
      System.exit(1);
    } catch (java.io.IOException e) {
      System.out.println("IOException: "+e.getMessage());
      System.exit(1);
    } catch (javax.xml.parsers.ParserConfigurationException e) {
      System.out.println("ParserConfigurationExceptioni: "+e.getMessage() );
      System.exit(1);
    } catch (org.xml.sax.SAXException e) {
      System.out.println("SAXException: "+e.getMessage() );
      System.exit(1);
    }
    return null;
  }

  private static String fullNameToFileName(String clname, String ddname) {
    clname = clname.replaceAll("::", "/");
    clname = clname.replace('[', '/');
    clname = clname.replace(']', '/');
    clname = clname + ddname;
    if (clname.charAt(0) != '/')
      clname = "/" + clname;
    return clname;
  }
}
