/* $Id: DeploymentDescriptorImpl.java,v 1.3 2004/06/11 09:33:59 bures Exp $ */
package SOFA.SOFAnode.Run.Deployment;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import SOFA.Util.Wildcards;

/** Implementation of deployment descriptor.
  *
  * @author Petr Hnetynka
  */
public class DeploymentDescriptorImpl implements DeploymentDescriptor {
  
  /** 'Root' of the DD */
  protected Element element;
  
  /** Implemenatation version */
  protected String releaseVersion;
  
  /** Instance ID. It is set by DeplDock in the <tt>instatiate</tt> method. */
  protected String instanceId;
  
  /** Constructor.
    * @param el   root element of the dd
    */
  public DeploymentDescriptorImpl(Element el) {
    element = el;
    initialize();
  }

  /** Constructor */
  public DeploymentDescriptorImpl() {
  }
  
  /** Constructor.
    * @param  fname   name of the file with depl. descr.
    */
  public DeploymentDescriptorImpl(String fname) throws java.io.IOException, ParserConfigurationException, SAXException {
    java.io.File file = new java.io.File(fname);
    if (!file.exists()) {
      throw new java.io.IOException("File \""+fname+"\" doesn't exist.");
    }

    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    DocumentBuilder builder = factory.newDocumentBuilder();
    Document document = builder.parse(file);
    element = document.getDocumentElement();
    element.normalize();
  }

  /** Constructor.
    * @param  is   file stream with depl. descr.
    */
  public DeploymentDescriptorImpl(java.io.InputStream is) throws java.io.IOException, ParserConfigurationException, SAXException {
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    DocumentBuilder builder = factory.newDocumentBuilder();
    Document document = builder.parse(is);
    element = document.getDocumentElement();
    element.normalize();
  }
  
  /** Initializing */
  private void initialize() {
    NodeList nl = element.getChildNodes();
    for (int i=0; i<nl.getLength(); i++) {
      Node n = nl.item(i);
      if ((n.getNodeType() == Node.ELEMENT_NODE) && (n.getNodeName().compareTo("version")==0)) {
        releaseVersion = n.getFirstChild().getNodeValue();
        break;
      }
    }
  }

  public boolean isSystem() {
    return ( element.getTagName().compareTo("sofa_system")==0);
  }
  
  public String getReleaseVersion() {
    return releaseVersion;
  }

  public String getItem(String name) {
    NodeList nl = element.getChildNodes();
    for (int i=0; i<nl.getLength(); i++) {
      Node n = nl.item(i);
      if ((n.getNodeType() == Node.ELEMENT_NODE) && (n.getNodeName().compareTo(name)==0)) {
        Node fn = n.getFirstChild();
        if (fn != null) 
          return fn.getNodeValue();
        else
          return "";
      }
    }
    return "";
  }
  
  public String getInstanceName() {
    return element.getAttribute("name");
  }
  public void setInstanceName(String name) {
    element.setAttribute("name", name);
  }
  
  public void setInstanceId(String id) {
    instanceId = id;
  }
  
  public String getInstanceId() { return instanceId;}
  
  public String getImplementationClass(String type) {
    NodeList nl = element.getChildNodes();
    for (int i=0; i<nl.getLength(); i++) {
      Node n = nl.item(i);
      if ((n.getNodeType() == Node.ELEMENT_NODE) && (n.getNodeName().compareTo("implementation")==0) && (((Element)n).getAttribute("type").compareTo(type)==0)) {
        Node fn = n.getFirstChild();
        if (fn != null) 
          return fn.getNodeValue();
        else
          return null;
      }
    }
    return null;
  }
  
  public String getProperty(String name) {
    NodeList nl = element.getChildNodes();
    for (int i=0; i<nl.getLength(); i++) {
      Node n = nl.item(i);
      if ((n.getNodeType() == Node.ELEMENT_NODE) && (n.getNodeName().compareTo("property")==0) && (((Element)n).getAttribute("name").compareTo(name)==0)) {
        return ((Element)n).getAttribute("value");
      }
    }
    return null;
  }
  
  public SOFA.SOFAnode.Run.Deployment.DeploymentDescriptor getSubComponentDeploymentDescriptor(String name) {
    // ??? clone returned nodes ???  
    NodeList nl = element.getChildNodes();
    for (int i=0; i<nl.getLength(); i++) {
      Node n = nl.item(i);
      if ((n.getNodeType() == Node.ELEMENT_NODE) && (n.getNodeName().compareTo("sofa_component")==0) && ( Wildcards.match(name, ((Element)n).getAttribute("name"))) ) {
        return new DeploymentDescriptorImpl((Element) n);
      } else {
        if ((n.getNodeType() == Node.ELEMENT_NODE) && (n.getNodeName().compareTo("unit")==0) ) {
          NodeList nl1 = n.getChildNodes();
          for (int j = 0; j < nl1.getLength(); j++) {
            Node n1 = nl1.item(j);
            if (n1.getNodeType() == Node.ELEMENT_NODE)
              if ((n1.getNodeName().compareTo("sofa_component")==0) && (Wildcards.match(name, ((Element)n1).getAttribute("name"))))
                return new DeploymentDescriptorImpl(((Element)n1));
          }
        }
      }
    }
    return null;
  }
  
  public String getSubComponentLocation(String name) {
    NodeList nl = element.getChildNodes();
    for (int i=0; i<nl.getLength(); i++) {
      Node n = nl.item(i);
      if ((n.getNodeType() == Node.ELEMENT_NODE) && (n.getNodeName().compareTo("sofa_component")==0) && ( Wildcards.match(name, ((Element)n).getAttribute("name"))) ) {
        return "sofa://local/local";
      } else {
        if ((n.getNodeType() == Node.ELEMENT_NODE) && (n.getNodeName().compareTo("unit")==0) ) {
          String location = null;
          NodeList nl1 = n.getChildNodes();
          for (int j = 0; j < nl1.getLength(); j++) {
            Node n1 = nl1.item(j);
            if (n1.getNodeType() == Node.ELEMENT_NODE) {
              if ((n1.getNodeName().compareTo("location")==0)) {
                location = n1.getFirstChild().getNodeValue();
              }
              if ((n1.getNodeName().compareTo("sofa_component")==0) && (Wildcards.match(name, ((Element)n1).getAttribute("name")))) {
                return location;
              }
            }
          }
        }
      }
    }
    return null;
  }

  
  public SOFA.Connector.DeploymentDescriptor getConnectorDeploymentDescriptor(String name) {
    NodeList nl = element.getChildNodes();
    for (int i=0; i<nl.getLength(); i++) {
      Node n = nl.item(i);
      if ((n.getNodeType() == Node.ELEMENT_NODE) && (n.getNodeName().compareTo("sofa_connector")==0) && ( Wildcards.match(name, ((Element)n).getAttribute("name"))) ) {
        if (((Element)n).hasAttribute("impl")) {
          try {
            return new SOFA.Connector.ECG.DeploymentDescriptor((Element) n);
          } catch (SOFA.Connector.DeploymentDescriptorException e) {
            return null;
          }
        }
      }
    }
    return null;
  }


  
  public java.util.Hashtable getCDLEntities() {
    NodeList nl = element.getChildNodes();
    for (int i=0; i<nl.getLength(); i++) {
      Node n = nl.item(i);
      if ((n.getNodeType() == Node.ELEMENT_NODE) && (n.getNodeName().compareTo("cdl_entities")==0)) {
        NodeList nl1 = n.getChildNodes();
        java.util.Hashtable ret = new java.util.Hashtable();
        for (int j=0; j<nl1.getLength(); j++) {
          Node n1 = nl1.item(j);
          if ((n1.getNodeType() == Node.ELEMENT_NODE) && (n1.getNodeName().compareTo("entity")==0)) {
            ret.put(((Element)n1).getAttribute("javaname"), ((Element)n1).getAttribute("cdlname"));
          }
        }
        return ret;
      }
    }
    return null;
  }

  public String getArchitectureAbsName() {
    String arch = null;
    NodeList nl = element.getChildNodes();
    for (int i=0; i<nl.getLength(); i++) {
      Node n = nl.item(i);
      if ((n.getNodeType() == Node.ELEMENT_NODE) && (n.getNodeName().compareTo("architecture_ref")==0)) {
        arch = n.getFirstChild().getNodeValue();
        break;
      }
    }
    int pos = arch.indexOf('?');
    return arch.substring(0,pos);
  }

  public String getArchitectureAbsNameWithVersion() {
    String arch = null;
    NodeList nl = element.getChildNodes();
    for (int i=0; i<nl.getLength(); i++) {
      Node n = nl.item(i);
      if ((n.getNodeType() == Node.ELEMENT_NODE) && (n.getNodeName().compareTo("architecture_ref")==0)) {
        arch = n.getFirstChild().getNodeValue();
        break;
      }
    }
    return arch;
  }

  public void _read(cz.cuni.sofa.lib.InputStream is) throws java.io.IOException {
    try {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      DocumentBuilder builder = factory.newDocumentBuilder();
      Document document = builder.parse(is);
      element = document.getDocumentElement();
      initialize();
    } catch (SAXException e) {
      throw new java.io.IOException("SAXException: "+e.getMessage());
    } catch (ParserConfigurationException e) {
      throw new java.io.IOException("ParserConfigurationException: "+e.getMessage());
    }
  }
  
  public void _write(cz.cuni.sofa.lib.OutputStream os) throws java.io.IOException {
    try {
      TransformerFactory tFactory = TransformerFactory.newInstance();
      Transformer transformer = tFactory.newTransformer();
      DOMSource source = new DOMSource(element);
      StreamResult result = new StreamResult(os);
      transformer.transform(source, result);
    } catch (TransformerConfigurationException e) {
      throw new java.io.IOException("TransformerConfigurationException: "+e.getMessage());
    } catch (TransformerException e) {
      throw new java.io.IOException("TransformerException: "+e.getMessage());
    }
  }
}
