/* $Id: UpdateTool.java,v 1.2 2004/05/20 14:23:52 bures Exp $ */
package SOFA.SOFAnode.Run;

import org.omg.CosNaming.NameComponent;

import SOFA.SOFAnode.Run.DeplDockRegistry.DeplDockRegistry;
import SOFA.SOFAnode.Run.DeplDockRegistry.DeplDockRegistryHelper;
import SOFA.SOFAnode.Run.DeplDockRegistry.NotFoundException;
import SOFA.SOFAnode.Run.Deployment.DeplDock;



/** Allows update components.
  *
  * @author Petr Hnetynka
  */
public class UpdateTool {
  
  /** Argument is name of the depldock.
    */
  public static void main(String[] argv) {
    if (argv.length != 1) {
      System.out.println("Bad number of arguments.");
      System.out.println("Specify name of the deployment dock");
      System.exit(1);
    }
    
    if (System.getSecurityManager() == null) {
      System.setSecurityManager(new java.rmi.RMISecurityManager());
    }

    SOFA.Util.CORBAAccess ca = new SOFA.Util.CORBAAccess();

    NameComponent nc = new NameComponent("DeplDockRegistry", "");
    NameComponent path[] = {nc};
    DeplDockRegistry ddr = null;
    try {
      System.out.print("Obtaining depldock registry...");
      ddr = DeplDockRegistryHelper.narrow(SOFA.Util.CORBAAccess.ncRef.resolve(path));
      System.out.println("OK");
    } catch (Exception e) {
      System.out.println("Can't get DeplDock registry.");
      e.printStackTrace();
      System.exit(1);
    }
    
    
    byte[] serRef = null;
    try {
      System.out.print("Obtaining reference to depldock...");
      serRef = ddr.lookup(argv[0]);
      System.out.println("OK");
    } catch (NotFoundException e) {
      System.out.println("No deployment dock with name \""+argv[1]+"\".");
      System.exit(1);
    }
    
    SOFA.Connector.Reference ddRef = SOFA.SOFAnode.Run.Deployment.DeplDockImpl.bytesToSofaReference(serRef);
   
    DeplDock ddock = null;
    try {
      System.out.print("Creating connector to depldock...");
      ddock = (DeplDock) SOFA.Connector.Boot.DeplDockConnector.createClt(ddRef);
      System.out.println("OK");
    } catch (SOFA.Connector.ConnectorException e) {
      System.out.println("Can't create connector to deployment dock.");
      System.exit(1);
    }
    
    String[] names = ddock.namesOfAllRunningComponents();
    SOFA.Connector.Reference[] refs = new SOFA.Connector.Reference [ names.length ];
    for (int i=0; i<names.length; i++) {
      try {
        refs[i] = ddock.getReference(names[i]);
      } catch (SOFA.SOFAnode.Run.Deployment.DeploymentException e) {
        System.out.println("DeploymentException: "+e.getMessage());
        e.printStackTrace();
        System.exit(1);
      }
    }

    SOFA.Component.DCUP.DCUPComponentManager[] aplcms = new SOFA.Component.DCUP.DCUPComponentManager[refs.length];
    for (int i=0; i<refs.length; i++) {
      try {
        aplcms[i] = (SOFA.Component.DCUP.DCUPComponentManager) SOFA.Connector.Boot.DCUPComponentManagerConnector.createClt(refs[i]);
      } catch (SOFA.Connector.ConnectorException e) {
        System.out.println("ConnectorException: "+e.getMessage());
        e.printStackTrace();
        System.exit(1);
      }
    }

    java.util.ArrayList updatableAplcms = new java.util.ArrayList ();
    java.util.ArrayList updatableNames = new java.util.ArrayList ();
    for (int i=0; i<aplcms.length; i++) {
      if (aplcms[i].isUpdatable()) {
        updatableAplcms.add(aplcms[i]);
        updatableNames.add(names[i]);
      }
    }

    if (updatableAplcms.size() == 0) {
      System.out.println("No updatable components in this depldock.");
      System.exit(0);
    }

    System.out.println("Choose component for update");
    System.out.println("===========================");
    for (int i=0;i<updatableNames.size(); i++) {
      System.out.print(i);
      System.out.print(" ... ");
      System.out.println(updatableNames.get(i)+" ("+((SOFA.Component.DCUP.DCUPComponentManager)updatableAplcms.get(i)).getFullName()+")");
    }
    System.out.print(updatableNames.size());
    System.out.println(" ... do nothing");
    int numOK = 0; 
    try {
      java.io.BufferedReader in = new java.io.BufferedReader(new java.io.InputStreamReader(System.in));
      String ln = in.readLine();
      ln.trim();
      int num = Integer.parseInt(ln);
      if (num<0 || num>=(updatableNames.size()+1)) {
        System.out.println("Bad number");
        System.exit(1);
      }
      if (num == updatableNames.size())
        System.exit(0);
      numOK = num;
    } catch (Exception e) {
      System.out.println("Exception: "+e.getMessage());
      e.printStackTrace();
      System.exit(1);
    }
    
    SOFA.Component.DCUP.DCUPComponentManager dcm = (SOFA.Component.DCUP.DCUPComponentManager) updatableAplcms.get(numOK);
    String fullName = dcm.getFullName();
    String version =  getVersionFromFullName(fullName);
    String name = getNameFromFullName(fullName);

    String trString = System.getProperty("sofa.tr.url","file:/sofa/tr/");
    String trPath = trString+"/impl/";
    java.util.Properties props = new java.util.Properties();
    try {
      java.net.URL url = new java.net.URL(trPath+"/info");
      java.io.InputStream uis = url.openStream();
      props.load(uis);
      uis.close();
    } catch (java.net.MalformedURLException e) {
      System.out.println("MalformedURLException: "+e.getMessage());
      e.printStackTrace();
      System.exit(1);
    } catch (java.io.IOException e) {
      System.out.println("IOException: "+e.getMessage());
      System.exit(1);
    }

    if (props.size() == 1) {
      System.out.println("No other versions of the component for update");
      System.exit(0);
    }

    String[] vers = new String [props.size()-1];
    int i = 0;
    System.out.println("Choose new component to update");
    System.out.println("==============================");
    for (java.util.Enumeration e = props.keys(); e.hasMoreElements(); ) {
      String v = (String) e.nextElement();
      if (v.compareTo(fullName) != 0) {
        System.out.print(i);
        System.out.print(" ... ");
        System.out.println(v);
        vers[i++] = v;
      }
    }
    System.out.print(i);
    System.out.println(" ... do nothing");
    numOK = 0; 
    try {
      java.io.BufferedReader in = new java.io.BufferedReader(new java.io.InputStreamReader(System.in));
      String ln = in.readLine();
      ln.trim();
      int num = Integer.parseInt(ln);
      if (num<0 || num>=(vers.length+1)) {
        System.out.println("Bad number");
        System.exit(1);
      }
      if (num == vers.length)
        System.exit(0);
      numOK = num;
    } catch (Exception e) {
      System.out.println("Exception: "+e.getMessage());
      e.printStackTrace();
      System.exit(1);
    }
    
    System.out.print("Obtaining and parsing new deployment descriptor...");
    SOFA.SOFAnode.Run.Deployment.DeploymentDescriptor depldesc = TRLauncher.getDeplDescr(vers[numOK], "index.dc");
    System.out.println("OK");

    System.out.print("Updating component...");
    try {
      dcm.updateComponent(depldesc, new SOFA.SOFAnode.Run.StorageImpl());
    } catch (SOFA.Component.ComponentLifecycleException e) {
      System.out.println("ComponentLifecycleException: "+e.getMessage());
      e.printStackTrace();
      System.exit(1);
    }
    System.out.println("OK");
    
  }


  private static String getVersionFromFullName(String name) {
    return name.substring(name.indexOf('[')+1, name.indexOf(']'));
  }

  private static String getNameFromFullName(String name) {
    return name.substring(0, name.indexOf('['));
  }

  private static String nameToFileName(String name) {
    return name.replaceAll("::", "/");
  }

  
}
