/* $Id: AssemblyDescriptorImpl.java,v 1.2 2004/05/20 14:23:52 bures Exp $ */
package SOFA.SOFAnode.TR.Impl;

import org.w3c.dom.*;

import javax.xml.parsers.*;

import org.xml.sax.*;

import java.util.*;
import java.io.IOException;
import java.io.File;

/** The assembly descriptor. Greatly inspired by the implementation of
 * {@link SOFA.SOFAnode.Run.Deployment.DeploymentDescriptorImpl}. It contains assembly information
 * about one SOFA component. This information is always stored in an XML file of the name
 * <em>index.dc</em>.
 *
 * @author Petr Panuska
 */
class AssemblyDescriptorImpl {

  /**
   * The root element of the document.
   */
  private Element element;

  /**
   * The document.
   */
  private Document document;

  /**
   * The component implemenatation version.
   */
  private String version;

  /**
   * Creates the assembly descriptor from the given file.
   * @param  file the file containing the assembly descriptor.
   * @throws IOException if some I/O operation fails.
   * @throws ParserConfigurationException if the component assembly file is corrupted or missing.
   * @throws SAXException if the component assembly file is corrupted or missing.
   *
   */
  public AssemblyDescriptorImpl (File file) throws IOException, ParserConfigurationException, SAXException {
    if (!file.exists()) {
      throw new IOException("File \"" + file.getName() + "\" doesn't exist.");
    }

    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    DocumentBuilder builder = factory.newDocumentBuilder();
    document = builder.parse(file);
    element = document.getDocumentElement();
    element.normalize();
    initialize();
  }

  /**
   * Initializes this descriptor.
   */
  private void initialize () {
    NodeList nl = element.getChildNodes();
    for (int i = 0; i < nl.getLength(); i++) {
      Node n = nl.item(i);
      if ((n.getNodeType() == Node.ELEMENT_NODE) && (n.getNodeName().compareTo("version") == 0)) {
        version = n.getFirstChild().getNodeValue();
        break;
      }
    }
  }

  /**
   * Gets the component implementation version.
   * @return the component implementation version.
   */
  public String getImplementationVersion () {
    return version;
  }

  /**
   * Gets a map of all interface types (structures, etc.) the component needs. Every key in the map
   * is the full Java name of the class implementing this type, the associated value is the full
   * interface name (::provider::name?specificationVersion).
   * @return a map containing all interfaces this component needs (types used in method parameters or
   * return values).
   */
  public Map getCDLEntities () {
    NodeList nl = element.getChildNodes();
    for (int i = 0; i < nl.getLength(); i++) {
      Node n = nl.item(i);
      if ((n.getNodeType() == Node.ELEMENT_NODE) && (n.getNodeName().compareTo("cdl_entities") == 0)) {
        NodeList nl1 = n.getChildNodes();
        HashMap ret = new HashMap();
        for (int j = 0; j < nl1.getLength(); j++) {
          Node n1 = nl1.item(j);
          if ((n1.getNodeType() == Node.ELEMENT_NODE) && (n1.getNodeName().compareTo("entity") == 0)) {
            ret.put(((Element) n1).getAttribute("javaname"), ((Element) n1).getAttribute("cdlname"));
          }
        }
        return ret;
      }
    }
    return null;
  }

  /**
   * Gets a set of all sub-components of this component. Every value in this set is a fullname
   * of a component.
   * @return a set of all sub-components.
   */
  public Set getSubComponents () {
    Set set = new HashSet();
    NodeList nl = document.getElementsByTagName("component_ref");
    int length = nl.getLength();
    for (int i = 0; i < length; i++) {
      assert(!(nl.item(i) instanceof Element));
      Element el = (Element) nl.item(i);
      set.add(el.getAttribute("arch") + "[" + el.getAttribute("version") + "]");
    }
    return set;
  }

}
