/*
 * $Id: IDFSRExtension.java,v 1.2 2005/07/08 12:04:11 kofron Exp $
 *
 * Copyright 2004
 *    Distributed Systems Research Group
 *    Department of Software Engineering
 *    Faculty of Mathematics and Physics
 *    Charles University, Prague
 *
 * Copyright 2005
 *    Formal Methods In Software Engineering Group
 *    Institute of Computer Science
 *    Academy of Sciences of the Czech Republic
 * 
 * This code was developed by Jan Kofron <kofron@nenya.ms.mff.cuni.cz>
 */

package SOFA.SOFAnode.Util.DFSRChecker.DFSR;

import SOFA.SOFAnode.Util.DFSRChecker.node.InvalidParameterException;
import SOFA.SOFAnode.Util.DFSRChecker.state.State;


/**
 * This interface defines the methods to be implemented by all extensions used by the DFSRtraverser.
 */
public interface IDFSRExtension {

    /**
     * This method is called as the first each time a node is visited.
     */
    public void action(State state) throws InvalidParameterException;

    /**
     * This method is called if a new state is visited.
     * 
     * @return true or false depending on the implementation
     */
    public boolean actionNew(State state) throws InvalidParameterException, CheckingException;

    /**
     * This method is called if a cycle is detected while traversing the state graph. 
     * Note that the actionVisited is not called.
     */
    public void actionCycle(State state) throws InvalidParameterException;

    /**
     * This method is called if an already visited node is being visited. Note
     * that ActionCycle method is not called.
     */
    public void actionVisited(State state) throws InvalidParameterException;

    /**
     * This method is called while returning from the state back to its parent.
     */
    public void actionBack(State state) throws InvalidParameterException;
    
    /**
     * resets the state of the extension to the beginning
     */
    public void reset();
    
}