/*
 * $Id: AtomicNode.java,v 1.1 2005/07/13 07:27:49 kofron Exp $
 * 
 * Copyright 2004
 *    Distributed Systems Research Group
 *    Department of Software Engineering
 *    Faculty of Mathematics and Physics
 *    Charles University, Prague
 *
 * Copyright 2005
 *    Formal Methods In Software Engineering Group
 *    Institute of Computer Science
 *    Academy of Sciences of the Czech Republic
 * 
 * This code was developed by Jan Kofron <kofron@nenya.ms.mff.cuni.cz>
 */

package SOFA.SOFAnode.Util.DFSRChecker.node;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.TreeSet;


import SOFA.SOFAnode.Util.DFSRChecker.DFSR.CheckingException;
import SOFA.SOFAnode.Util.DFSRChecker.state.SimpleState;
import SOFA.SOFAnode.Util.DFSRChecker.state.State;
import SOFA.SOFAnode.Util.DFSRChecker.state.TransitionPair;
import SOFA.SOFAnode.Util.DFSRChecker.state.TransitionPairs;
import SOFA.SOFAnode.Util.DFSRChecker.utils.AnotatedProtocol;


/**
 * This class represents an atomic action inside of a parse tree. 
 */
public class AtomicNode extends TreeNode {

    /**
     * Creates a new instance of AtomicNode.
     * 
     * @param events the events forming this atomic action
     * @param repository the action repository
     */
    public AtomicNode(int eventindex, TreeSet events, ActionRepository repository) {
        super(getProtocol(events, repository));

        this.eventIndex = eventindex;
        this.events = events;  
        this.initState = null;
        this.initTransitions = new TransitionPairs(new TransitionPair[1]);
        this.initTransitions.transitions[0] = null;
        this.secondTransitions = new TransitionPairs(new TransitionPair[0]);
        this.nodes = new TreeNode[0];
        this.weight = 2;
        this.repository = repository;
    }

    /**
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.node.TreeNode#getWeight()
     */
    public long getWeight() {
        return 2;
    }

    /**
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.node.TreeNode#getInitial()
     */
    public State getInitial() {
        if (initState == null) {
            initState = new SimpleState(0);
            this.initTransitions.transitions[0] = new TransitionPair(eventIndex, new SimpleState(1));
        }
        
        return initState;
    }

    /**
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.node.TreeNode#isAccepting(SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.state.State)
     */
    public boolean isAccepting(State state) {
        return (((SimpleState) state).state == 1);
    }

    /**
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.node.TreeNode#getTransitions(SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.state.State)
     */
    public TransitionPairs getTransitions(State state) throws InvalidParameterException, CheckingException {
        if (((SimpleState) state).state == 0)
            return initTransitions;
        else if (((SimpleState) state).state == 1)
            return secondTransitions;
        else
            throw new InvalidParameterException();
    }

    /**
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.node.TreeNode#getTypeName()
     */
    public String[] getTypeName() {
        String[] result = { "AtomicAction", "atomic action" };
        return result;
    }
    
    
    public TreeNode forwardCut(TreeSet livingevents) {
        return this;
    }

    /**
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.node.TreeNode#getAnotatedProtocol(SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.state.State)
     */
    public AnotatedProtocol getAnotatedProtocol(State state) {
        SimpleState sstate = (SimpleState) state;
        String result = new String();
        ArrayList indicesresult = new ArrayList();
        
        result = protocol;
        if (state != null)
            if (((SimpleState)state).state == 0)
                indicesresult.add(new Integer(0));
            else
                indicesresult.add(new Integer(result.length()));
        
        return new AnotatedProtocol(result, indicesresult);
    }
    
    /**
     * Creates the protocol (i.e. [_actionlist]) string
     * @param events the events creating this atomic action
     * @param repository the action repository
     * @return the protocol describing this aotmic action
     */
    private static String getProtocol(TreeSet events, ActionRepository repository) {
        String p = new String();
         
        Iterator it = events.iterator();
        
        p = repository.getItemString(((Integer)it.next()).intValue());
        
        while (it.hasNext())
            p = p + ", " + repository.getItemString(((Integer)it.next()).intValue());
            
        p = "[" + p + "]";
        
        return p;
    }
    
    
    /**
     * The index of thic atomic action
     */
    private int eventIndex;
    
    /**
     * the events forming this atomic action
     */
    private TreeSet events;
    
    /**
     * Simple state for initial state representation
     */
    private SimpleState initState;
    
    /**
     * The action repository
     */
    private ActionRepository repository;

    /**
     * Transitions for initial state
     */
    final private TransitionPairs initTransitions;

    /**
     * Transitions for the second state
     */
    final private TransitionPairs secondTransitions;

    
}
