/* 
 * $Id: Debug.java,v 1.4 2005/07/08 12:04:12 kofron Exp $ 
 *
 * Copyright 2004
 *    Distributed Systems Research Group
 *    Department of Software Engineering
 *    Faculty of Mathematics and Physics
 *    Charles University, Prague
 * 
 * This code was implemented by Stanislav Visnovsky <visnovsky@nenya.ms.mff.cuni.cz>
 */




package SOFA.SOFAnode.Util.DFSRChecker.parser;

/**
 * The debug enabling class. If the {@link Debug#debug}is set to number higher
 * than 0, all methods prints the arguments, otherwise the output is skipped.
 * The set of methods allowing specification of a debug level to which the
 * message is intended is also provided. Supposed use of debug level: 0 - no
 * debug info 1 - only high level debug about progress 2 - progress of
 * complicated methods 3 - all debug info
 * 
 */
public class Debug {
    /**
     * Indication, if the debugging stuff should be send to the standard output
     */
    public static int debug = 0;

    /**
     * Print a string to the standard output and the new line.
     * 
     * @param a
     *            a string to print
     */
    public static void println(String a) {
        if (debug > 0)
            System.out.println(a);
    }

    public static void println(int level, String a) {
        if (debug >= level)
            System.out.println(a);
    }

    /**
     * Print a new line to the standard output.
     * 
     */
    public static void println() {
        if (debug > 0)
            System.out.println();
    }

    public static void println(int level) {
        if (debug >= level)
            System.out.println();
    }

    /**
     * Print an integer to the standard output and the new line.
     * @param a
     *            an integer to print
     * @param level
     *            debug level the message should be printed at
     */
    public static void println(int level, int a) {
        if (debug >= level)
            System.out.println(a);
    }

    /**
     * Print a string to the standard output.
     * @param a
     *            a string to print
     */
    public static void print(String a) {
        if (debug > 0)
            System.out.print(a);
    }

    public static void print(int level, String a) {
        if (debug >= level)
            System.out.print(a);
    }

    /**
     * Print an integer to the standard output.
     * 
     * @param a
     *            an integer to print
     * @param level
     *            debug level the message should be printed at
     */
    public static void print(int level, int a) {
        if (debug >= level)
            System.out.print(a);
    }

    /**
     * Sets the debug level to the specified value
     * 
     * @param level
     *            the new debug level
     */
    public static void setLevel(int level) {
        debug = level;
    }

    /**
     * Returns the current debug level
     * 
     * @return the current debug level
     */
    public static int getLevel() {
        return debug;
    }
}