/* $Id: machineImpl.java,v 1.2 2004/05/20 14:23:53 bures Exp $ */

package SOFA.SOFAnode.Util;

import java.util.Collection;
import java.util.Comparator;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.TreeMap;
import java.util.Vector;


/**
 * The implementation class for automata. 
 *
 * @author 	Stanislav Visnovsky
 * @version 	1.0.0
*/
public class machineImpl implements Machine, Printer {

/**
 * list of states
*/
    public IntVector states[][];
    public boolean marked[];

    private int allocated;

/**
 * reference to the starting state
*/
    private int start;

/**
 * the list of accepting states
*/
    private Hashtable stop;

/**
 * the list of update states
*/
    private Hashtable update;

/**
 * ID counter for new states
*/
    public int nextID;

/**
 * Constructor for an automaton. It just sets defaults, i.e., no states at all.
 *
 * @author	Stanislav Visnovsky
 * @version	1.0.0
*/
    public machineImpl() {
        int i,j;

	states = new IntVector[10][EdgeFactory.getActionTokensNum()];
	nextID = 0;
	allocated = 10; 
        stop = new Hashtable(); 
	update = new Hashtable();
        start = 0;
	marked = new boolean[allocated];
    }

    public machineImpl(int size) {
        int i,j;

	Debug.println(2, "Initial size "+size+" states (no resize)");
	states = new IntVector[size][EdgeFactory.getActionTokensNum()];
	nextID = 0;
	allocated = size; 
        stop = new Hashtable(); 
	update = new Hashtable();
        start = 0;
	marked = new boolean[allocated];
	if( maxstates < allocated ) maxstates = allocated;
    }
/**
 * Adds a new state to the automaton.
 *
 * @author 	Stanislav Visnovsky
 * @version 	1.0.0
 * @param n	the new state to be added
*/
    public void AddState( IntVector n[] ){
	if( nextID == allocated ) Resize( allocated+5 );
        states[ nextID ] = n;
        nextID++;
    }


/**
 * Eliminates all unreachable states. It marks all reachable states by BFS algorithm and
 * then eliminates all unmarked states.
 *
 * @author 	Stanislav Visnovsky
 * @version 	1.0.0
 * @see State#Mark
*/
    public void Reachable() {

	int a,t,tgt,st;

	TidyUp();

	Debug.println(3, "Cleaned");

	LinkedList buffer = new LinkedList();
	buffer.add( new Integer(start) );
	while( !buffer.isEmpty() ) {
	    a = ((Integer)buffer.removeFirst()).intValue();
	    if( !marked[a] ) {
		marked[a] = true;
		for( t = 0; t< EdgeFactory.getActionTokensNum(); t++) {
		    IntVector edges = states[a][t];
		    if( edges != null ) 
		        for( tgt = 0 ; tgt < edges.used ; tgt++ )
		            if( !marked[ edges.data[tgt] ] ) buffer.add( new Integer( edges.data[tgt] ) );
		}
	    }
	}

	Debug.println(3, "Reachable found, creating mapping"); 

	int map[] = new int[ nextID ];
	IntVector[][] newstates = new IntVector[ nextID ][ EdgeFactory.getActionTokensNum()];
	
	int mapnext = 0;

	for( a = 0 ; a < nextID ; a++ )
	    if( marked[a] ) map[a] = mapnext++;

	Debug.println(3, "Reachable mapping found"); 

	for( a = 0 ; a < nextID ; a++ )
	    if( marked[a] ) {
		for( t = 0; t < EdgeFactory.getActionTokensNum(); t++ )
		if( states[a][t] != null ) {
		    if( newstates[map[a]][t] == null ) newstates[map[a]][t] = new IntVector();
		    for( tgt = 0; tgt < states[a][t].used; tgt++ )
			newstates[map[a]][t].sortedAddIntIfNExists( map[ states[a][t].data[tgt] ] );
		}
	    } else Debug.println(2,"Reachable deletes "+a);

	Hashtable newstop = new Hashtable();
	for( Iterator i = stop.values().iterator(); i.hasNext(); ) {
	    st = ((Integer)i.next()).intValue();
	    if( marked[ st ] ) {
		Integer newst = new Integer( map[st] );
		newstop.put( newst, newst );
	    }
        }

	Hashtable newupdate = new Hashtable();
	for( Iterator i = update.values().iterator(); i.hasNext(); ) {
	    st = ((Integer)i.next()).intValue();
	    if( marked[ st ] ) {
		Integer newst = new Integer( map[st] );
		newupdate.put( newst, newst );
	    }
        }

	states = newstates;
	stop = newstop;
	update = newupdate;
	start = map[start];

	allocated = nextID;
	
	marked = new boolean[allocated];
	
	nextID = mapnext;

	if( maxstates < allocated ) maxstates = allocated;

	Print(2);
    }

/**
 * Adds new states to the automaton. It also must set IDs. However, edges are not added
 *
 * @author 	Stanislav Visnovsky
 * @version 	1.0.0
 * @param n	a list of new states to be added
*/
    public int[] AddStates( IntVector n[][], int size ){

	int a,t,tgt;

	int map[] = new int[ size ];

	if( allocated <= size+nextID ) Resize( size+nextID );

	int mapnext = nextID;

	for( a = 0 ; a < size ; a++ )
	    map[a] = nextID++;

	for( a = 0 ; a < size ; a++ ) {
		for( t = 0; t < EdgeFactory.getActionTokensNum(); t++ )
		if( n[a][t] != null ) {
		    if( states[map[a]][t] == null ) states[map[a]][t] = new IntVector();
		    for( tgt = 0; tgt < n[a][t].used; tgt++ )
			states[map[a]][t].sortedAddIntIfNExists( map[ n[a][t].data[tgt] ] );
		}
	    } 
	return map;
    }

/**
 * Adds a new accepting state of the automaton.
 *
 * @author 	Stanislav Visnovsky
 * @version 	1.0.0
 * @param n	the new accepting state
*/
    public void AddStop( int n ){
        Integer nn = new Integer(n);
	stop.put( nn, nn );
    }

    public int addState() {
	if( nextID == allocated ) Resize( allocated+5 );
        return nextID++;
    }

/**
 * Adds a list of new accepting states of the automaton.
 *
 * @author 	Stanislav Visnovsky
 * @version 	1.0.0
 * @param n	a list of new accepting states
*/
    public void AddStops( Collection n ){
	Integer a;
        Iterator i = n.iterator();
	while( i.hasNext() ) {
	    a = (Integer)i.next();
	    stop.put( a, a );
	}
    }

/**
 * Sets a new starting state of the automaton.
 *
 * @author 	Stanislav Visnovsky
 * @version 	1.0.0
 * @param n	the new starting state
*/
    public void SetStart( int n ){
        start = n;
    }

/**
 * Print the debugging info through Debug class.
 *
 * @author 	Stanislav Visnovsky
 * @version 	2.0.0
 *
 * @see Debug
*/
    public void Print(int level) {
	if( level > Debug.debug ) return;
        Debug.println(level,"Machine:");
        Debug.print(level,"Start:");
        Debug.println(level, start );
        Debug.print(level,"Stop:");
        Iterator i = stop.values().iterator();
        while( i.hasNext() ) {
            Debug.print(level, ((Integer) i.next()).intValue());
            Debug.print(level," , ");
        }
        Debug.print(level,"Update:");
        i = update.values().iterator();
        while( i.hasNext() ) {
            Debug.print(level, ((Integer) i.next()).intValue());
            Debug.print(level," , ");
        }
        Debug.println(level);
        Debug.println(level,"States:");
	for ( int j = 0 ; j < nextID ; j++ ) {
	    Debug.println( level, j );
	    for( int k = 0; k < EdgeFactory.getActionTokensNum(); k++ )
		if( states[j][k] != null && states[j][k].used > 0 )
		Debug.println( "--" + EdgeFactory.ActionTokens[k].name+"-->" + states[j][k].dump() );
	}
    }
  
    public void Print() {
	this.Print(1);
    }

/**
 * Returns the starting state of the automaton.
 *
 * @author 	Stanislav Visnovsky
 * @version 	1.0.0
 * @return	the starting state
*/
    public int Start() { return start; }

/**
 * Returns the list of accepting states of automaton.
 *
 * @author 	Stanislav Visnovsky
 * @version 	1.0.0
 * @return	the list of states
*/
    public Hashtable Stop() { return stop; }

/**
 * Returns the list of update states of automaton.
 *
 * @author 	Stanislav Visnovsky
 * @version 	1.0.0
 * @return	the list of states
*/
    public Hashtable Update() { return update; }

/**
 * Reduces the automaton, that all equivalent states are represented as only one state.
 *
 * @author 	Stanislav Visnovsky
 * @version 	2.0.0
*/
    public void Reduce() {

        List Sets = new LinkedList();

	IntVector setof[] = new IntVector[ nextID ]; // references for the set of equivalent states

	IntVector s1 = new IntVector();
	IntVector s2 = new IntVector();

	for( int n = 0; n < nextID; n++ ) {
	        if( stop.containsKey( new Integer(n) ) ) { 
		    s1.addInt(n); 
		    setof[n] = s1;
		} else {
	            s2.addInt(n);
		    setof[n] = s2;
		}
        }

        Debug.println(1,"Reduce: Starting equivalence done");
        if( s1.used > 0 ) Sets.add(s1);
        if( s2.used > 0 ) Sets.add(s2);

	TidyUp();

	boolean in_n, in_n1, OKstate = true;
	int e,n,n1,nstate,n1state;
	Iterator i;
	List Sets2;
	IntVector s;
	
        boolean changed;
        do {
            changed = false;

            Sets2 = new LinkedList();
            i = Sets.iterator();
            while( i.hasNext() ) {			// for all sets
                s = (IntVector) i.next();
		for(  n = 0; n < s.used; n++ ) {
		    nstate = s.data[n];
                    if( !marked[ nstate ] ) {		// not done yet
                        s1 = new IntVector();		// it's own set
                        Sets2.add( s1 );
                        s1.addInt( nstate );
			setof[ nstate ] = s1;
                        marked[ nstate ]=true;
                        for( n1 = n+1; n1<s.used; n1++ ) {
			    n1state = s.data[n1];
                            if( !marked[ n1state ] ) {
				// first test, whether they are both in update
				if( update.containsKey( new Integer(nstate) ) != 
				    update.containsKey( new Integer(n1state) ) ) OKstate = false;
				else
                                for( e = 0 ; e < EdgeFactory.getActionTokensNum(); e++ ) { 
				        // let us require deterministic automaton :-)
				    in_n = states[nstate][e] != null ;
				    in_n1 = states[n1state][e] != null ;
				    if( in_n && in_n1 ) {
					OKstate =  
					   setof[ states[n1state][e].data[0]] == setof[ states[nstate][e].data[0] ];
				    } else OKstate = ( !in_n && !in_n1 );
				    if( !OKstate ) break;
                                }

		                if( OKstate ) {
                                    s1.addInt( n1state );	// add to n's set
				    setof[n1state]=s1;
                                    marked[n1state] = true;
                                } else changed = true;
                            }
                        }
                    }
                }
            }
            Sets = Sets2;			// changed, use new sets
	    TidyUp();
        } while( changed );

        Debug.println(1,"Reduce: Equivalence found");

            if( Debug.debug >2 ) {
                i = Sets.iterator();
                while( i.hasNext() ) {
                    IntVector j = ((IntVector)i.next());
                    Debug.print(2, "{ " );
                    for( n = 0; n< j.used; n++ ) {
                    Debug.print(2, j.data[n] );
                    if( n < j.used-1 ) Debug.print( 2,", " );
                    }
                    Debug.println(2,"}");
                }
                Debug.println(2,"---------------------------");
            }

	// create reduced machine based on Sets

        IntVector oldstates[][] = states;
        Hashtable oldstop = stop;
	Hashtable oldupdate = update;
	int oldstart = start;
        stop = new Hashtable();
	update = new Hashtable();
	
        states = new IntVector[Sets.size()][EdgeFactory.getActionTokensNum()];
	allocated = Sets.size();
	marked = new boolean[ allocated ];
	nextID = 0;
	if( maxstates < allocated ) maxstates = allocated;

        Hashtable map = new Hashtable();

        for( Iterator index = Sets.iterator() ; index.hasNext(); ) {
	    s = (IntVector) index.next() ;
            map.put( s, new Integer( nextID++ ) );
	    Debug.println(3, "Map: "+s+" -> "+(nextID-1) );
        }

        Debug.println(1,"New states created");

	int j,j1;
	int nextindex = 0;
        for( Iterator index = Sets.iterator() ; index.hasNext(); ) {
	    s = (IntVector) index.next();
	    int state = s.data[0];
	    Debug.println( 3, "As a example for "+nextindex+" taken "+state );
	    for( j1 = 0 ; j1 < EdgeFactory.getActionTokensNum() ; j1++ ) {
		if( oldstates[state][j1] != null && oldstates[state][j1].used>0 ) 
		{
		    if( states[nextindex][j1] == null ) states[nextindex][j1] = new IntVector();
		    for( j = 0; j < oldstates[state][j1].used; j++ ) {
			Debug.println( 3, "-- "+EdgeFactory.ActionTokens[j1].name+" --> "+setof[oldstates[state][j1].data[j]] );
			Integer a = (Integer) map.get( setof[oldstates[state][j1].data[j]] );
			if( a == null ) Debug.println( 3, "Null !!!");
			states[nextindex][j1].sortedAddIntIfNExists( a.intValue() );
		    }
	        }
	    }
	    if( oldstop.containsKey( new Integer( state ) ) ) stop.put( new Integer( nextindex ), new Integer(nextindex) );
	    if( oldupdate.containsKey( new Integer( state ) ) ) update.put( new Integer( nextindex ), new Integer(nextindex) );
	    nextindex++;
        }

	Debug.println(3, "Now setting new start state to "+setof[ oldstart ]);
	start = ((Integer) map.get( setof[ oldstart ] )).intValue();

	Print(2);
    }

/**
 * A Vector-like class for handling arrays of primitive types (int)
 *
 * @author	Vladimir Mencl
 * @version	1.0.0
 */

    public static class IntVectorComparator implements Comparator {
      public int compare(Object o1, Object o2) {
	IntVector v1,v2;

	v1=(IntVector)o1;
	v2=(IntVector)o2;

	if (v1.used<v2.used) return -1;
	else if (v1.used>v2.used) return 1;
	else {
	  for (int i=0;i<v1.used;i++) {
	    if (v1.data[i]<v2.data[i]) return -1;
	    else if (v1.data[i]>v2.data[i]) return 1;
	  };
	  return 0;
	}
      }
    };

/**                 
 * Transforms the automaton to a deterministic one.
 *
 * Optimized by Vladimir Mencl
 *
 * @author      Vladimir Mencl
 * @version     2.0.0
 */                  
    public void Determinize() {

      Debug.println(1,"Starting new determinize");
      this.Print(2);

      Vector represents, newedges, newtokens;
      IntVector newstop;
      IntVector newupdate;
      int newNstates;

      /* variables representing the new deterministic machine *
       *  Vector of int[] represents;
       *    * Idxs of original states represented by state Idx
       *
       *  Vector of IntVector newedges;
       *    * for each new state, Idxes of edge targets
       *
       *  Vector of IntVector newtokens;
       *    * for newedges (above), IDs of tokens
       */
      
      Vector origedges, origtokens;
      IntVector origstops;
      IntVector origupdate;
      int origNstates, origstart;


      /* variables representing the original non-deterministic machine *
       *
       *  Vector of IntVector edges;
       *    * for each new state, Idxes of edge targets
       *
       *  Vector of IntVector tokens;
       *    * for edges (above), IDs of tokens
       *
       *  idx2states and state2idx map idxs and original states the same way
       *  as tokenname maps (below)
       */


      /* for mapping the concept of ActionToken objects to int[]
       * tokennames is indexed by tokenID and contains the ActionToken object
       * tokenIDs is indexed by token names (getHashCode()) 
       *     and contains Integer(tokenIdx)
       */

      /* Phase 1: represent original graph in new structures */

      
      origedges = new Vector(nextID);
      origtokens = new Vector(nextID);
      origstops = new IntVector();
      origupdate = new IntVector();


      {
	for (int tstate=0;tstate<nextID;tstate++) {
	  origedges.add(new IntVector());
	  origtokens.add(new IntVector());
	  if (stop.containsKey( new Integer(tstate))) {
		origstops.addInt(tstate);
		Debug.println(3, "Adding original stop "+tstate);
	  }
	  if (update.containsKey( new Integer(tstate))) {
		origupdate.addInt(tstate);
		Debug.println(3, "Adding original update "+tstate);
	  }
	};
      };

      Debug.println(2, "Original stops: "+origstops.used );
      Debug.println(2, "Original update: "+origupdate.used );

      origNstates = nextID;
      origstart = start;

      Debug.println(2, "New determinize: states created, now edges");

      /* token translations - only create vectors, data will be entered 
       * on-the-fly */

      {
	int si,t;
	int tedge;

	for (si=0; si<origNstates; si++) {
	  for( t = 0 ; t < EdgeFactory.getActionTokensNum(); t ++ ) 
	     if( states[si][t] != null ) 
		for( tedge = 0 ; tedge < states[si][t].used; tedge ++ ) {

	        ((IntVector)origedges.elementAt(si)).addInt( states[si][t].data[tedge]);
	        ((IntVector)origtokens.elementAt(si)).addInt(t);
	  };
	};
      }; /* scope block */
      Debug.println(1,"Phase 1: "+nextID+" states");

      if( Debug.debug >1 ) {
	  Debug.println( 2, "Machine representation:");
	  for( int si = 0; si < origNstates; si ++ )
	  for( int t = 0 ; t < EdgeFactory.getActionTokensNum(); t ++ ) 
	     if( states[si][t] != null ) 
		Debug.println( 2, "State "+si+": states "+ ((IntVector)origedges.elementAt(si)).dump()+" -> tokens "+((IntVector)origtokens.elementAt(si)).dump() );
      }

      /* Phase 2: determinize the machine - create new machine in new* */

	   /* Note: after creating a state (the "represents" element),
	    * the array of state ids has to be sorted to allow for fast
	    * _n_-time comparision
	    */
      /* create an empty state as the initial state of the new machine,
       * put this state into the buffer */


      { LinkedList buffer = new LinkedList();
	IntVector cedges, ctokens; /* current - being created */
	IntVector oedges, otokens; /* old - edges and corresponding tokens
				    * of an original state */

	TreeMap altrep;

	int ostates [];

	newNstates = 0;
	int npasses = 0;
	int debugDeterminize = Integer.parseInt(System.getProperty(
	    "debugDeterminize","0"));
	newedges = new Vector();
	newtokens = new Vector();
	represents = new Vector();
	altrep = new TreeMap(new IntVectorComparator());

	{ int a[];
	  Integer b;
	  IntVector iv;

	  newedges.add(new IntVector());
	  newtokens.add(new IntVector());
	  represents.add(a=new int [] { origstart } );
	  altrep.put( iv=new IntVector(a,false),b=new Integer(0));
	  if (debugDeterminize>=2) Debug.println(1,"Adding state "+iv.dump());
	  buffer.add( b );
	};

	Debug.println(1,"#tokens="+EdgeFactory.getActionTokensNum()+", origNstates="+
	    origNstates+", sizeof matrix="+EdgeFactory.getActionTokensNum()*origNstates);

	int sidx1,osidx,ti;

	while (! buffer.isEmpty()) {

	  sidx1 = ((Integer)buffer.getFirst()).intValue(); 
	  buffer.removeFirst();
          if (npasses % 100 == 0)
             Debug.println(1,"Already created "+represents.size()+" new states, [buffer]="+buffer.size());
	  npasses++;

	  cedges = (IntVector)newedges.elementAt(sidx1);
	  ctokens = (IntVector)newtokens.elementAt(sidx1);
	  ostates = (int [])represents.elementAt(sidx1);
	  if (debugDeterminize>=2) Debug.println(1,"Processing state "+new IntVector(ostates,false).dump());

	  /* [tokens] sorted { vectorids }
	    Vector [tokenId] of int[] - sized by origNstates

	    alternative represents:
	      sorted { sorted vectorids }
          */

	  IntVector statematrix[] = new IntVector[ EdgeFactory.getActionTokensNum()];
	  for (int i=0; i<EdgeFactory.getActionTokensNum(); i++) 
	    statematrix[i]=new IntVector();

	  /* iterate over states in represents and outgoing edges */
	  /* create Vector [tokenIDs] of IntVector [stateIDs]
	   */
	  for (int i=0; i<ostates.length; i++) {
	    osidx=ostates[i];
	    oedges = (IntVector)origedges.elementAt(osidx);
	    otokens = (IntVector)origtokens.elementAt(osidx);
	    for (int j=0; j<oedges.used; j++) {
	      ti=otokens.data[j];
	      statematrix[ti].sortedAddIntIfNExists(oedges.data[j]);
	    };
	  };

	  /* now, statematrix has for each tokens and new state 
	   * the count of transitions to that state
	   */

	  /* iterate over transitions, for each check, whether a target 
	   * state already exists in the new state set, create it if not
	   */
	  for (int i=0; i<EdgeFactory.getActionTokensNum(); i++) if (statematrix[i].used>0) {
	    IntVector trep = new IntVector(statematrix[i]); 
			       /* target represents */
	    int tri;  /* Targer Represents Id */
	    Integer trio;

	    if ((trio=(Integer)altrep.get(trep))!=null) {
	      tri=trio.intValue();
	    } else {
	      /* add state */
	      tri=represents.size(); trio=new Integer(tri);
	      if (debugDeterminize>=2) Debug.println(1,"Adding state "+trep.dump());
	      altrep.put(trep,trio);
	      represents.add(trep.data);
	      newedges.add(new IntVector());
	      newtokens.add(new IntVector());
	      buffer.add(trio);
	    };
	    cedges.addInt(tri);
	    ctokens.addInt(i);
	  };

	};
      };
      Debug.println(1,"Done phase 2: "+represents.size()+" states");

      /* Phase 2s: create newstop data */

      { 
	int oldstopmatrix[] = new int[origNstates];
	int r[];
	int i,j;


	/* create an "hash-array" - indexed by statenumber, >0 when state is a stop-state */
	for ( i=0;i<origstops.used;i++) oldstopmatrix[origstops.data[i]]++;

	newstop=new IntVector();
	for ( i=0;i<represents.size();i++) {
	  r=(int[])represents.elementAt(i);
	  for ( j=0;j<r.length;j++)
	    if (oldstopmatrix[r[j]]>0) { 
                  Debug.println(3, "Added new stop "+i); 
                  newstop.addInt(i); break; 
            };
	};
      };
      Debug.println(1,"Done phase 2s!");

      /* Phase 2u: create newupdate data */

      { 
	int oldupdatematrix[] = new int[origNstates];
	int r[];
	int i,j;


	/* create an "hash-array" - indexed by statenumber, >0 when state is a update-state */
	for ( i=0;i<origupdate.used;i++) oldupdatematrix[origupdate.data[i]]++;

	newupdate=new IntVector();
	for ( i=0;i<represents.size();i++) {
	  r=(int[])represents.elementAt(i);
	  for ( j=0;j<r.length;j++)
	    if (oldupdatematrix[r[j]]>0) { 
                  Debug.println(3, "Added new update "+i); 
                  newupdate.addInt(i); break; 
            };
	};
      };
      Debug.println(1,"Done phase 2u!");

      /* Phase 3: create a traditional representation of new data */

      {
	states = new IntVector[represents.size()][ EdgeFactory.getActionTokensNum()];
	allocated = nextID = represents.size();
	marked = new boolean[allocated];
	if( maxstates < allocated ) maxstates = allocated;

	IntVector cedges,ctokens,v;
	int i,j;

	for (i=0;i < nextID ;i++) {
	  cedges = (IntVector)newedges.elementAt(i);
	  ctokens = (IntVector)newtokens.elementAt(i);
	  for (j=0;j<cedges.used;j++) {
	      v = states[i][ctokens.data[j]];
	      if( v == null ) v = states[i][ctokens.data[j]] = new IntVector();
		Debug.println(3, "Adding edge "+i+" --"+ctokens.data[j]+"-->"+cedges.data[j]);
	      v.sortedAddIntIfNExists(cedges.data[j]);
	  }
	};

	start = 0;
	stop = new Hashtable();
	update = new 
Hashtable();

	for (i=0;i<newstop.used;i++) {
	  Integer num = new Integer( newstop.data[i] );
	  stop.put(num, num);
	};

	for (i=0;i<newupdate.used;i++) {
	  Integer num = new Integer( newupdate.data[i] );
	  update.put(num, num);
	};

	Print(3);
      };
    };


/**
 * Find trace from starting state to the given state. Used for counterexample lookup.
 * Expects that endState is reachable.
 *
 * @author 		Stanislav Visnovsky
 * @version 		2.0.0
 * @param endState	end state of the trace
 * @return		trace found
*/
    private String findTraceTo( int fromState, int edgetransition, int edgetarget ) {

	Iterator j;

	Debug.println(2, "Trying to find a trace to state "+fromState );

	// cleanup
	TidyUp();

	// using BSF algorithm with recording shortest path by edges to use
	LinkedList paths = new LinkedList();
	LinkedList buffer = new LinkedList();
	buffer.addLast( new Integer(start) );
	paths.addLast( new LinkedList() ); // to start state is no path
	marked[start] = true;
	while( buffer.size() > 0 ) {
	    int current = ((Integer)buffer.removeFirst()).intValue();
	    LinkedList pathToCurrent = (LinkedList)paths.removeFirst();

	    Debug.println(3, "FindTrace: at state:"+current );
	    if( current == fromState ) {
		Debug.println( 3, "Current ("+current+") is endstate");
	    
		String trace = "";
		
		while( !pathToCurrent.isEmpty() ) {
		    int t = ((Integer)pathToCurrent.removeFirst()).intValue();
		    trace = trace + EdgeFactory.ActionTokens[ t ].name + ",";
		    Debug.println( 3, trace );
		}
		
		trace = trace + EdgeFactory.ActionTokens[edgetransition].name;

		// cleanup
		TidyUp();

		return "<"+trace+">";
	    } else {
	    Debug.println(3, "Filling buffer");
	    for( int t = 0; t < EdgeFactory.getActionTokensNum(); t++ ) 
	    if( states[current][t] != null ) {
		for( int f = 0; f < states[current][t].used ; f++ )
		    if( ! marked[ states[current][t].data[f] ] ) {
		        Debug.println(3, "Adding state "+states[current][t].data[f]+" to buffer");
		        buffer.addLast( new Integer( states[current][t].data[f] ) );
		        LinkedList newpath = (LinkedList)pathToCurrent.clone();
		        newpath.addLast( new Integer( t ) );
		        paths.addLast( newpath );
		        marked[ states[current][t].data[f] ] = true;
		    }
	    }}
	}

	// cleanup
	TidyUp();

	return "Unable to find a counterexample";
    }

/**
 * Query the automaton, whether its language is included in other automaton
 * given as the argument.
 *
 * @author 	Stanislav Visnovsky
 * @version 	1.0.0
 * @param m	the automaton to be compared
 * @return	null, if is included, counterexample otherwise
*/
    public String isIncludedIn( Machine m1 ) {

	machineImpl m = (machineImpl) m1;

	Debug.println( 1, "This is inclusion lookup for these automata");
	if( Debug.debug >1 ) {
	    ((Printer)m).Print(2);
	    Debug.println(2, "and");
	    Print(2);
	}

	int max = m.nextID > nextID ? m.nextID : nextID;
	int iso[][] = new int[ max ] [2];

	Debug.println(2, "Maximal number of states: "+max );

	// if this.start is stop and other start is not, wrong
	if( (stop.containsKey( new Integer(start) ) && !m.Stop().containsKey( new Integer(m.Start() ) ) ) )
	{
             Debug.println( 1, "Not both of start states are/are not stop states, not included" );
             return "NULL";
	}
	
	if( (update.containsKey( new Integer(start) ) && !m.Update().containsKey( new Integer(m.Start() ) ) ) )
	{
             Debug.println( 1, "Not both of start states are/are not update states, not included" );
             return "NULL";
	}
	
	iso[0][0] = start;
	iso[0][1] = m.Start();
        Debug.print( 2, "Associating state ");
	Debug.print( 2, start );
	Debug.print( 2, " of machine 1 and state ");
	Debug.print( 2, m.Start() );
	Debug.println( 2, " of machine 2 at index 0");

	int index,index2,i,nextindex = 1;

	List buffer = new LinkedList();
	buffer.add( new Integer(0) );
	IntVector v;

	while( ! buffer.isEmpty() ) {
	    index = ((Integer)buffer.get(0)).intValue();
	    buffer.remove(0);
	    Debug.println(2,"Started new round with index "+index);
	    Debug.println(2,"Starting edge walk");
	    for( i = 0 ; i < EdgeFactory.getActionTokensNum(); i++ ) 
		if( states[iso[index][0]][i] != null ) { // is there an edge for this transition from state index?
		v = m.states[iso[index][1]][i];
		if( v == null ) {
		   Debug.println(2,"corresponding edge not found");
		   Debug.println( 1, "edges are not equivalent, not included");
		   return findTraceTo( iso[index][0], i, states[iso[index][0]][i].data[0]  );
		}
		// now we need determinism, i.e., in v should be only one item
		for( index2 = 0; (index2 < nextindex ) && (iso[index2][1]!=v.data[0] ) ; index2++ );
		Debug.println( 2, "State lookup finished at:"+index2 );

                if( (stop.containsKey(new Integer( states[iso[index][0]][i].data[0] ) ) 
		  && !m.Stop().containsKey( new Integer( v.data[0] ) ) ) )
        	{
                    Debug.println(1, "Not both of states "+states[iso[index][0]][i].data[0]+" and " +v.data[0]+ " are/are not stop states, not included" );
		    return findTraceTo( iso[index][0], i, states[iso[index][0]][i].data[0] );
            	}
                if( (update.containsKey(new Integer( states[iso[index][0]][i].data[0] ) ) 
		  && !m.Update().containsKey( new Integer( v.data[0] ) ) ) )
        	{
                    Debug.println(1, "Not both of states "+states[iso[index][0]][i].data[0]+" and " +v.data[0]+ " are/are not update states, not included" );
		    return findTraceTo( iso[index][0], i, states[iso[index][0]][i].data[0] );
            	}
		if( index2 == nextindex ) {
		    iso[ nextindex ][0] = states[iso[index][0]][i].data[0];
		    iso[ nextindex ][1] = m.states[ iso[index][1]][i].data[0];
		    Debug.print( 2,"Associating state ");
		    Debug.print( 2, states[iso[index][0]][i].data[0] );
		    Debug.print( 2," of machine 1 and state ");
		    Debug.print( 2,m.states[ iso[index][1]][i].data[0]);
		    Debug.println( 2," of machine 2 at index "+nextindex);
		    buffer.add( new Integer(nextindex++) );
		}
	     }	
	}
	Debug.println(1,"Included");
	return null;
    }



/**
 * Restrict the automaton by the set passed as the argument.
 *
 * @author 	Stanislav Visnovsky
 * @version 	2.0.0
 * @param restrict	the set of action tokens to be eliminated
*/
    public void Restrict( ActionTokenArray restrict ) {

    int ed[];

    Debug.println(1,"Starting event restriction");
    Print(2);
    Debug.println( 2,restrict.toString() );

	// tidy up
	TidyUp();

    IntVector newstates[][] = new IntVector[ allocated ][EdgeFactory.getActionTokensNum()]; // table for new states

    // find all edges to be eliminated

    int actual,e,target;

    Debug.println(2,"Edges to be eliminated:");
    Hashtable edgesToBeEliminated = new Hashtable();
    for( actual = 0; actual < nextID; actual ++ ) {
        Debug.println(2,"State: "+actual );

        // identify the edges to be eliminated
	for( e = 0; e < EdgeFactory.getActionTokensNum(); e++ )
	if( states[actual][e] !=  null ) {
            if( !restrict.tokens[e] ) 
		for( target = 0; target < states[actual][e].used ; target++) {
		    ed = new int[3];
		    ed[0] = actual;
		    ed[1] = e;
		    ed[2] = states[actual][e].data[target];
                    edgesToBeEliminated.put(ed,ed);
                    Debug.print(2,"Added to be eliminated: ");
		    Debug.print(2, actual+"--"+EdgeFactory.ActionTokens[e].name+"--> "+states[actual][e].data[target]);
                }
	}
    }

    Debug.println(1,"List is completed");

    if( Debug.debug >1 ) {    
        Iterator edgs = edgesToBeEliminated.values().iterator();
        while( edgs.hasNext() ) { 
	  ed = (int[])edgs.next(); 
	  Debug.println(2, ed[0]+"--"+EdgeFactory.ActionTokens[ed[1]].name+"--> "+ed[2]); }
        Debug.println(2,"That's all");
    }

    // propagate stop states

    Hashtable stops = new Hashtable();
    Iterator nowstops = stop.values().iterator();
    while( nowstops.hasNext() ) {
	Integer s = (Integer)nowstops.next();
	stops.put( s, s);
    }
    
    int oldsize = -1;
    while( oldsize != stops.size() ) {
        
	Debug.println( 2,"Stop state propagation cycle" );
        oldsize = stops.size();
	
        Iterator ie = edgesToBeEliminated.values().iterator();
        while( ie.hasNext() ) {

	    ed = (int [])ie.next();

	    if( stops.containsKey( new Integer( ed[2] ) ) ) {
		Integer n = new Integer( ed[0] );
    		Debug.print(2,"Adding stop state:"+ed[0]+" due " );
		Debug.println(2, ed[0]+"--"+EdgeFactory.ActionTokens[ed[1]].name+"--> "+ed[2]);
		stops.put( n, n );
	    } Debug.println( 3,"State "+ed[2]+" is not stop" );

	}
    }

    Debug.println(1,"Stop state propagation done");

    stop = stops;  // use new stop states
    
    // find new sets of edges

    for( actual = 0 ; actual<nextID ; actual++ ) {
	Debug.println(2, "Lookup edges to be added to "+actual );
	eliminationClosure( actual, actual, restrict, newstates );
	TidyUp();
    }

    states = newstates;
    
    // eliminate updates if necessary
    if( !restrict.tokens[1] ) update = new Hashtable();

	// tidy up
	TidyUp();

        Reachable();

	Debug.println(1,"Finishing event restriction");
	Print(2);
    }

    private void eliminationClosure( int s, int addToState, ActionTokenArray edgesToSurvive, IntVector newstates[][] ) {
	int t,j;

        marked[s] = true;
	for( t = 0; t < EdgeFactory.getActionTokensNum(); t++ )
	if( states[s][t] != null )
	    for( j = 0; j < states[s][t].used; j++ ) {
            if( !edgesToSurvive.tokens[t]  && !marked[ states[s][t].data[j] ] ) 
                eliminationClosure( states[s][t].data[j], addToState, edgesToSurvive, newstates );
            else 
		if( edgesToSurvive.tokens[t] ) {
  		    Debug.print(2, "To state "+addToState+" will be added edge ");
		    Debug.print(2, "--"+EdgeFactory.ActionTokens[t].name+"--> "+states[s][t].data[j]);
		    if( newstates[addToState][t] == null ) newstates[addToState][t] = new IntVector();
		    newstates[addToState][t].sortedAddIntIfNExists( states[s][t].data[j] );
		}

        }
    }

    public void ClearStops() {
	stop.clear();
    }

    public void addEdge( int from, int transition, int to ) {
	if( states[from][transition] == null ) states[from][transition] = new IntVector();
	states[from][transition].sortedAddIntIfNExists( to );
    }

    public void addUpdateEdge( int from ) {
        Integer nn = new Integer(from);
	update.put( nn, nn );
    }

    public void TidyUp() {
        for( int tidy = 0; tidy < nextID; tidy++ )
	    marked[tidy] = false;

    }

    private void Resize( int newsize ) {
	int i,j;
	Debug.println(2, "Trying resize to "+newsize+" states");
	IntVector a[][] = new IntVector[ newsize ][ EdgeFactory.getActionTokensNum() ];
	boolean newmarked[] = new boolean[ newsize ];
	for( i=0; i<allocated; i++ )
	    for( j = 0; j < EdgeFactory.getActionTokensNum(); j++ ) {
		a[i][j] = states[i][j];
		newmarked[i] = marked[i];
	    }
	states = a;
	marked = newmarked;
	allocated = newsize;
	if( maxstates < allocated ) maxstates = allocated;
    }

    public void checkSpace( int required ) {
	if( required > allocated ) Resize( required );
    }

    public void Merge( int to, int from ) {
	int j,k;
	for( j = 0; j< EdgeFactory.getActionTokensNum(); j++) 
	if( states[from][j] != null ) {
	    if( states[to][j] == null ) states[to][j] = new IntVector();
	    for( k = 0; k < states[from][j].used; k++ ) 
  	  	states[to][j].sortedAddIntIfNExists( states[from][j].data[k] );
	}
    }

    public static int maxstates = 0;

    public void Minimize() {
	Determinize();
	Reduce();
    }
    
    public void NonUpdateOnly() {
	// simply clear the list of states, from which the update event exists
	update = new Hashtable();
    }
    
    public void UpdateOnly() {
	// simply clear the list of stop states. Since update states are stop states.
	stop = new Hashtable();
    }
    
    public void StrippedUpdateOnly() {
	// update list becomes stop list and update list is cleared (stripping [UPDATE])
	stop = update;
	update = new Hashtable();
    }
}
