/*
 * XML.java
 *
 * Created on 23. listopad 2003, 16:30
 */

package SOFA.Util;

import java.io.*;
import org.w3c.dom.*;
import org.xml.sax.*;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;

/**
 *
 * @author  Ladislav Sobr
 */
public class XML
{
  
  /** Creates a new instance of XML */
  public XML()
  {
  }
  
  public static Document parse(File file) throws SAXException, IOException
  {
    Document doc = null;
    Element element;
    try
    {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      DocumentBuilder builder = factory.newDocumentBuilder();
      doc = builder.parse(file);
      element = doc.getDocumentElement();
      element.normalize();
    }
    catch (ParserConfigurationException e)
    {
      System.out.println("ParserConfigurationException: " + e.getMessage());
      System.exit(1);
    }
    return doc;
  }
  
  public static Document parse(InputStream stream) throws SAXException, IOException
  {
    Document doc = null;
    Element element;
    try
    {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      DocumentBuilder builder = factory.newDocumentBuilder();
      doc = builder.parse(stream);
      element = doc.getDocumentElement();
      element.normalize();
    }
    catch (ParserConfigurationException e)
    {
      System.out.println("ParserConfigurationException: " + e.getMessage());
      System.exit(1);
    }
    return doc;
  }
  
  public static void dump(File file, Element element) throws IOException
  {
    FileOutputStream out = new FileOutputStream(file);
    dump(out, element);
    out.close();
  }
  
  public static void dump(OutputStream stream, Element element) throws IOException
  {
    try
    {
      TransformerFactory tFactory = TransformerFactory.newInstance();
      Transformer transformer = tFactory.newTransformer();
      transformer.setOutputProperty("indent", "yes");
      transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "2");
      DOMSource source = new DOMSource(element);
      StreamResult result = new StreamResult(stream);
      transformer.transform(source, result);
    } catch  (TransformerConfigurationException e) {
      System.out.println("TransformerConfigurationException: " + e.getMessage() );
      System.exit(1);
    } catch (TransformerException e) {
      System.out.println("TransformerException: " + e.getMessage() );
      System.exit(1);
    }
  }
  
  public static Document newDocument()
  {
    Document doc = null;
    try
    {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      DocumentBuilder builder = factory.newDocumentBuilder();
      doc = builder.newDocument();
    }
    catch (ParserConfigurationException e)
    {
      System.out.println("ParserConfigurationException: " + e.getMessage());
      System.exit(1);
    }
    return doc;
  }

  public static void newTextElement(Element element, String tagName, String content)
  {
    Document doc = element.getOwnerDocument();
    Element el = doc.createElement(tagName);
    Text tx = doc.createTextNode(content);
    el.appendChild(tx);
    element.appendChild(el);
  }
  
  public static void newOneAttributeElement(Element element, String tagName, String attributeName, String attributeValue)
  {
    Document doc = element.getOwnerDocument();
    Element el = doc.createElement(tagName);
    el.setAttribute(attributeName, attributeValue);
    element.appendChild(el);
  }
  
  public static String getTextContent(Element element)
  {
    NodeList nl = element.getChildNodes();
    for (int i = 0; i < nl.getLength(); i++)
    {
      Node node = nl.item(i);
      if (node.getNodeType() == Node.TEXT_NODE) return ((Text)node).getData();
    }
    return "";
  }    
 
}
